/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2007 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: bottle.c
***                                                            
*** Comments: 
*** 
***    demo program for German bottle recycling (DPG) 
***
*** Revision History:
***    Date          Version    Author  Changes
***    Mar 22, 2007  3.02       ME      initial version
***
***************************************************************************
*END***********************************************************************/

#define NEW_IMAGE_VAR
#include <vcrt.h>
#include <vclib.h>
#include <flib.h>
#include <macros.h>
#include <stdlib.h>
#include <register.h>
#include <sysvar.h>
#include <math.h>

#define PRINTOUT

#define LOGOSIZEX	100
#define LOGOSIZEY	100
#define RLCSIZE     0x080000		/* 0.5 MBytes for RLC					*/
#define CONTRAST	10

#define MINAREA		100
#define MAXAREA     400
#define MXOBJ		10000
#define MNOBJ       1000

//#define MNTPHI1	(5.0E3)
//#define MXTPHI1   (8.0E3)
//#define MNTPHI2   (0.5E7)
//#define MXTPHI2   (3.0E7)
//#define MNTECC	(0.20)
//#define MXTECC    (0.28)


#define MNTPHI1		(2.0E3)
#define MXTPHI1     (3.0E4)
#define MNTPHI2     (0.2E7)
#define MXTPHI2     (2.0E8)
#define MNTECC		(0.10)
#define MXTECC      (0.40)

#define MNRLD		1
#define MXRLD		8



typedef struct
  {
  U32 area;            	/* object area                  */
  U32 x_center;        	/* x_center - U32       		*/
  U32 y_center;        	/* y_center - U32      			*/
  float xcf;			/* x_center	- float				*/
  float ycf;			/* y_center	- float				*/
  } pftr;

#define Round(x)	((I32)floorf(x+0.5))
#define PI			(3.141592654)
#define RADTODEG    (180/PI)

void MarkCross(image *Hough8, I32 ix, I32 iy, I32 Color, I32 Size);
I32 check_distance(I32 count, pftr *objlist, I32 *ndx1, I32 *ndx2);


void main(void)
{
  image src, dst, logo, logo1;
  I32 ms, sec, i, n;
  U16 *rlc, *slc;
  I32 nobj;
  ftr *features;
  pftr *ftr2;
  moment *moments, *mx;
  float phi1, phi2, ecc;
  float vxf, vyf, angle;
  char c=0;
  I32 error;
  I32 ndx1, ndx2;					/* indexes for trinagle	pair	*/
  I32 pitch = getvar(VPITCH);
  I32 xstart, ystart;				/* start of logo frame in image	*/
  float xcenter, ycenter;			/* center between triangles		*/
  float q[2][2];					/* affine transformation matrix	*/
  float t[2];						/* affine displacement vector	*/
  I32 *select;						/* select list object deletion	*/

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
	/**/ 	print("Init Extension Lib = %d\n", init_licence("E1122334455"));     /**/
    /*******************************************************************************/

  /////////////////////////////////////////////////////////////////////////////////////
  // set some image variables and allocate memory for destination image, RLC, features 
  /////////////////////////////////////////////////////////////////////////////////////

  ImageAssign(&src, getvar(DISP_START), getvar(HWIDTH), getvar(VWIDTH), pitch);
  ImageAllocate(&dst,       IMAGE_GREY, getvar(HWIDTH), getvar(VWIDTH));
  ImageAllocate(&logo,      IMAGE_GREY, LOGOSIZEX, LOGOSIZEY);

  rlc      = rlcmalloc(RLCSIZE);					/* main RLC buffer				*/
  select   = byte_malloc(MXOBJ * sizeof(I32));		/* select list object deletion	*/
  features = byte_malloc(MXOBJ * sizeof(ftr));		/* basic features				*/
  moments  = byte_malloc(MNOBJ * sizeof(moment));	/* order 2 moments				*/
  ftr2     = byte_malloc(MNOBJ * sizeof(pftr));		/* special feature list			*/

  /////////////////////////////////////////////////////////////////////////////////////
  // start while
  /////////////////////////////////////////////////////////////////////////////////////


while(c != 0x1b)
  {
  if(rbempty() != -1) c=rs232rcv();             /* read character       */


  /////////////////////////////////////////////////////////////////////////////////////
  // take a picture 
  /////////////////////////////////////////////////////////////////////////////////////

//  print("reading image\n");
//  fread_image("fd:/user/test10.002", &src);
//  rs232rcv();
  
  tpict();

  /////////////////////////////////////////////////////////////////////////////////////
  // perform unsharp masking
  /////////////////////////////////////////////////////////////////////////////////////


  ms  = getvar(MSEC);
  sec = getvar(SEC);

  maskx(&src, &dst, 32, 32, 128);           /* unsharp masking  */

#ifdef PRINTOUT
  copy(&dst, &src);							/* just to see		*/
#endif

#ifdef PRINTOUT
  rs232rcv();
#endif

  /////////////////////////////////////////////////////////////////////////////////////
  // now run length encoding and object labelling
  /////////////////////////////////////////////////////////////////////////////////////

  if((slc = rlcmk(&dst, 128-CONTRAST, rlc, RLCSIZE)) == NULL)
    {
    pstr("RLC overrun\n");
    return;
    }

#ifdef PRINTOUT
  rlcout(&src, rlc, 0, 255);					/* test output			*/
#endif
  
  if(sgmt(rlc, slc) == NULL)                   /* object labelling     	*/
    {
    pstr("object number overrun\n");
    return;
    }

  /////////////////////////////////////////////////////////////////////////////////////
  // area filtering
  /////////////////////////////////////////////////////////////////////////////////////

#ifdef PRINTOUT
  rs232rcv();
#endif

  nobj=rlc_arf(rlc, rlc, MINAREA/2);

#ifdef PRINTOUT
  rlcout(&src, rlc, 0, 180);					/* test output			*/
#endif

  /////////////////////////////////////////////////////////////////////////////////////
  // second object labelling
  /////////////////////////////////////////////////////////////////////////////////////

  if(sgmt(rlc, slc) == NULL)                   /* object labelling     	*/
    {
    pstr("object number overrun\n");
    return;
    }


  /////////////////////////////////////////////////////////////////////////////////////
  // feature extraction
  /////////////////////////////////////////////////////////////////////////////////////

  nobj = rl_ftr2(rlc, features, MXOBJ);         /* feature extraction   */

#ifdef PRINTOUT
  print("number of objects found: %d\n", nobj);
  rs232rcv();
#endif

  /////////////////////////////////////////////////////////////////////////////////////
  // select only black objects with area within reasonable range
  /////////////////////////////////////////////////////////////////////////////////////

  for(i=0; i<nobj; i++)
    {
	if(((features[i].area > MINAREA) && (features[i].area < MAXAREA)) && !(features[i].color & 1))
	  {
#ifdef PRINTOUT
      MarkCross(&src, (I32)features[i].x_center, (I32)features[i].y_center, 255, 10);
#endif
	  select[i] = 1;
	  }
	else
	  {
	  if(features[i].color & 1)
	    select[i] = 1;									 
	  else
	    select[i] = 0;									 
	  }
    }


  /////////////////////////////////////////////////////////////////////////////////////
  // delete all other objects from RLC
  /////////////////////////////////////////////////////////////////////////////////////

#ifdef PRINTOUT
  rs232rcv();
#endif

  rlc_delete(rlc, rlc, select);

#ifdef PRINTOUT
  rlcout(&src, rlc, 0, 255);					/* test output			*/
#endif
  /////////////////////////////////////////////////////////////////////////////////////
  // third object labelling
  /////////////////////////////////////////////////////////////////////////////////////

  if(sgmt(rlc, slc) == NULL)                   /* object labelling     	*/
    {
    pstr("object number overrun\n");
    return;
    }

  /////////////////////////////////////////////////////////////////////////////////////
  // feature and moment calculation	/ only very few objects left
  /////////////////////////////////////////////////////////////////////////////////////

  nobj = rl_ftr2(rlc, features, MNOBJ);           /* feature extraction   	*/
  nobj = rlc_moments(rlc, moments, MNOBJ);		  /* moment calculation		*/

  mx = moments;	n = 0;

  for(i=0; i<nobj; i++)
    {
	phi1 = mom_calc_phi1(mx);
	phi2 = mom_calc_phi2(mx);
	ecc  = mom_calc_ecc(mx);

#ifdef PRINTOUT
	print("phi1: %e phi2: %e\n", phi1, phi2); 
    MarkCross(&src, (I32)mom_calc_cgx(mx), (I32)mom_calc_cgy(mx), 255, 10);
	rs232rcv();
#endif

	if((phi1 > MNTPHI1)	&& (phi1 < MXTPHI1)	&& (phi2 > MNTPHI2)	&& (phi2 < MXTPHI2) && (ecc > MNTECC) && (ecc < MXTECC))
	  {
#ifdef PRINTOUT
	  print("phi1: %e phi2: %e\n", phi1, phi2); 
	  print("eccentricity: %e angle: %e\n", ecc, mom_calc_angle(mx));
      MarkCross(&src, (I32)mom_calc_cgx(mx), (I32)mom_calc_cgy(mx), 255, 10);
#endif

	  ftr2[n].area 		=  features[i].area;
	  ftr2[n].x_center 	=  features[i].x_center;
	  ftr2[n].y_center 	=  features[i].y_center;
	  ftr2[n].xcf 		=  mom_calc_cgx(mx);
	  ftr2[n].ycf 		=  mom_calc_cgy(mx);

	  n++;
	  }

	mx++;
    }

#ifdef PRINTOUT
  print("number of triangles found: %d\n", n);

  for(i=0; i<n; i++)
    {
	print("# %d area: %d\n", i, ftr2[i].area);
	} 
#endif

  error = check_distance(n, ftr2, &ndx1, &ndx2);


  if(error < 0)
    {
    print("logo not found\n");
	goto endloop;
	}

  MarkCross(&src, ftr2[ndx1].x_center, ftr2[ndx1].y_center, 255, 10);
  MarkCross(&src, ftr2[ndx2].x_center, ftr2[ndx2].y_center, 255, 10);

  vxf = ftr2[ndx1].xcf - ftr2[ndx2].xcf;
  vyf = ftr2[ndx1].ycf - ftr2[ndx2].ycf;
  xcenter = (ftr2[ndx1].xcf + ftr2[ndx2].xcf)/2.0; 
  ycenter = (ftr2[ndx1].ycf + ftr2[ndx2].ycf)/2.0; 

  angle = atanf(vyf/vxf) * RADTODEG;

#ifdef PRINTOUT
  print("difference vector: %e %e xcenter: %e ycenter: %e angle: %e\n", vxf, vyf, xcenter, ycenter, angle);
#endif

  /////////////////////////////////////////////////////////////////////////////////////
  // enlarge, normalize and copy logo
  /////////////////////////////////////////////////////////////////////////////////////

  xstart = Round(xcenter) - 50;
  xstart = (xstart < 0) ? 0 : xstart;
  ystart = Round(ycenter) - 50;
  ystart = (ystart < 0) ? 0 : ystart;

  ImageAssign(&logo1, dst.st + ystart * pitch + xstart, 100, 100, pitch);

  framed(&logo1, 0);

  calc_rotation_matrix(-angle, xcenter, ycenter, q, t);
  affine_image_transform(&logo1, &logo, q, t, 0);

  copy(&logo, &src);

  /////////////////////////////////////////////////////////////////////////////////////
  // end while
  /////////////////////////////////////////////////////////////////////////////////////

  ms  = getvar(MSEC)-ms; if(ms<0) ms+=1000;
  sec = getvar(SEC)-sec;
  ms += 1000 * sec;

  print("processing time[%3dx%3d] = %4dms\n", src.dx, src.dy, ms);


  endloop:
#ifdef PRINTOUT
  rs232rcv();
#endif

  time_delay(500);
  }


  /////////////////////////////////////////////////////////////////////////////////////
  // deallocate memory
  /////////////////////////////////////////////////////////////////////////////////////

  byte_free(ftr2);
  byte_free(moments);
  byte_free(features);	  
  byte_free(select);	  
  rlcfree(rlc);
  ImageFree(&logo);
  ImageFree(&dst);

}


/*FUNCTION*-------------------------------------------------------------
*
* Function Name   : check_distance
* Input Values	  : 
*	count  	      : number of objects in objlist
*	objlist	  	  :	feature list
*	ndx1, ndx2 	  :	indexes of objects with matching relative distance
*                   (return values)
* Returned Values : 0: objects found, -1: not found
* Comments        :	searches object list for objects with matching
*                   relative distance. Relative distance is
*                   squared distance divided by sum of object areas
*
*END*-----------------------------------------------------------------*/

I32 check_distance(I32 count, pftr *objlist, I32 *ndx1, I32 *ndx2)
{
I32 i, k;
I32 area1, area2, area_diff, area_sum;
I32 x1, y1, x2, y2;
I32 dist2, rel_dist2;


for(i=0; i<count; i++)
  {
  area1 = objlist[i].area;
  x1    = objlist[i].x_center;
  y1    = objlist[i].y_center;

#ifdef PRINTOUT
  print("area1: %d x1: %d y1: %d\n", area1, x1, y1);
#endif

  for(k = i+1; k<count; k++)
    {
	area2 = objlist[k].area;
    x2    = objlist[k].x_center;
    y2    = objlist[k].y_center;

#ifdef PRINTOUT
    print("area2: %d x2: %d y2: %d\n", area2, x2, y2);
#endif

    area_sum  = area1 + area2;
	area_diff = abs(area1 - area2);

#ifdef PRINTOUT
    print("area_sum: %d area_diff: %d\n", area_sum, area_diff);
#endif

	if(area_diff < area_sum/8)			/* relative area difference must be < 12.5%	*/
	  {
	  dist2 = (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2);	/* euclidean squared distance	*/

	  rel_dist2 = (dist2 + area_sum/2)/area_sum; 

#ifdef PRINTOUT
	  print("area_sum: %d dist2: %d relative squared distance: %d\n", area_sum, dist2, rel_dist2);
#endif

	  if((rel_dist2 > MNRLD) && (rel_dist2 < MXRLD))
	    {
		*ndx1 = i;
		*ndx2 = k;
		goto loop_exit;
		}
	  }


	}
  }


if((k==count) && (i==count))
  {
  print("pair not found\n");
  return -1;
  }

loop_exit:

#ifdef PRINTOUT
print("i: %d area: %d x: %d y: %d\n", i, objlist[i].area, objlist[i].x_center, objlist[i].y_center); 
print("k: %d area: %d x: %d y: %d\n", k, objlist[k].area, objlist[k].x_center, objlist[k].y_center); 
#endif

return 0;
}




void MarkCross(image *Hough8, I32 ix, I32 iy, I32 Color, I32 Size)
{
  I32 i, pitch, dx, dy;
  U8  *ppix;

  dx    = Hough8->dx;
  dy    = Hough8->dy;
  pitch = Hough8->pitch;

  ppix  = (U8 *)(Hough8->st + ix + iy * pitch);

  ix    -= Size;
  iy    -= Size;

  for (i=-Size; i<=Size; i++)
  {
    if( ix>=0 && ix<dx ) *(ppix+i)       = Color;
    if( iy>=0 && iy<dy ) *(ppix+i*pitch) = Color;

    ix++;
    iy++;
  }
}

/* EOF */
