/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2009 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: soccer.c
***                                                            
*** Comments: 
*** 
***	   robot color sensor for soccer robot
***
*** Revision History:
***    Date          Version    Author  Changes
***    Jun 10, 2009  3.06       ME      initial version
***    Jul 09, 2009  3.06       ME      release vs. 1.1
***
***************************************************************************
*END***********************************************************************/

#define NEW_IMAGE_VAR


#include <licence.h>

#include <vcrt.h>
#include <flib.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

#define _TI_ENHANCED_MATH_H 1

#include <math.h>
#include <string.h>

#define PI			(3.141592654)
#define DEGTORAD	(2.0*PI/360.0)
#define RADTODEG    (180.0/PI)

/************************************************************************************************************************/

#define  CompName "Vision Components"
#define  PrgName  "Soccer Robot Camera"
#define  PrgVer   11

#define TRANRED    1
#define TRANYEL    3
#define COLGREY    4
#define COLRED     8
#define COLBLACK  16
#define COLWHITE  32
#define COLGREEN  64
#define COLBLUE  128

#define COLCROSS 255		/* crosshair color for SBC4016	*/
#define COLCIRC1 128		/* circle 1 color  for SBC4016	*/
#define COLCIRC2 180		/* circle 2 color  for SBC4016	*/
#define COLCIRC3 200		/* circle 3 color  for SBC4016	*/


/************************************************************************************************************************/

#define  EVEN(x)             ((x)&0xFFFFFFFE)
#define  ODD(x)              (((x)&0xFFFFFFFE)+1)
#define  Round(x)            ((I32)floorf(x+0.5))


typedef struct
{
  //////////////////////////////////////////////////
  // input parameters
  //////////////////////////////////////////////////

  I32   FindZoom;   /* number of demagnification stages			*/
  
  //////////////////////////////////////////////////
  // geometric properties of image, camera, lens
  //////////////////////////////////////////////////

  I32     	ScreenX;		   /* field of view: x-position		*/
  I32       ScreenY;		   /* field of view: y-position		*/
  I32       ScreenDx;		   /* field of view: x-size			*/
  I32       ScreenDy;		   /* field of view: y-size			*/

  image     SearchArea;
  point   	LensCentre;			/* lens center, optical axis	*/
  float     focal;				/* focal length of lens			*/
  I32       RobotRadius;		/* radius of don't care field	*/
  I32       WbRadius;    		/* maximum white balance radius */
  I32       MaxRadius;			/* maximum field of view		*/
  I32       GoalRadius1;    	/* minimum goal radius 			*/
  I32       GoalRadius2;		/* maximum goal radius			*/

  I32       VideoDispl;			/* does camera have a display?	*/
  I32       Gain;				/* AD gain value				*/
  I32       Red;				/* white balance red			*/
  I32       Green;				/* white balance green			*/
  I32       Blue;				/* white balance blue			*/

  U8        *LUT;				/* SW LUT for SBC4016			*/

  //////////////////////////////////////////////////
  // properties of color detection
  //////////////////////////////////////////////////

  I32       GH;					/* hue for green field			*/
  I32		YH;					/* hue for yellow goal			*/
  I32       BH;					/* hue for blue goal			*/

  I32       GT;					/* green tolerance				*/
  I32       YT;					/* yellow tolerance				*/
  I32       BT;					/* blue tolerance				*/

  I32       GS;					/* green saturation				*/
  I32       YS;					/* yellow saturation			*/
  I32       BS;					/* blue saturation				*/

  U8        *ctable;			/* table for color detection	*/

  //////////////////////////////////////////////////
  // properties of goal and field detection
  //////////////////////////////////////////////////

  I32		AG;					/* area filter for goal			*/
  I32		AF;					/* area filter for field		*/
  I32       DISP;				/* test display 0 = off			*/

  //////////////////////////////////////////////////
  // output parameters
  //////////////////////////////////////////////////

  float		CenterYellow;		/* angle for yellow goal		*/
  float     DistYellow;			/* distance to goal				*/
  float     MaxYellow;			/* maximum goal angle			*/
  float     MinYellow;			/* minimum goal angle			*/
  
  float		CenterBlue;			/* angle for blue goal			*/
  float     DistBlue;			/* distance to goal				*/
  float     MaxBlue;			/* maximum goal angle			*/
  float     MinBlue;			/* minimum goal angle			*/
  						
  
  I32   	Error;				/* error code					*/
} 

ROB_Par;



I32 InitROB 				   (ROB_Par *Par);

void IniCamera                 (I32 *ScreenX, I32 *ScreenY, I32 *ScreenDx, I32 *ScreenDy);
void IniCameraBack             (void);

void StoreImage                (image *Source);
void WhiteBalance			   (ROB_Par *Par);
void AutoCalibrate			   (ROB_Par *Par);
void RunMode			       (ROB_Par *Par);

void ChangeParameter           (ROB_Par *Par, I32 *SearchX, I32 *SearchY, I32 *SearchDx, I32 *SearchDy, float *Focal);
void ChangeValue               (I32 *ParValue, I32 Min, I32 Max, I32 Step, I8 *text);
I32 SaveParameters             (ROB_Par *Par);

void SelectArea                (I32 *px, I32 *py, I32 *pdx, I32 *pdy, I32 xmin, I32 xmax, I32 ymin, I32 ymax, I32 text, I32 even, I32 color);

void CapturePosition           (I32 x, I32 y);

void ChangeColor               (image *area, I32 start, I32 end, I32 newcol);

void High                      (image *Area);
void Low                       (image *Area);

U16 *							make_field_rlc(image *src, U16 *rlc, I32 nx);
I32 							get_field_contour(ROB_Par *Par, U16 *rlc, point *center);

I32 							line_intersect(image *src, float angle, point *center, float *amin, float *dmin, float *amax, float *dmax);

I32 							find_goal(image *src, U16 *fld, I32 value, point *center, I32 area, float *angle, 
																					float *dist, float *maxangle, float *minangle);
I32                             select_3_colors(U8 *table, I32 hg, I32 tg, I32 sg, I32 hy, I32 ty, I32 sy, I32 hb, I32 tb, I32 sb);
I32								InitCtable(ROB_Par *Par);

extern U16 *ATAN;
extern U16 *NORM;


/************************************************************************************************************************/

void main(void)
{
  char    	AutoMode=0, response;
  I32     	SearchAreaX, SearchAreaY, SearchAreaDx, SearchAreaDy, SearchBorder = 8;
  image   	ImgOvl;
  float   	Focal;
  ROB_Par   Par;

  /////////////////////////////////////////////////////////////////////////////////
  // init all VC library licences
  /////////////////////////////////////////////////////////////////////////////////

  INIT_LICENCE();


  /////////////////////////////////////////////////////////////////////////////////
  // allocate memory and init tables
  /////////////////////////////////////////////////////////////////////////////////

  hue_histo_init(128, 128);		/* must be before InitRob						 */

  /////////////////////////////////////////////////////////////////////////////////
  // init variables
  /////////////////////////////////////////////////////////////////////////////////

  InitROB(&Par);

  // define search area 
  SearchAreaX        = Par.ScreenX  +     SearchBorder;
  SearchAreaY        = Par.ScreenY  +     SearchBorder;
  SearchAreaDx       = Par.ScreenDx - 2 * SearchBorder;
  SearchAreaDy       = Par.ScreenDy - 2 * SearchBorder;
  ImageAssign(&(Par.SearchArea), ScrByteAddr(SearchAreaX, SearchAreaY), SearchAreaDx, SearchAreaDy, ScrGetPitch);  


  /////////////////////////////////////////////////////////////////////////////////
  // start program
  /////////////////////////////////////////////////////////////////////////////////

  printf("\n");
  printf("--- %s Version %d.%d ---\n", PrgName, PrgVer/10, PrgVer%10);
  printf("\n");

  printf("Press ENTER to start single read\n");
  printf("Press  ESC  to quit program\n");
  printf("Press  'p'  to set the program to \"Parameter Adjustment Mode\"\n");
  printf("Press  'w'  for whitebalance\n");
  printf("Press  'c'  for color auto calibration\n");
  printf("Press  'r'  run mode\n");
  printf("Press  's'  to store image\n");
  printf("Press  'a'  to toggle automatic trigger mode\n");
  printf("\n");



  do
  {
//    printf("Mem=%x: \n", sysmemfree() );

    /////////////////////////////////////////////////////////////////////////////////
    // display search area
    /////////////////////////////////////////////////////////////////////////////////

    if(Par.VideoDispl) 
    {
      // draw search area into overlay
      ImageAssign(&ImgOvl, OvlByteAddr(SearchAreaX, SearchAreaY), SearchAreaDx, SearchAreaDy, OvlGetPitch);  
      framed(&ImgOvl, COLBLUE);

      vmode(vmOvlLive);
    }

    /////////////////////////////////////////////////////////////////////////////////
    // wait for terminal input
    /////////////////////////////////////////////////////////////////////////////////

    do
    {
      response = KEY_INVALID;

      // read new character
      if (kbhit()) 
      {
        response = ReadKeyboard(1);
        EmptyKeyboardBuffer();
      }

      if (response == 'p') {ChangeParameter(&Par, &SearchAreaX, &SearchAreaY, &SearchAreaDx, &SearchAreaDy, &Focal);}
      if (response == 'w') {WhiteBalance(&Par);}
      if (response == 'c') {AutoCalibrate(&Par);}
      if (response == 'r') {RunMode(&Par);}
      if (response == 's') {StoreImage(&(Par.SearchArea));}
      if (response == 'a') if(AutoMode) AutoMode=0; else AutoMode=1; 

      if (response == KEY_ESC) goto END;

	  time_delay(10);		/* allow some multi-tasking	*/
    }
    while( (AutoMode == 0) && (response != KEY_ENTER) );


    /////////////////////////////////////////////////////////////////////////////////
    // delete old drawings
    /////////////////////////////////////////////////////////////////////////////////

    if(Par.VideoDispl) 
    {
      ImageAssign(&ImgOvl, OvlByteAddr(Par.ScreenX, Par.ScreenY), Par.ScreenDx, Par.ScreenDy, OvlGetPitch); 
      set(&ImgOvl, 0); 
    }


    /////////////////////////////////////////////////////////////////////////////////
    // RUN
    /////////////////////////////////////////////////////////////////////////////////

    RunMode(&Par);

  }
  while(response != KEY_ESC);

  /////////////////////////////////////////////////////////////////////////////////
  // end of VC program
  /////////////////////////////////////////////////////////////////////////////////

  END:

  /////////////////////////////////////////////////////////////////////////////////
  // deallocate memory
  /////////////////////////////////////////////////////////////////////////////////

  if(Par.LUT) byte_free(Par.LUT); /* deallocate software LUT	*/
  DeinitCtable(Par);

  hue_histo_deinit();

return;
}

/*************************************************************************************************************************/

void IniCamera (I32 *ScreenX, I32 *ScreenY, I32 *ScreenDx, I32 *ScreenDy)

{
  I32   ScrX, ScrY, ScrDx, ScrDy;
  I32   VideoDisplCamera, FullScreenCamera, BorderDx, BorderDy;
  char  string[80];
  image Ovl, Area;


  /////////////////////////////
  // Ini Overlay Border
  /////////////////////////////

  BorderDx = 16;
  BorderDy = 64;

  /////////////////////////////
  // Define Camera Type
  /////////////////////////////

  // cameras witout video output have DispGetRows and DispGetColumns = 0
  if ( (DispGetRows == 0) ||  (DispGetColumns == 0) ) 
  {
    VideoDisplCamera = 0;
    FullScreenCamera = 1;
    BorderDx = 0;
    BorderDy = 0;
  }
  else
  {
    VideoDisplCamera = 1;

    // cameras sensor resolution compared with display resolution
    if (abs(DispGetRows - ScrGetRows) > BorderDy) 
    {
      FullScreenCamera = 0;
    }
    else
    {
      FullScreenCamera = 1;
    }
  }


  /////////////////////////////
  // Define Overlay Color
  /////////////////////////////

  if(VideoDisplCamera)
  {
    /* Define Color */
    set_translucent(1, 250,   0,   0); /* TRANRED    1 red translucent 		*/
    set_translucent(2, 255, 255,   0); /* TRANYEL    3 yellow translucent 	*/
    set_overlay_bit(2, 210, 210, 210); /* COLGREY    4 grey 				*/
    set_overlay_bit(3, 250,   0,   0); /* COLRED     8 red 					*/
    set_overlay_bit(4,  70,  70,  70); /* COLBLACK  16 black 				*/
    set_overlay_bit(5, 250, 250, 250); /* COLWHITE  32 white 				*/
    set_overlay_bit(6,   0, 200,   0); /* COLGREEN  64 green 				*/
    set_overlay_bit(7,  65,  90, 255); /* COLBLUE  128 blue 				*/
  }


  /////////////////////////////
  // Define Logical Pages
  /////////////////////////////

  // screen page
  ScrSetLogPage((I32)ScrGetPhysPage);

  // overlay page
  if(VideoDisplCamera)
  {
    // draw into overlay
    OvlSetLogPage((I32)OvlGetPhysPage);
  }
  else
  {
    // draw into screen
    OvlSetLogPage((I32)ScrGetPhysPage);
  }

  /////////////////////////////
  // Position of the image
  /////////////////////////////

   // Define result output border
   ScrDx = ScrGetColumns - 2 * BorderDx;
   ScrDy = ScrGetRows    - 2 * BorderDy;
   ScrX  = (ScrGetColumns-ScrDx)/2;
   ScrY  = (ScrGetRows   -ScrDy)/2;

  if (FullScreenCamera == 0) 
  {
    // Define screen position in the center. X-position of sensor image must be in allignment of 32
    ScrDx = ScrGetColumns;
    ScrDy = ScrGetRows;
    ScrX  = ((DispGetColumns-ScrDx)/64)*32;
    ScrY  = ((DispGetRows   -ScrDy)/2);

    CapturePosition(ScrX, ScrY);
  }


  /////////////////////////////
  // clear overlay and image
  /////////////////////////////

  if (VideoDisplCamera) 
  {
    ImageAssign(&Ovl,  getvar(OVLY_START), DispGetColumns, DispGetRows, OvlGetPitch);
    set(&Ovl,0);

    ImageAssign(&Area, getvar(DISP_START), DispGetColumns, DispGetRows, DispGetPitch);
    set(&Area, 210);
  }


  /////////////////////////////
  // Draw Screen and Overlay
  /////////////////////////////

  if (VideoDisplCamera) 
  {
    // set background color
    ImageAssign(&Ovl, OvlByteAddr(0, 0), DispGetColumns, DispGetRows, OvlGetPitch);
    set(&Ovl, COLGREY);

    // open screen size
    ImageAssign(&Ovl, OvlByteAddr(ScrX, ScrY), ScrDx, ScrDy, OvlGetPitch);
    set(&Ovl, 0);
    Low(&Ovl);
    
    // Company
    ImageAssign(&Ovl, OvlByteAddr(ScrX, 20), 400, 16, OvlGetPitch);
    chprint1(CompName,&Ovl, 2, 2, COLBLUE);
    ChangeColor(&Ovl,0,0,COLGREY);

    // Version
    sprintf(string, "%s %d.%d", PrgName, PrgVer/10, PrgVer%10);
    ImageAssign(&Ovl, OvlBitAddr(ScrX + ScrDx - 8*strlen(string), 28), 8*strlen(string), 8, OvlGetPitch);
    chprint1(string, &Ovl, 1, 1, COLBLUE);
    ChangeColor(&Ovl,0,0,COLGREY);

    // live image with overlay
    set_ovlmask(255);
    vmode(vmOvlLive);

    if (FullScreenCamera == 0) 
    {
      // now refer to new screen position
      ScrSetLogPage((I32)ScrByteAddr(ScrX, ScrY));
      OvlSetLogPage((I32)OvlByteAddr(ScrX, ScrY));

      ScrX  = 0;
      ScrY  = 0;
      ScrDx = ScrGetColumns;
      ScrDy = ScrGetRows;
    }
  }

  *ScreenX  = ScrX;
  *ScreenY  = ScrY;
  *ScreenDx = ScrDx;
  *ScreenDy = ScrDy;
}

/*************************************************************************************************************************/

void IniCameraBack (void)

{
  I32   VideoDisplCamera;
  image Img;

  /////////////////////////////
  // Define Camera Type
  /////////////////////////////

  // cameras witout video output have DispGetRows and DispGetColumns = 0
  if ( (DispGetRows == 0) ||  (DispGetColumns == 0) ) 
  {
    VideoDisplCamera = 0;
  }
  else
  {
    VideoDisplCamera = 1;
  }

  /////////////////////////////
  // Position of the image
  /////////////////////////////

  if(VideoDisplCamera)
  {
    ScrSetPhysPage(getvar(DISP_START));
    ScrSetLogPage (getvar(DISP_START));
    OvlSetLogPage (getvar(OVLY_START));
  }

  /////////////////////////////
  // stop live mode
  /////////////////////////////

  tpict();

  /////////////////////////////
  // clear overlay and image
  /////////////////////////////

  if(VideoDisplCamera)
  {
    ImageAssign(&Img, OvlByteAddr(0, 0), DispGetColumns, DispGetRows, ScrGetPitch);
    set(&Img,0);

    ImageAssign(&Img, ScrByteAddr(0, 0), DispGetColumns, DispGetRows, ScrGetPitch);
    set(&Img,0);

    // refresh memory
    clean_cache();
  }


  /////////////////////////////
  // switch to live mode
  /////////////////////////////

  vmode(vmLive);
}

/************************************************************************************************************************/



void ChangeParameter (ROB_Par *Par, I32 *SearchX, I32 *SearchY, I32 *SearchDx, I32 *SearchDy, float *Focal)

{
  I32   response, value;
  image Ovl, Scr;

  EmptyKeyboardBuffer();
                                                             
  do
  {
    printf("\n");  
    printf("--------------------------------------------------------------------------------\n");
    printf("l:   live-picture\n"); 
    printf("s:   change search area\n"); 
    printf("o:   change lens parameters\n"); 
    printf("r:   change robot parameters\n"); 
    printf("g:   change goal parameters\n"); 
    printf("c:   change color detection parameters\n"); 
    printf("d:   change goal/field detection parameters\n"); 
    printf("S:   save parameters to file\n"); 
    printf("\n"); 
    printf("q:   quit Menu\n"); 
    printf("--------------------------------------------------------------------------------\n");

    response = ReadKeyboard(1);

    switch(response)
    {
      case 'l' : 
                 vmode(vmOvlLive);
                 break;

      case 's' : 
                 // define search area overlay
                 printf("\nPlease select search area\n"); 
                 SelectArea(SearchX, SearchY, SearchDx, SearchDy, Par->ScreenX, Par->ScreenX+Par->ScreenDx, Par->ScreenY, Par->ScreenY+Par->ScreenDy, 1, 1, COLBLUE);
                 break;

      case 'o' : 
                 value = Round(10.0 * *Focal);
                 ChangeValue(&value,                  40, 1000, 1, "Focal Distance 1/10 mm");
                 *Focal = (float)value / 10.0;

                 if(Par->VideoDispl) 
                   {
                   ImageAssign(&Ovl,  OvlByteAddr(Par->ScreenX, Par->ScreenY), Par->ScreenDx, Par->ScreenDy, OvlGetPitch);
                   MarkCross(&Ovl, Par->LensCentre.x - Par->ScreenX, Par->LensCentre.y - Par->ScreenY, COLBLUE,   4);
                   }
				 else
                   {
                   ImageAssign(&Scr,  ScrByteAddr(Par->ScreenX, Par->ScreenY), Par->ScreenDx, Par->ScreenDy, OvlGetPitch);
                   MarkCross(&Scr, Par->LensCentre.x - Par->ScreenX, Par->LensCentre.y - Par->ScreenY, COLCROSS,   8);
                   }

                 value = Round(Par->LensCentre.x);
                 ChangeValue(&value,                   0, ScrGetColumns, 1, "Lens Centre X (Pixel)");
                 Par->LensCentre.x = (float)value;
                 
                 value = Round(Par->LensCentre.y);
                 ChangeValue(&value,                   0, ScrGetRows,    1, "Lens Centre Y (Pixel)");
                 Par->LensCentre.y = (float)value;
                 
                 break;
      case 'r' : 
                 if(Par->VideoDispl) 
                   {
                   ImageAssign(&Ovl,  OvlByteAddr(Par->ScreenX, Par->ScreenY), Par->ScreenDx, Par->ScreenDy, OvlGetPitch);

                   draw_circled(&Ovl, Par->LensCentre.x - Par->ScreenX, Par->LensCentre.y - Par->ScreenY, Par->RobotRadius, COLBLUE);
                   draw_circled(&Ovl, Par->LensCentre.x - Par->ScreenX, Par->LensCentre.y - Par->ScreenY, Par->WbRadius, COLGREEN);
                   draw_circled(&Ovl, Par->LensCentre.x - Par->ScreenX, Par->LensCentre.y - Par->ScreenY, Par->MaxRadius, COLRED);
				   }
				 else
                   {
                   ImageAssign(&Scr,  OvlByteAddr(Par->ScreenX, Par->ScreenY), Par->ScreenDx, Par->ScreenDy, OvlGetPitch);

                   draw_circled(&Scr, Par->LensCentre.x - Par->ScreenX, Par->LensCentre.y - Par->ScreenY, Par->RobotRadius, COLCIRC1);
                   draw_circled(&Scr, Par->LensCentre.x - Par->ScreenX, Par->LensCentre.y - Par->ScreenY, Par->WbRadius, COLCIRC2);
                   draw_circled(&Scr, Par->LensCentre.x - Par->ScreenX, Par->LensCentre.y - Par->ScreenY, Par->MaxRadius, COLCIRC3);
				   }

                 ChangeValue(&Par->RobotRadius,             0,400, 1, "Select Robot Radius");    
                 ChangeValue(&Par->WbRadius, Par->RobotRadius,500, 1, "Select WhiteBalance Radius");    
                 ChangeValue(&Par->MaxRadius, Par->WbRadius  ,600, 1, "Select Maximum Radius");    
                 break;
      case 'g' : 
                 if(Par->VideoDispl) 
                   {
                   ImageAssign(&Ovl,  OvlByteAddr(Par->ScreenX, Par->ScreenY), Par->ScreenDx, Par->ScreenDy, OvlGetPitch);

                   draw_circled(&Ovl, Par->LensCentre.x - Par->ScreenX, Par->LensCentre.y - Par->ScreenY, Par->GoalRadius1, COLBLUE);
                   draw_circled(&Ovl, Par->LensCentre.x - Par->ScreenX, Par->LensCentre.y - Par->ScreenY, Par->GoalRadius2, COLGREEN);
				   }
				 else
                   {
                   ImageAssign(&Scr,  OvlByteAddr(Par->ScreenX, Par->ScreenY), Par->ScreenDx, Par->ScreenDy, OvlGetPitch);

                   draw_circled(&Scr, Par->LensCentre.x - Par->ScreenX, Par->LensCentre.y - Par->ScreenY, Par->GoalRadius1, COLCIRC1);
                   draw_circled(&Scr, Par->LensCentre.x - Par->ScreenX, Par->LensCentre.y - Par->ScreenY, Par->GoalRadius2, COLCIRC2);
				   }

                 ChangeValue(&Par->GoalRadius1, 100, Par->MaxRadius, 1, "Select Minimum Goal Radius");    
                 ChangeValue(&Par->GoalRadius2, 100, Par->MaxRadius, 1, "Select Maximum Goal Radius");    
                 break;
      case 'c' : 
                 ChangeValue(&Par->GH, 0, 511, 1, "Select Green Hue Value");    
                 ChangeValue(&Par->GT, 0, 100, 1, "Select Green Tolerance");    
                 ChangeValue(&Par->GS, 0, 128, 1, "Select Green Saturation");    
                 ChangeValue(&Par->YH, 0, 511, 1, "Select Yellow Hue Value");    
                 ChangeValue(&Par->YT, 0, 100, 1, "Select Yellow Tolerance");    
                 ChangeValue(&Par->YS, 0, 128, 1, "Select Yellow Saturation");    
                 ChangeValue(&Par->BH, 0, 511, 1, "Select Blue Hue Value");    
                 ChangeValue(&Par->BT, 0, 511, 1, "Select Blue Tolerance");    
                 ChangeValue(&Par->BS, 0, 511, 1, "Select Blue Saturation");    
                 InitCtable(Par);
                 break;
      case 'd' : 
                 ChangeValue(&Par->AG,   0, 1000,  1,  "Select Area Filter for Goal");    
                 ChangeValue(&Par->AF,   0, 10000, 10, "Select Area Filter for Field");    
                 ChangeValue(&Par->DISP, 0, 1,     1,  "Select Test Display (0 = off)");
                 ChangeValue(&Par->FindZoom, 0, 2, 1,  "Select FindZoom (0 = none)");    
                 break;
      case 'S' :
                 SaveParameters(Par); 
                 break;
    }

    /////////////////////////////////////////////////////////////////////////////////
    // delete old drawings
    /////////////////////////////////////////////////////////////////////////////////

    if(Par->VideoDispl) 
      {
      ImageAssign(&Ovl, OvlByteAddr(Par->ScreenX, Par->ScreenY), Par->ScreenDx, Par->ScreenDy, OvlGetPitch); 
      set(&Ovl, 0); 
      }
	else
	  {
	  tpict();
	  }

  }
  while(response != 'q');

  
  printf("RUN mode\n"); 
}

/*************************************************************************************************************************/


void SelectArea (I32 *px, I32 *py, I32 *pdx, I32 *pdy, I32 xmin, I32 xmax, I32 ymin, I32 ymax, I32 text, I32 even, I32 color)

  {
  I32   x, y, dx, dy, response, increment=10, mode=0, smallstep, bigstep;
  image Ovl, Text;
  char  textstr[40];

  printf("\n");
  printf("Press F1     step size\n");
  printf("Press F2     toggle between Position and Size\n");
  printf("Press CURSOR to change values\n");
  printf("Press RETURN to accept values\n");

  x  = *px;
  y  = *py;
  dx = *pdx;
  dy = *pdy;

  bigstep = 10;
  increment = bigstep;
  if(even) smallstep = 2;
  else     smallstep = 1;

  vmode(vmOvlLive);

  ImageAssign(&Ovl, OvlByteAddr(x, y), dx, dy, OvlGetPitch);  

  /* select size */
  do
  {
    framed(&Ovl, color);

    if(text)
    {
      ImageAssign(&Text, OvlBitAddr(x+2, y+2), dx, 8, OvlGetPitch);  
      sprintf(textstr,"X=%d Y=%d DX=%d DY=%d", x, y, dx, dy); 
      chprint1(textstr,&Text, 1, 1, color);
    }

    response = ReadKeyboard(1);
    switch(response)
        {
        case KEY_DOWN : if(mode) {dy+=increment; if(y+dy > ymax) dy=ymax- y;}
                        else     { y+=increment; if(y+dy > ymax)  y=ymax-dy;} break;
        case KEY_RIGHT: if(mode) {dx+=increment; if(x+dx > xmax) dx=xmax- x;}
                        else     { x+=increment; if(x+dx > xmax)  x=xmax-dx;} break;
        case KEY_UP   : if(mode) {dy-=increment; if(dy <    2) dy=2;} 
                        else     { y-=increment; if(y  < ymin)  y=ymin;} break;
        case KEY_LEFT : if(mode) {dx-=increment; if(dx <    2) dx=2;} 
                        else     { x-=increment; if(x  < xmin)  x=xmin;} break;

        case KEY_F1   : if(increment==bigstep) increment=smallstep; else increment=bigstep; break;
        case KEY_F2   : if(mode     == 1) mode =0; else mode = 1; break;
        }

    if (even)
    {
      x  = EVEN(x);
      y  = EVEN(y);
      dx = EVEN(dx);
      dy = EVEN(dy);
    }

    if(text)
    {
      chprint1(textstr,&Text,1,1,0);
    }

    framed(&Ovl, 0);
    ImageAssign(&Ovl,OvlBitAddr(x,y), dx, dy, OvlGetPitch);  
  }
  while(response!=KEY_ENTER && response!=KEY_ESC);

  if (response==KEY_ENTER)
    {
    *px  =  x;
    *py  =  y;
    *pdx = dx;
    *pdy = dy;
    }
  }

/*************************************************************************************************************************/

void StoreImage (image *Source)
{
  static I32 ImgCount=0;
  char ImgName[80];

  // take image
  tpict();

  { 
    ChangeValue(&ImgCount, 0, 500, 1, "Write image number");

    ////////////////////
    // write jpg image
    ////////////////////
    /*
    if (Par->DmError != 0) sprintf(ImgName, "fd:/Dm%03dErr.jpg", ImgCount++);
    else                     sprintf(ImgName, "fd:/Dm%03d.jpg",    ImgCount++);
    print("Write image '%s'. Please wait...\n", ImgName);
    fwrite_jpeg(&Source, ImgName, 50, (I32)mremain());
    */

    ////////////////////
    // write bmp image
    ////////////////////
    sprintf(ImgName, "md:/Img%03d.bmp",    ImgCount++);
    printf("Write image '%s'. Please wait...\n", ImgName);
    fwrite_image(ImgName, Source); 
     
    printf("Write image ready.\n");
  }
}


void CapturePosition(I32 x, I32 y)
{
  image area;


  /* define image variable at the image page */
  ImageAssign(&area, ScrGetCaptPage, ScrGetColumns, ScrGetRows, ScrGetPitch);

  if (x < 0) x=0;
  if (x > DispGetColumns-ScrGetColumns) x=DispGetColumns-ScrGetColumns;

  x=(x/16)*16;

  if (y < 0) y=0;
  if (y > DispGetRows-ScrGetRows) y=DispGetRows-ScrGetRows;

  ScrSetCaptPage(ScrGetDispPage + x + y*ScrGetPitch);
   
  /* delete drawings from old position */
  set(&area, 0);

  tpict(); // get new image
}

/*************************************************************************************************************************/

void ChangeColor (image *area, I32 start, I32 end, I32 newcol)

{
  I32 i, j, dx, dy, pitch;
  U8 *restrict ppix, *addr;

  
  addr  = area->st;
  dx    = area->dx;
  dy    = area->dy;
  pitch = area->pitch;

  for (j=0; j<dy; j++)
  {
    ppix = addr+(pitch*j);

    for (i=0; i<dx; i++)
    {
      if ( (*ppix>=start) && (*ppix<=end) )
      {
        *ppix=newcol;
      }
      ppix++;
    }
  }
}

/*************************************************************************************************************************/

void High (image *a)

{
  image area;
  I32   dx, dy;
  
  ImageAssign(&area, a->st-2-2*a->pitch, a->dx+4, a->dy+4, a->pitch);

  dx = area.dx-1;
  dy = area.dy-1;
  
  lined(&area,    0,    0,   dx,    0, COLWHITE);
  lined(&area,    1,    1, dx-1,    1, COLWHITE);
  lined(&area,    0,    0,    0,   dy, COLWHITE);
  lined(&area,    1,    1,    1, dy-1, COLWHITE);

  lined(&area,    1,   dy,   dx,   dy, COLBLACK);
  lined(&area,    2, dy-1, dx-1, dy-1, COLBLACK);
  lined(&area,   dx,    1,   dx,   dy, COLBLACK);
  lined(&area, dx-1,    2, dx-1, dy-1, COLBLACK);
}

/*************************************************************************************************************************/

void Low (image *a)

{
  image area;
  I32   dx, dy;
  
  ImageAssign(&area, a->st-2-2*a->pitch, a->dx+4, a->dy+4, a->pitch);
  
  dx = area.dx-1;
  dy = area.dy-1;

  lined(&area,    0,    0,   dx,    0, COLBLACK);
  lined(&area,    1,    1, dx-1,    1, COLBLACK);
  lined(&area,    0,    0,    0,   dy, COLBLACK);
  lined(&area,    1,    1,    1, dy-1, COLBLACK);

  lined(&area,    1,   dy,   dx,   dy, COLWHITE);
  lined(&area,    2, dy-1, dx-1, dy-1, COLWHITE);
  lined(&area,   dx,    1,   dx,   dy, COLWHITE);
  lined(&area, dx-1,    2, dx-1, dy-1, COLWHITE);
}



I32 InitROB (ROB_Par *Par)
{
  FILE       *fp;
  I32 x, y;
  I32 err = ERR_NONE;
  I32 err2;

  //////////////////////////////////////////////////
  // ini screen parameters
  //////////////////////////////////////////////////

  // ini display and define Screen Size and Positon (Screen = image capture position)
  IniCamera(&Par->ScreenX, &Par->ScreenY, &Par->ScreenDx, &Par->ScreenDy);

  //////////////////////////////////////////////////
  // ini input parameters
  //////////////////////////////////////////////////

  Par->FindZoom    = 0;

  //////////////////////////////////////////////////
  // geometric properties of image, camera, lens
  //////////////////////////////////////////////////

  Par->LensCentre.x = getvar(HWIDTH)/2.0;					/* lens center, optical axis	*/
  Par->LensCentre.y = getvar(VWIDTH)/2.0;					/* lens center, optical axis	*/
  Par->focal        = 1.3;									/* focal length of lens			*/
  Par->RobotRadius  = (2*getvar(HWIDTH)/8);					/* radius of don't care field	*/
  Par->WbRadius     = (3*getvar(HWIDTH)/8);    				/* maximum white balance radius */
  Par->MaxRadius    = (4*getvar(HWIDTH)/8);					/* maximum field of view		*/
  Par->GoalRadius1  = (Par->WbRadius+Par->MaxRadius)/2;		/* maximum field of view		*/
  Par->GoalRadius2  = (Par->GoalRadius1+Par->MaxRadius)/2;	/* maximum field of view		*/

  // cameras witout video output have DispGetRows and DispGetColumns = 0
  if ( (DispGetRows == 0) ||  (DispGetColumns == 0) ) Par->VideoDispl = 0;
  else                                                Par->VideoDispl = 1; 

  printf("VideoDisplay = %d\n", Par->VideoDispl);

  Par->Gain  = getvar(GAIN);
  Par->Red   = getvar(RED);
  Par->Green = getvar(GREEN);
  Par->Blue  = getvar(BLUE);

  Par->LUT   = NULL;								/* default: no LUT				*/

  Par->GH = 439;									/* default for green			*/
  Par->YH = 372;									/* default for yellow			*/
  Par->BH = 66;										/* default for blue				*/

  Par->GT = 10;										/* default tolerance for green	*/
  Par->YT = 60;										/* default tolerance for yellow	*/
  Par->BT = 60;										/* default tolerance for blue	*/

  Par->GS = 10;										/* default saturation for green	*/
  Par->YS = 40;										/* default saturation for yel.	*/
  Par->BS = 10;										/* default saturation for blue	*/

  Par->ctable = NULL;								/* table for color detection	*/

  Par->AG = 100;									/* area filter for goal			*/
  Par->AF = 1000;									/* area filter for field		*/
  Par->DISP = 0;									/* display off					*/
  Par->Error = 0;

  /* is it a real color camera with hardware color LUT ?	*/

  if( (getvar(COLOR_MODE) == -1) || (getvar(OVLY_START) == 0))
    {
	printf("no LUT, allocate Software LUT\n");
	Par->LUT = byte_malloc(1024);
	}

  fp = fopen("fd:/SOCCER.asc","r");
  if(!fp) {err = ERR_OPEN; goto ERR_RET;}

  printf("loading data from file\n");

  fscanf(fp,"%d\n", &x); Par->LensCentre.x = x;
  fscanf(fp,"%d\n", &y); Par->LensCentre.y = y;
  fscanf(fp,"%6.2f\n", &Par->focal);			 
 
  fscanf(fp,"%d\n", &Par->RobotRadius);		 
  fscanf(fp,"%d\n", &Par->WbRadius);			 
  fscanf(fp,"%d\n", &Par->MaxRadius);
  fscanf(fp,"%d\n", &Par->GoalRadius1);			 
  fscanf(fp,"%d\n", &Par->GoalRadius2);
  
  fscanf(fp,"%d\n", &Par->Gain);		   			 
  fscanf(fp,"%d\n", &Par->Red);		  	 
  fscanf(fp,"%d\n", &Par->Green);				 
  fscanf(fp,"%d\n", &Par->Blue);		

  fscanf(fp,"%d\n", &Par->GH);
  fscanf(fp,"%d\n", &Par->YH);
  fscanf(fp,"%d\n", &Par->BH);
  fscanf(fp,"%d\n", &Par->GT);
  fscanf(fp,"%d\n", &Par->YT);
  fscanf(fp,"%d\n", &Par->BT);
  fscanf(fp,"%d\n", &Par->GS);
  fscanf(fp,"%d\n", &Par->YS);
  fscanf(fp,"%d\n", &Par->BS);

  fscanf(fp,"%d\n", &Par->AG);
  fscanf(fp,"%d\n", &Par->AF);
  fscanf(fp,"%d\n", &Par->FindZoom);			
  
  fclose(fp);

ERR_RET:

  setvar(GAIN,  Par->Gain);	
  setvar(RED,   Par->Red);	
  setvar(GREEN, Par->Green);
  setvar(BLUE,  Par->Blue);

  init_color_lut_ALL(Par, getvar(RED), getvar(GREEN), getvar(BLUE));
  err2 = InitCtable(Par);
  if(err2 != ERR_NONE) printf("error in InitCtable() = %d\n", err2); 

  return err;
}


void WhiteBalance(ROB_Par *Par)
{
image Ovl, wb1, wb2, wo1, wo2, Scr;
point center;
I32   wbsize, wb1X, wb2X, wb1Y;

int   r1, g1, b1, r2, g2, b2, red, green, blue; 
int   bright, br1, br2, brmax=200, brwhite = 255;
//image bayer, ovl;

center.x = Par->LensCentre.x - Par->ScreenX;
center.y = Par->LensCentre.y - Par->ScreenY;
wbsize   = EVEN(Par->WbRadius - Par->RobotRadius);

wb1X     = EVEN(Par->ScreenX + (I32)center.x - Par->WbRadius);			/* must be even	*/ 
wb2X     = EVEN(Par->ScreenX + (I32)center.x + Par->WbRadius - wbsize);	/* must be even	*/

if(getvar(MODEL) != 4016)
  {
  wb1Y     = EVEN(Par->ScreenY + (I32)center.y - wbsize/2);					/* must be even	*/
  }
else
  {
  wb1Y     = ODD(Par->ScreenY + (I32)center.y - wbsize/2);					/* must be odd	*/
  }

printf("white balance, center = [%3d, %3d], wbsize = %d\n", (I32)center.x, (I32)center.y, wbsize);

ImageAssign(&wb1,  ScrByteAddr(wb1X, wb1Y), wbsize, wbsize, OvlGetPitch);
ImageAssign(&wb2,  ScrByteAddr(wb2X, wb1Y), wbsize, wbsize, OvlGetPitch);

wb1.type=IMAGE_BAYER;
wb2.type=IMAGE_BAYER;

if(Par->VideoDispl) 
  {
  ImageAssign(&Ovl,  OvlByteAddr(Par->ScreenX, Par->ScreenY), Par->ScreenDx, Par->ScreenDy, OvlGetPitch);
  ImageAssign(&wo1,  OvlByteAddr(wb1X, wb1Y), wbsize, wbsize, OvlGetPitch);
  ImageAssign(&wo2,  OvlByteAddr(wb2X, wb1Y), wbsize, wbsize, OvlGetPitch);

  framed(&wo1, COLGREY);
  framed(&wo2, COLGREY);

  draw_circled(&Ovl, (I32)center.x, (I32)center.y, Par->RobotRadius, COLBLUE);
  draw_circled(&Ovl, (I32)center.x, (I32)center.y, Par->WbRadius, COLGREEN);
  draw_circled(&Ovl, (I32)center.x, (I32)center.y, Par->MaxRadius, COLRED);
  }
else
  {
  ImageAssign(&Scr,  ScrByteAddr(Par->ScreenX, Par->ScreenY), Par->ScreenDx, Par->ScreenDy, OvlGetPitch);
  ImageAssign(&wo1,  ScrByteAddr(wb1X, wb1Y), wbsize, wbsize, OvlGetPitch);
  ImageAssign(&wo2,  ScrByteAddr(wb2X, wb1Y), wbsize, wbsize, OvlGetPitch);

  framed(&wo1, COLGREY);
  framed(&wo2, COLGREY);

  draw_circled(&Scr, (I32)center.x, (I32)center.y, Par->RobotRadius, COLCIRC1);
  draw_circled(&Scr, (I32)center.x, (I32)center.y, Par->WbRadius, COLCIRC2);
  draw_circled(&Scr, (I32)center.x, (I32)center.y, Par->MaxRadius, COLCIRC3);
  }

 
  init_color_lut_ALL(Par, 1024, 1024, 1024);	/* IMPORTANT: select standard values before calculate white balance !!! */
  
  /* brigthness adjustment and white balance, keep shutter fixed, change gain	*/

  setvar(GAIN, 0);

  do
  {
    tpict();  
    br1 = WhiteBalanceValues(&wb1, &r1, &g1, &b1);
    br2 = WhiteBalanceValues(&wb2, &r2, &g2, &b2);

	bright = (br1 + br2) / 2;
	red    = (r1 + r2)  / 2;
	green  = (g1 + g2)  / 2;
	blue   = (b1 + b2)  / 2;

    printf("gain = %3d bright = %3d red = %4d green = %4d blue = %4d\n", getvar(GAIN), bright, red, green, blue);    
    if(bright < brmax) setvar(GAIN, getvar(GAIN) + 1);
  }
  while (bright < brmax);

  /* increase gain for maximum performance	*/
  do
  {
    tpict();  
    br1 = WhiteBalanceValues(&wb1, &r1, &g1, &b1);
    br2 = WhiteBalanceValues(&wb2, &r2, &g2, &b2);

	bright = (br1 + br2) / 2;

    printf("gain = %3d bright = %3d\n", getvar(GAIN), bright);    
    if(bright < brwhite) setvar(GAIN, getvar(GAIN) + 1);
  }
  while (bright < brwhite);

  // program new values for white balance
  init_color_lut_ALL(Par, red, green, blue);

  print("\n");
  print("Red   = %3d\n", red);
  print("Green = %3d\n", green);
  print("Blue  = %3d\n", blue);
  print("\n");
  

  Par->Gain 	= getvar(GAIN);	
  Par->Red  	= red;	
  Par->Green	= green;
  Par->Blue     = blue;

}



void AutoCalibrate(ROB_Par *Par)
{
image Ovl, wb1, wb2, wo1, wo2, Scr;
image ycbcr1, ycbcr2;
point center;
I32   wbsize, wb1X, wb2X, wb1Y, gbsize;

U32 *hist, *hue, *sat;
U8 *table;

I32 green  = -1;
I32 yellow = -1;
I32 blue   = -1;

center.x = Par->LensCentre.x - Par->ScreenX;
center.y = Par->LensCentre.y - Par->ScreenY;
wbsize   = EVEN(Par->WbRadius - Par->RobotRadius);
wb1X     = EVEN(Par->ScreenX + (I32)center.x - Par->WbRadius);			/* must be even	*/ 
wb2X     = EVEN(Par->ScreenX + (I32)center.x + Par->WbRadius - wbsize);	/* must be even	*/

if(getvar(MODEL) != 4016)
  {
  wb1Y     = EVEN(Par->ScreenY + (I32)center.y - wbsize/2);					/* must be even	*/
  }
else
  {
  wb1Y     = ODD(Par->ScreenY + (I32)center.y - wbsize/2);					/* must be odd	*/
  }

printf("auto calibrate, center = [%3d, %3d], wbsize = %d\n", (I32)center.x, (I32)center.y, wbsize);

tpict_with_LUT(Par);		/* take picture	*/

ImageAssign(&wb1,  ScrByteAddr(wb1X, wb1Y), wbsize, wbsize, ScrGetPitch);
ImageAssign(&wb2,  ScrByteAddr(wb2X, wb1Y), wbsize, wbsize, ScrGetPitch);

wb1.type=IMAGE_BAYER;
wb2.type=IMAGE_BAYER;

if(Par->VideoDispl) 
  {
  ImageAssign(&Ovl,  OvlByteAddr(Par->ScreenX, Par->ScreenY), Par->ScreenDx, Par->ScreenDy, OvlGetPitch);
  ImageAssign(&wo1,  OvlByteAddr(wb1X, wb1Y), wbsize, wbsize, OvlGetPitch);
  ImageAssign(&wo2,  OvlByteAddr(wb2X, wb1Y), wbsize, wbsize, OvlGetPitch);

  framed(&wo1, COLGREY);
  framed(&wo2, COLGREY);

  draw_circled(&Ovl, (I32)center.x, (I32)center.y, Par->RobotRadius, COLBLUE);
  draw_circled(&Ovl, (I32)center.x, (I32)center.y, Par->WbRadius, COLGREEN);
  draw_circled(&Ovl, (I32)center.x, (I32)center.y, Par->MaxRadius, COLRED);
  }
else
  {
  ImageAssign(&Scr,  ScrByteAddr(Par->ScreenX, Par->ScreenY), Par->ScreenDx, Par->ScreenDy, OvlGetPitch);
  ImageAssign(&wo1,  ScrByteAddr(wb1X, wb1Y), wbsize, wbsize, OvlGetPitch);
  ImageAssign(&wo2,  ScrByteAddr(wb2X, wb1Y), wbsize, wbsize, OvlGetPitch);

  framed(&wo1, COLGREY);
  framed(&wo2, COLGREY);

  draw_circled(&Scr, (I32)center.x, (I32)center.y, Par->RobotRadius, COLCIRC1);
  draw_circled(&Scr, (I32)center.x, (I32)center.y, Par->WbRadius, COLCIRC2);
  draw_circled(&Scr, (I32)center.x, (I32)center.y, Par->MaxRadius, COLCIRC3);
  }

printf("detecting green color\n");

ImageAllocate(&ycbcr1, IMAGE_CBCR444, wbsize, wbsize);	/* allocate YCbCr color image	*/
ImageAllocate(&ycbcr2, IMAGE_CBCR444, wbsize, wbsize);	/* allocate YCbCr color image	*/

hist  = (U32 *)vcmalloc(65536);
table =  (U8 *)vcmalloc(16384);
hue   = (U32 *)vcmalloc(512);	/* 512 should be enough	*/
sat   = (U32 *)vcmalloc(512);	/* 512 should be enough */ 

BayerToYCbCr(&wb1, &ycbcr1);	/* convert left	green field color	*/
BayerToYCbCr(&wb2, &ycbcr2);	/* convert right green field color	*/


ycbcr1.dx -= 4; ycbcr1.dy -= 4;		/* correct image for border of 5x5 operator	*/
color_histo(&ycbcr1, hist);			/* full 2D color histogram					*/
ycbcr1.dx += 4;	ycbcr1.dy += 4;		/* restore to original size					*/
hue_histo(hist, hue, sat);			/* convert color histogram into hue histo	*/
FL_MAX_U32P(512, hue, &green);		/* find maximum, some averaging maybe later	*/

printf("hue: maximum at green = %d hue_count = %d sat = %d\n", green, hue[green], sat[green]);

Par->GH = green;					/* store green field hue value				*/

color_sector(table, green, Par->GT, Par->GS, 128, 255);

color_classify(&ycbcr1, &wo1, table);
color_classify(&ycbcr2, &wo2, table);

ImageFree(&ycbcr2);
ImageFree(&ycbcr1);

printf("detecting yellow and blue goals\n");

gbsize   = EVEN(Par->GoalRadius2 - Par->GoalRadius1);
wb1X     = EVEN(Par->ScreenX + (I32)center.x - Par->GoalRadius2);			/* must be even	*/ 
wb2X     = EVEN(Par->ScreenX + (I32)center.x + Par->GoalRadius2 - gbsize);	/* must be even	*/

ImageAssign(&wb1,  ScrByteAddr(wb1X, wb1Y), gbsize, wbsize, ScrGetPitch);
ImageAssign(&wb2,  ScrByteAddr(wb2X, wb1Y), gbsize, wbsize, ScrGetPitch);

wb1.type=IMAGE_BAYER;
wb2.type=IMAGE_BAYER;

if(Par->VideoDispl) 
  {
  ImageAssign(&wo1,  OvlByteAddr(wb1X, wb1Y), gbsize, wbsize, OvlGetPitch);
  ImageAssign(&wo2,  OvlByteAddr(wb2X, wb1Y), gbsize, wbsize, OvlGetPitch);

  framed(&wo1, COLGREY);
  framed(&wo2, COLGREY);

  draw_circled(&Ovl, (I32)center.x, (I32)center.y, Par->GoalRadius1, COLBLUE);
  draw_circled(&Ovl, (I32)center.x, (I32)center.y, Par->GoalRadius2, COLGREEN);
  }
else
  {
  ImageAssign(&wo1,  ScrByteAddr(wb1X, wb1Y), gbsize, wbsize, OvlGetPitch);
  ImageAssign(&wo2,  ScrByteAddr(wb2X, wb1Y), gbsize, wbsize, OvlGetPitch);

  framed(&wo1, COLGREY);
  framed(&wo2, COLGREY);

  draw_circled(&Scr, (I32)center.x, (I32)center.y, Par->GoalRadius1, COLCIRC1);
  draw_circled(&Scr, (I32)center.x, (I32)center.y, Par->GoalRadius2, COLCIRC2);
  }

ImageAllocate(&ycbcr1, IMAGE_CBCR444, gbsize, wbsize);	/* allocate YCbCr color image	*/
ImageAllocate(&ycbcr2, IMAGE_CBCR444, gbsize, wbsize);	/* allocate YCbCr color image	*/

BayerToYCbCr(&wb1, &ycbcr1);		/* convert left	yellow field color			*/
BayerToYCbCr(&wb2, &ycbcr2);		/* convert right blue field color			*/

ycbcr1.dx -= 4; ycbcr1.dy -= 4;		/* correct image for border of 5x5 operator	*/
color_histo(&ycbcr1, hist);			/* full 2D color histogram					*/  
ycbcr1.dx += 4;	ycbcr1.dy += 4;		/* restore to original size					*/
hue_histo(hist, hue, sat);			/* convert color histogram into hue histo	*/
FL_MAX_U32P(512, hue, &yellow);		/* find maximum, some averaging maybe later	*/

ycbcr2.dx -= 4; ycbcr2.dy -= 4;		/* correct image for border of 5x5 operator	*/
color_histo(&ycbcr2, hist);			/* full 2D color histogram					*/  
ycbcr2.dx += 4;	ycbcr2.dy += 4;		/* restore to original size					*/
hue_histo(hist, hue, sat);			/* convert color histogram into hue histo	*/
FL_MAX_U32P(512, hue, &blue);		/* find maximum, some averaging maybe later	*/


printf("hue: maximum at yellow = %d blue = %d\n", yellow, blue);

Par->YH = yellow;					/* store yellow goal hue value				*/
Par->BH = blue;						/* store blue goal hue value				*/

color_sector(table, yellow, Par->YT, Par->YS, 128, 255);
color_classify(&ycbcr1, &wo1, table);

color_sector(table, blue,   Par->BT, Par->BS, 128, 255);
color_classify(&ycbcr2, &wo2, table);


InitCtable(Par);


vcfree(sat);
vcfree(hue);
vcfree(table);
vcfree(hist);
ImageFree(&ycbcr2);
ImageFree(&ycbcr1);
}





I32 SaveParameters(ROB_Par *Par)
{

FILE       *fp;


fp = fopen("fd:/SOCCER.asc","w");

if(!fp) return ERR_OPEN;

fprintf(fp,"%6.2f\n", Par->LensCentre.x);	/* LensCentre.x = 	*/
fprintf(fp,"%6.2f\n", Par->LensCentre.y);	/* LensCentre.y = 	*/
fprintf(fp,"%6.2f\n", Par->focal);			/* focal = 			*/

fprintf(fp,"%d\n", Par->RobotRadius);		/* RobotRadius = 	*/
fprintf(fp,"%d\n", Par->WbRadius);			/* WbRadius = 		*/
fprintf(fp,"%d\n", Par->MaxRadius);			/* MaxRadius =		*/ 
fprintf(fp,"%d\n", Par->GoalRadius1);		/* GoalRadius1 = 	*/
fprintf(fp,"%d\n", Par->GoalRadius2);		/* GoalRadius2 = 	*/

fprintf(fp,"%d\n", Par->Gain);	   			/* Gain = 			*/
fprintf(fp,"%d\n", Par->Red);		  		/* Red =			*/ 
fprintf(fp,"%d\n", Par->Green);				/* Green = 			*/
fprintf(fp,"%d\n", Par->Blue);				/* Blue = 			*/

fprintf(fp,"%d\n", Par->GH);
fprintf(fp,"%d\n", Par->YH);
fprintf(fp,"%d\n", Par->BH);
fprintf(fp,"%d\n", Par->GT);
fprintf(fp,"%d\n", Par->YT);
fprintf(fp,"%d\n", Par->BT);
fprintf(fp,"%d\n", Par->GS);
fprintf(fp,"%d\n", Par->YS);
fprintf(fp,"%d\n", Par->BS);

fprintf(fp,"%d\n", Par->AG);
fprintf(fp,"%d\n", Par->AF);
fprintf(fp,"%d\n", Par->FindZoom);			/* FindZoom = 		*/

fclose(fp);


return 0;
}
 



#define RLCBYTES (0x80000)
#define RLCWORDS (0x40000)
#define NOBJ     (100)



image *xdisp = NULL;				/* global variable for debug display	*/



void RunMode(ROB_Par *Par)
{
image img, ovl, cls;
image ycbcr;
point center;						/* center with reference to screen		*/
point centerS;						/* center with reference to search area	*/
I32   imgDx, imgDy, imgX, imgY;
I32   i, fz, ms;
U16   *frlc;


center.x = Par->LensCentre.x - (float)Par->ScreenX;
center.y = Par->LensCentre.y - (float)Par->ScreenY;
imgDx    = 2*min(Par->MaxRadius, min((I32)center.x, Par->ScreenDx - (I32)center.x));
imgDy    = 2*min(Par->MaxRadius, min((I32)center.y, Par->ScreenDy - (I32)center.y));

imgX     = EVEN(Par->ScreenX + (I32)center.x - imgDx/2);			/* must be even	*/ 


if(getvar(MODEL) != 4016)
  {
  imgY     = EVEN(Par->ScreenY + (I32)center.y - imgDy/2);			/* must be even	*/
  }
else
  {
  imgY     = ODD(Par->ScreenY + (I32)center.y - imgDy/2);			/* must be odd	*/
  }


centerS.x = (Par->LensCentre.x - (float)imgX)/(float)(1<<Par->FindZoom);
centerS.y = (Par->LensCentre.y - (float)imgY)/(float)(1<<Par->FindZoom);

ms  = getvar(MSEC); 

ImageAssign(&img,  ScrByteAddr(imgX, imgY), imgDx, imgDy, ScrGetPitch);
img.type=IMAGE_BAYER;
ImageAllocate(&cls, IMAGE_GREY, imgDx, imgDy);		/* color classification result image	*/ 
ImageAllocate(&ycbcr, IMAGE_CBCR444, imgDx, imgDy);	/* allocate YCbCr color image			*/

if(Par->VideoDispl) 
  {
  ImageAssign(&ovl,  OvlByteAddr(imgX, imgY), imgDx, imgDy, OvlGetPitch);
  framed(&ovl, COLGREY);
  if(Par->DISP) xdisp = &ovl; else xdisp = NULL;
  }
else
  if(Par->DISP) xdisp = &img; else xdisp = NULL;
 
frlc = rlcmalloc(RLCWORDS);					/* allocate memory for field RLC			*/

tpict_with_LUT(Par);

if(fz=Par->FindZoom)						/* pyramid zoom for higher speed			*/
  {
  for(i=0;i<fz; i++)
    {				 
    pyr_col(&img, &img);
    ImageAssignC(&img, img.st, img.type, imgDx/2, imgDy/2, img.pitch, img.ccmp1, img.ccmp2); 
	}

  ImageAssignC(&ycbcr, ycbcr.st, ycbcr.type, ycbcr.dx>>fz, ycbcr.dy>>fz, ycbcr.pitch, ycbcr.ccmp1, ycbcr.ccmp2);
  ImageAssign(&cls, cls.st, cls.dx>>fz, cls.dy>>fz, cls.pitch); 
  ImageAssign(&ovl, ovl.st, ovl.dx>>fz, ovl.dy>>fz, ovl.pitch); 
  }

BayerToYCbCr(&img, &ycbcr);							/* convert to YCbCr							*/

if(Par->ctable == NULL) goto err_exit;				/* no classification table, return			*/

color_classify(&ycbcr, &cls, Par->ctable);			/* color classification for all 3 colors	*/

if(Par->DISP)
  {
  if(Par->VideoDispl) copy(&cls, &ovl);				/* display result							*/
  else                copy(&cls, &img);     		/* display result							*/
  }

make_field_rlc(&cls, frlc, 4 - (1<<Par->FindZoom));	/* dilate field RLC for goal detection		*/

find_goal(&cls, frlc, COLRED,   &centerS, Par->AG, &(Par->CenterYellow), &(Par->DistYellow), &(Par->MaxYellow), &(Par->MaxYellow));
find_goal(&cls, frlc, COLGREEN, &centerS, Par->AG, &(Par->CenterBlue),   &(Par->DistBlue),   &(Par->MaxBlue),   &(Par->MinBlue));

get_field_contour(Par, frlc, &centerS);

ms = getvar(MSEC)-ms; if(ms<0) ms+=1000;

printf("GY %d\n", (I32)Par->CenterYellow);			/* output format: angle [deg]		*/
printf("DY %d\n", (I32)Par->DistYellow<<fz);		/* output format: distance [pixels]	*/
printf("GB %d\n", (I32)Par->CenterBlue);			/* output format: angle [deg]		*/
printf("DB %d\n", (I32)Par->DistBlue<<fz);			/* output format: distance [pixels]	*/
printf("TI %d\n", ms);								/* output format: time [msec]		*/

err_exit:
rlcfree(frlc);
ImageFree(&ycbcr);
ImageFree(&cls);

}



I32 select_3_colors(U8 *table, I32 hg, I32 tg, I32 sg, I32 hy, I32 ty, I32 sy, I32 hb, I32 tb, I32 sb)
{
image scratch, tab;
I32 err = ERR_NONE;

if(ImageAllocate(&scratch, IMAGE_GREY, 256, 256) == NULL) return ERR_MEMORY;
ImageAssign(&tab, table, 256, 256, 256);

err=color_sector(table, hg, tg, sg, 256, COLBLUE);  	if(err) goto ERR_RET;
err=color_sector(scratch.st, hy, ty, sy, 256, COLRED); 	if(err) goto ERR_RET;

or2(&tab, &scratch, &tab);

color_sector(scratch.st, hb, tb, sb, 256, COLGREEN); 	if(err) goto ERR_RET;

or2(&tab, &scratch, &tab);

ERR_RET:
ImageFree(&scratch);

return err;
}


I32 InitCtable(ROB_Par *Par)	/* must be done whenever a color selection value changes	*/
{
I32 err;

if(Par->ctable == NULL)
  Par->ctable =  (U8 *)vcmalloc(16384);

if(Par->ctable == NULL)
  err = ERR_MEMORY;
else
  {
//  printf("green = %d yellow = %d blue = %d\n", Par->GH, Par->YH, Par->BH);
  err=select_3_colors(Par->ctable, Par->GH, Par->GT, Par->GS, Par->YH, Par->YT, Par->YS, Par->BH, Par->BT, Par->BS);
  }

return err;
}

DeinitCtable(ROB_Par *Par)
{

if(Par->ctable) vcfree(Par->ctable);

Par->ctable = NULL;
}


#define NOBJ2 1000

I32 find_goal(image *src, U16 *fld, I32 value, point *center, I32 area, float *angle, float *dist, float *maxangle, float *minangle)
{
int i, nobj, err = ERR_NONE;
U16 *rlc, *r1, *r2;
U16 *next;
ftr f[NOBJ2];
I32 index = -1, amax=0;
point v;


/* default values for output, must be set first in case of error	*/

*angle			= -1.0;
*dist			= -1.0;
*maxangle		= -1.0;
*minangle		= -1.0;

rlc = (U16 *)byte_malloc(RLCBYTES);
r1	= (U16 *)byte_malloc(RLCBYTES);
r2	= (U16 *)byte_malloc(RLCBYTES);

next = rlcmkbit(src, value, rlc, RLCBYTES);	/* generate rlc         			*/
if(next == NULL) {err = -1; goto err_exit;}

next=sgmt(rlc, next);                  	/* object labelling     				*/
if(next == NULL) {err = -1; goto err_exit;}

rlc_arf(rlc, rlc, area);				/* area filter, min object size = 100	*/
										/* reduces number of objects			*/

erode(rlc, r1);							/* dilate goal RLC						*/
erode2(r1, r2);						    /* dilate goal RLC						*/
erode(r2, r1);							/* dilate goal RLC						*/

next=rlcand(fld, r1, rlc);				/* mask with field RLC					*/

next=sgmt(rlc, next);                  	/* object labelling again				*/
if(next == NULL) return ERR_OVERRUN;

if(xdisp) rlcout(xdisp, rlc, 0, value); /* optional rlc display 				*/

nobj=rl_ftr2(rlc, f, NOBJ2);			/* feature extraction					*/

for(i=0;i<nobj;i++)
  {
  if((f[i].area > amax) && (f[i].color&1)) {index = i; amax = f[i].area; }
  }
										/* object not found						*/
if(index == -1) 
  {
  err = ERR_CORR;
  *angle = 0.0;
  *dist  = 0.0;
  } 	
else
  {
  v.x = (float)f[index].x_center - center->x;
  v.y = (float)f[index].y_center - center->y;

  *angle = AngleP(&v) * RADTODEG;			/* goal angle							*/
  *dist  = Norm(&v);						/* goal distance						*/
  }

err_exit:
byte_free(r2);
byte_free(r1);
byte_free(rlc);

return err;
}



U16 *make_field_rlc(image *src, U16 *rlc, I32 nx)  /* dilate field RLC for goal detection	*/
{
U16 *r1, *r2;
I32 i;

r1 = rlcmalloc(RLCWORDS);						/* allocate memory for rlc				*/
r2 = rlcmalloc(RLCWORDS);						/* allocate memory for rlc				*/

rlcmkbit(src, COLBLUE, r1, 2*RLCWORDS);			/* generate rlc         				*/

if(r1 == NULL) goto err_exit;

for(i=0; i<nx-1; i++)
  {
  erode(r1, r2);
  erode2(r2, r1);
  }

erode(r1, r2);
erode2(r2, rlc);								/* final RLC output						*/

if(xdisp) rlcout(xdisp, rlc, 0, COLBLUE);		/* optional rlc display 				*/

err_exit:
rlcfree(r2);
rlcfree(r1);

return NULL;
}


tpict_with_LUT(ROB_Par *Par)					/* take picture with white balance		*/
{												/* cameras with HW LUT have				*/
tpict();										/* built-in white balance, all others	*/
												/* use software LUT for white balance	*/
if(Par->LUT != NULL)
  {
  clut_bayer(&(Par->SearchArea), &(Par->SearchArea), Par->LUT);	/* Software LUT	*/
  }

}


init_color_lut_ALL(ROB_Par *Par, I32 red, I32 green, I32 blue)
{
if(Par->LUT != NULL)
  {
  init_color_table(red, green, blue, Par->ctable);
  }
else
  init_color_lut(red, green, blue);
}



I32 get_field_contour(ROB_Par *Par, U16 *rlc, point *center)
{
U16 *r1, *r2, *next;
ftr f[NOBJ2];
I32 i, index = -1;
I32 af = Par->AF;
I32 xc, yc, nobj;
float angle;
image img;
I32 dx, dy, nx;
float amin, amax, dmin, dmax;
I32 err = ERR_NONE;
I32 fz = Par->FindZoom;

r1 = rlc;						
r1 += 2;
dx  = *r1++;         			/* read line delimiter                  */
dy  = *r1++;         			/* read dy                              */

r1 = rlcmalloc(RLCWORDS);						/* allocate memory for rlc			*/
r2 = rlcmalloc(RLCWORDS);						/* allocate memory for rlc			*/

/* the RLC of the field is dilated, so make sure to erode it to the original size	*/

nx = 4-(1<<Par->FindZoom);

dilate2(rlc, r1);
dilate(r1, r2);

for(i=0; i<nx-1; i++)
  {
  dilate2(r2, r1);
  next = dilate(r1, r2);
  }

if(xdisp) rlcout(xdisp, r2, 0, COLBLUE);		/* optional rlc display 			*/

next=sgmt(r2, next);                  			/* object labelling 				*/
if(next == NULL) {err = ERR_OVERRUN; goto err_exit;}

nobj=rl_ftr2(r2, f, NOBJ2);						/* object features					*/

xc = (I32)center->x;
yc = (I32)center->y;

for(i=0;i<nobj;i++)
  {
  if((f[i].area > af) && (f[i].color&1) && (f[i].x_min < xc) && (f[i].x_max > xc) && (f[i].y_min < yc) && (f[i].y_max > xc)) 
     {
     index = i; 
     af = f[i].area; 
     }
  }

if(index == -1) {err = ERR_CORR; goto err_exit;}	/* object not found	*/

rlc_cut(r2, r2, index);						/* select only one object				*/

ImageAllocate(&img, IMAGE_GREY, dx, dy);	/* allocate image for boundary search	*/
rlcout(&img, r2, 0, 255);					/* output rlc to image variable			*/

for(i=0; i<8; i++)
  {
  angle = (float)i  * (PI/8.0);

  line_intersect(&img, angle, center, &amin, &dmin, &amax, &dmax);	/* intersection of line with object		*/

  printf("CA %3d\n", (I32)amin);  
  printf("CD %3d\n", (I32)dmin<<fz);  
  printf("CA %3d\n", (I32)amax);  
  printf("CD %3d\n", (I32)dmax<<fz);  
  }

ImageFree(&img);
err_exit:
rlcfree(r2);
rlcfree(r1);

return err;
}



I32 line_intersect(image *src, float angle, point *center, float *amin, float *dmin, float *amax, float *dmax)
{
I32 i;
I32 x0, y0, x1, y1, sx, sy;
I32 x, y;
I32 xmin = MAXINT;
I32 ymin = MAXINT;
I32 xmax = -1;
I32 ymax = -1;
I32 *buf, *p;
vcline line;
point vmin, vmax;

U8 *start, *addr;
I32 pitch;

start = src->st;
pitch = src->pitch;

line.cx = sinf(angle);					/* normal vector for line: cx		*/
line.cy = cosf(angle);					/* normal vector for line: cy		*/

LineParallel(center, &line, &line);		/* line must pass through center	*/

/* calculate xy-list for line												*/
/* calculate intersection with window										*/

if(ERR_NONE != IntersectionPoints (src, line.cx, line.cy, line.b, &x0, &y0, &x1, &y1)) return ERR_SINGULAR;

sx = x1 - x0;
sy = y1 - y0;

buf = p = (I32 *)vcmalloc(2 * (1 + max(abs(sx), abs(sy)))); 

linexy(sx, sy, buf);			/* call Bresenham						*/

if((angle >= 45*DEGTORAD) && (angle < 135*DEGTORAD))
  {
  for(i=0; i<abs(sy); i++)		/* vertical intersection				*/
    {
    x = *p++ + x0;				/* get pixel x position of line			*/
    y = *p++ + y0;				/* get pixel y position of line			*/

	addr = start + x + y*pitch;

	if(*addr)
	  {
	  if(y < ymin) {xmin = x; ymin = y;}
	  if(y > ymax) {xmax = x; ymax = y;}
	  }
    }
  }
else
  {
  for(i=0; i<abs(sx); i++)		/* horizontal intersection				*/
    {
    x = *p++ + x0;				/* get pixel x position of line			*/
    y = *p++ + y0;				/* get pixel y position of line			*/

	addr = start + x + y*pitch;

	if(*addr)
	  {
	  if(x < xmin) {xmin = x; ymin = y;}
	  if(x > xmax) {xmax = x; ymax = y;}
	  }
    }
  }

//printf("min = [%d, %d] max = [%d, %d]\n", xmin, ymin, xmax, ymax);

if(xdisp) MarkCross(xdisp, xmin, ymin, COLRED,     4);
if(xdisp) MarkCross(xdisp, xmax, ymax, COLGREEN,   4);

if(xmin == MAXINT)							/* not found					*/
  {
  *amin = 0.0;
  *dmin = 0.0;
  }
else
  {
  vmin.x = (float)xmin - center->x;
  vmin.y = (float)ymin - center->y;
  *amin   = AngleP(&vmin)*RADTODEG; 		/* calculate polar coordinates	*/
  *dmin   = Norm(&vmin);
  } 


if(xmax == -1)							    /* not found					*/
  {
  *amin = 0.0;
  *dmin = 0.0;
  }
else
  {
  vmax.x = (float)xmax - center->x;
  vmax.y = (float)ymax - center->y;
  *amax   = AngleP(&vmax)*RADTODEG;			/* angles output in degrees		*/
  *dmax   = Norm(&vmax); 					/* distance output in pixels	*/
  }

vcfree(buf);

return ERR_NONE;
}




/* EOF */


