/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2000-2004 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: rlcbench.c
***                                                            
*** Comments: 
*** 
***    benchmark for RLC functions
***
*** Revision History:
***    Date          Version    Author  Changes
***    Oct 14, 2004  2.00       ME      Initial Version
***
***************************************************************************
*END***********************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <register.h>
#include <sysvar.h>

#define DX 640
#define DY 480

/**************************************************************************/

void main(void)
	{
	image a = { 0L, DX, DY, 768};
	int i, t0, t1;
	U16 *rlc, *r1;
	int pitch;
	feature f;

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
    /*******************************************************************************/

	print("RLC BENCHMARK VS 1.1\n");
	rs232rcv();

	pitch = getvar(VPITCH);
	a.st = (long)getvar(DISP_START);
	a.pitch = pitch;

	rlc = (U16 *)vcmalloc(0x10000);			/* 0x10000 * 4 bytes of memory	*/


	/* rlcmk ****************************************************************/

	t0 = getvar(MSEC);

	for(i=0; i<10; i++)
	  rlcmk(&a, 128, rlc, 0x40000);

	t1 = getvar(MSEC) - t0;
	if (t1 < 0) t1 += 1000; 

	print("rlcmk:       %d.%d msec\n",t1/10,t1-10*(t1/10));

	/* rlcout ***************************************************************/

	r1=testrlc(rlc, DX, DY, 32);		/* testimage with size=10084 U16	*/ 
										/* for DX=640 DY=480				*/
	t0 = getvar(MSEC);

	for(i=0; i<10; i++)
	  rlcout(&a, rlc, 0, 255);

	t1 = getvar(MSEC) - t0;
	if (t1 < 0) t1 += 1000; 

	print("rlcout:      %d.%d msec\n",t1/10,t1-10*(t1/10));
	print("size: %d\n",r1-rlc);

	/* erode ***************************************************************/

	t0 = getvar(MSEC);

	for(i=0; i<10; i++)
	  erode(rlc,r1);

	t1 = getvar(MSEC) - t0;
	if (t1 < 0) t1 += 1000; 

	print("erode:       %d.%d msec\n",t1/10,t1-10*(t1/10));

	/* erode2 **************************************************************/

	t0 = getvar(MSEC);

	for(i=0; i<10; i++)
	  erode2(rlc,r1);

	t1 = getvar(MSEC) - t0;
	if (t1 < 0) t1 += 1000; 

	print("erode2:      %d.%d msec\n",t1/10,t1-10*(t1/10));
	  
	/* rlc_feature *********************************************************/

	t0 = getvar(MSEC);

	for(i=0; i<10; i++)
	  rlc_feature(&f,rlc,-1);

	t1 = getvar(MSEC) - t0;
	if (t1 < 0) t1 += 1000; 

	print("rlc_feature: %d.%d msec\n",t1/10,t1-10*(t1/10));

	/* rlc2 ***************************************************************/
	/* rl_ftr2 ************************************************************/
	   
	/* rlc_inv ************************************************************/

	t0 = getvar(MSEC);

	for(i=0; i<10; i++)
	   rlc_inv(rlc);

	t1 = getvar(MSEC) - t0;
	if (t1 < 0) t1 += 1000; 

	print("rlc_inv:     %d.%d msec\n",t1/10,t1-10*(t1/10));

	/* rlc_mf *************************************************************/

	t0 = getvar(MSEC);

	for(i=0; i<10; i++)
	  rlc_mf(rlc, rlc, -1, 10); 

	t1 = getvar(MSEC) - t0;
	if (t1 < 0) t1 += 1000; 

	print("rlc_mf:      %d.%d msec\n",t1/10,t1-10*(t1/10));

	/* rlc_move ***********************************************************/

	t0 = getvar(MSEC);

	for(i=0; i<10; i++)
	  rlc_move(rlc, r1, -100, -100); 

	t1 = getvar(MSEC) - t0;
	if (t1 < 0) t1 += 1000; 

	print("rlc_move:    %d.%d msec\n",t1/10,t1-10*(t1/10));


	/* sgmt **************************************************************/

	t0 = getvar(MSEC);

	for(i=0; i<10; i++)
	  sgmt(rlc,r1);

	t1 = getvar(MSEC) - t0;
	if (t1 < 0) t1 += 1000; 

	print("sgmt:        %d.%d msec\n",t1/10,t1-10*(t1/10));



	vcfree((int *)rlc);
	}



/* EOF */
