/*******************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Klaus Schneider,  VC
		 JME, VC revised for Image Transfer
		 LB, comments added, ping() added


Program Description:
--------------------

Demofile for 100 MBit Ethernet Client
Image transfer from VC Image Server (img2.c)

*/
/*******************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

#define SEND_BYTES  24
#define MEMORY      950000    /***** Memory to store one image *****/
						      /***** enough for a 1024*768 image, for 640*480 500000 is sufficient *****/

/*******************************************************************/

int GetInt (void);

/*******************************************************************/



void main(void)
{
   unsigned char *linebuffer, *ximg, *zz;
   unsigned int senddata[SEND_BYTES/4]; 
   int  i, j, status, optlen, result, PingRes, ms, dx = 640, dy = 480;     /***** Resolution of the camera which sends the image *****/
   long sec;
   int count;
   sockaddr_in    raddr;
   sockaddr_in    laddr;
   unsigned       sock, sendsock;
   unsigned       error;
   U32 optval, PingTimeout = 5000;

   /* allocate memory for image raw data */
   linebuffer=(unsigned char *)sysmalloc(MEMORY,3);

   ximg = (unsigned char *)getvar(DISP_START);

   /* server services port 2002 */
   raddr.sin_family      = AF_INET;
   raddr.sin_port        = 2002;
   raddr.sin_addr.s_addr = 0xc0a80041;   /***** IP address of the camera which sends the image, here 192.168.0.65 *****/

   /* client */
   laddr.sin_family      = AF_INET;
   laddr.sin_port        = 0;
   laddr.sin_addr.s_addr = 0;


    /* Listen on TCP port */
   sock = socket_stream();

   if (sock == VCRT_HANDLE_ERROR) 
     {
     print("\nCreate stream socket failed");
     return;
     }
	 

   error = bind(sock, &laddr, sizeof(laddr));

   if (error != VCRT_OK) 
     {
     print("\nStream bind failed - 0x%x", error);
     return;
     }
   
   // Ping before connect()ing
   PingRes = VCRT_ping(raddr.sin_addr.s_addr, &PingTimeout, 123);
   if (PingRes != VCRT_OK) goto END;

   print("\nconnecting to server: 0x%08x\n",raddr.sin_addr.s_addr);
   error = connect(sock, &raddr, sizeof(raddr));

   if (error != VCRT_OK) 
     {
     print("\nconnect failed - 0x%x", error);
     return;
     }

   sendsock = sock;

   print("\n\nRemote Ethernet port %d\n",raddr.sin_port);
   vmode(vmStill);

   for (;;) {

        print("\nConnection to %d.%d.%d.%d, port %d",
           (raddr.sin_addr.s_addr >> 24) & 0xFF,
           (raddr.sin_addr.s_addr >> 16) & 0xFF,
           (raddr.sin_addr.s_addr >>  8) & 0xFF,
            raddr.sin_addr.s_addr        & 0xFF,
            raddr.sin_port);


        do
        {

   		  ximg = (unsigned char *)getvar(DISP_START);
  
          senddata[0]=(unsigned int)0;		/* x 	*/
          senddata[1]=(unsigned int)0;		/* y 	*/
          senddata[2]=(unsigned int)dx;	    /* dx	*/
          senddata[3]=(unsigned int)dy;  	/* dy	*/
          senddata[4]=(unsigned int)1;		/* incx	*/
          senddata[5]=(unsigned int)1;		/* incy	*/

          print("\nSend loop data;  x=%d y=%d dx=%d dy=%d incx=%d incy=%d ",
           	senddata[0], senddata[1], senddata[2], senddata[3], senddata[4], senddata[5] );
  
          result=send(sock, (char *)senddata, SEND_BYTES, 0);
          print("\nSend done");

          if(result==VCRT_ERROR) 
          {
            print("\nSend VCRT_ERROR=%lx",VCRT_geterror(sock));
          }
          else
          {
            if(result!=SEND_BYTES)    
            print("\nSend error wrong length=%d result=%d",SEND_BYTES,result);
          }


          sec = getvar(SEC);
          ms  = getvar(MSEC);
           
          print("\nReceive data");
          count = recv(sock, (char *)linebuffer, 28+dx*dy, 0); 

          if(count == VCRT_ERROR)
            {
            print("\nVCRT_ERROR 0x%x",VCRT_geterror(sock));
            break;
            }
          else
            {
			zz = linebuffer+28;	 /***** Write to image buffer *****/

            for(i=0;i<dy;i++)
              { 
           	  for(j=0;j<dx;j++)
           		{
				*ximg++ = *zz++;
                }
			  ximg += 160;    /***** Add the difference between horizontal size of video output and horizontal size of image *****/
			  				  /***** Here, camera with resolution of 640*480, video output of 800*600, difference = 800-640 = 160 *****/
							  /***** For a camera with resolution of 1024*768, video output of 1280*1024, difference = 1280-1024 = 256 *****/
			  }
            }


            sec = getvar(SEC) - sec;
            ms  = getvar(MSEC) - ms;
            
            ms = 1000*(int)sec + ms;
            print("\nt=%dms ",ms);
        }
		while(!kbhit());
 
   } /* Endfor */

 END:
   print("Ping timed out\n");
   shutdown(sendsock, FLAG_CLOSE_TX);

   sysfree(linebuffer);


} /* Endbody */

/*******************************************************************/

int GetInt (void)
{
  int v=0, r;

  while(1)
  {
    r=(int)(rs232rcv());

    /* CR or LF */
    if (r==13 || r==10) return(v);

    /* number */
    if (r>=0x30 && r<=0x39)
    {
      print("%c",(char)r);
      r=r&0xF;
      v=10 * v + r; 
    }
  }
}

/*******************************************************************/

