/*******************************************************************/

/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Michael Steinel,  VC


Program Description:
--------------------

resolve a given hostname to IP address.

*/

/*******************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <flib.h>
#include <macros.h>
#include <sysvar.h>


#define CHECK_RESOLVER
#define PING_HOST

/*******************************************************************/

int main(int argc, char * argv)
{
  HOSTENT_STRUCT_PTR he;
  int i;
  int id=0, next, running=0;
  char ip[16] = "XXX.XXX.XXX.XXX";

  if(!argc) argv="smtp.t-online.de"; //default hostname if no argument

  /*******************************************************************/

#ifdef CHECK_RESOLVER  
  /* check if resolver is running */
  while(next=task_get_next_id(id))
  {
	id = next;
	if(0==strcmp("DNS Resolver",task_get_name_from_id(id)))
	{  
	  running = 1;
	  break;
	}
  }

  if(running == 0)
  {
	printf("DNS Resolver task not running!!!\n");
	printf("check DNS entries in #IP.asc !!!\n");
	return -1;  
  }	
#endif

  /*******************************************************************/

  /* resolve hostname */
  {
	printf("resolve hostname with gethostbyname():\n"); 
	he = gethostbyname(argv);                 // resolve forward lookup

	if(he==0) return -1;

	print("name='%s'\n",he->h_name);          // show name

	for(i=0;he->h_aliases[i] && i < 16;i++)   // show list of alias names
	  print("alias[%d]='%s'\n",i,he->h_aliases[i]);

	print("type=%d\n",he->h_addrtype);        // show type always AF_INET = 1

	print("lenght=%d\n",he->h_length);        // show size always size_of(inet_addr)

	for(i=0;he->h_addr_list[i] && i < 16;i++) // show list of IP address strings
	  print("addr[%d]='%s'\n",i,VCRT_inet_ntoa(*(in_addr *)(he->h_addr_list[i])));
  }

  /*******************************************************************/

  /* resolve IP address */
  {
	in_addr addr = *(in_addr *)(he->h_addr_list[0]);
	printf("resolve hostname with gethostbyaddr():\n"); 
	memcpy(ip,VCRT_inet_ntoa(addr),16);     // store IP as string in ip[] array
	printf("\n\n");
	he = gethostbyaddr(ip,4,1);             // resolve reverse lookup
	if(he==0) return -1;
	print("name='%s'\n",he->h_name);        // show reverse lookup name
  } 

  /*******************************************************************/

#ifdef PING_HOST
  /* ping the first resolved host ip */
  {
	in_addr addr; 
	uint_32 dtimeout=1000;
	uint_16 id = 0;

	if(VCRT_inet_aton(ip,&addr))
	  printf("\nIP: %s -> 0x%x\n",ip,addr.s_addr);
	else
	  print("Error converting IP string in int addr!!!\n");
	
	print("\n\npinging IP %s ... (press any key to stop)\n",ip);
	
	while(!kbhit())
	{
	  uint_32 timeout = dtimeout;
	  int res;
	  time_delay(timeout);
	  res = VCRT_ping(addr.s_addr, &timeout, id);
	  if(res == VCRT_OK)
		print("%s seq=%u rtt=%d ms\n",ip,id,timeout);
	  else
		print("%s seq=%u timeout=%d ms error=%s\n",
			   ip,id,timeout,VCRT_ip_err_str(res));
	  id++;
	}
  }
#endif

  /*******************************************************************/
}
