/*******************************************************************/

/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Michael Steinel,  VC


Program Description:
--------------------

ftpclient recording images to a file and transfer it to a remote FTP server.

*/

/*******************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <flib.h>
#include <macros.h>
#include <sysvar.h>

/*******************************************************************/

#define FTPMODE_DEFAULT 1 
#define FTPMODE_PORT    2
#define FTPMODE_PASV    3
#define FTPDIR_RECV     0
#define FTPDIR_SEND     4

int_32  FTP_open           (pointer *, ip_address, FILE_PTR);
int_32  FTP_command        (pointer, char *, FILE_PTR);
int_32  FTP_command_data   (pointer, char *, FILE_PTR, FILE_PTR, uint_32);
int_32  FTP_close          (pointer, FILE_PTR);

/*******************************************************************/

int main()
{
  image iScreen;
  int  nr = 0;
  int  nResult = 0;
  char cmd[256];
  char cFileName[28];
  char * tmpfile  = ("md:/tmp.jpg"); // temporary file - TODO: should be on a ramdisk
  FILE * fd_data  = NULL;
  //FILE * fd_ctrl  = NULL; // handy to have no output
  FILE * fd_ctrl  = (FILE *)io_get_handle(IO_STDOUT); // print ctrl messages to stdout
  ip_address ip   = 0xc0a80001; // FTP server IP 192.168.0.1
  pointer handle  = 0;          // FTP session handle

  //!!!!! init license here !!!!!!!!

  nResult = FTP_open(&handle,ip,fd_ctrl);                   //connect
  if(nResult == -1) return -1;
  nResult = FTP_command(handle,"USER testftp\r\n",fd_ctrl); //authenticate user 'testftp'
  if(nResult == -1) return -1;
  nResult = FTP_command(handle,"PASS testftp\r\n",fd_ctrl); //with password     'testftp'
  if(nResult == -1) return -1;
  nResult = FTP_command(handle,"TYPE I\r\n",fd_ctrl);       //set binary transfer mode
  if(nResult == -1) return -1;

  ScrSetLogPage (ScrGetPhysPage);

  ImageAssign (&iScreen, ScrByteAddr (0,0), ScrGetColumns, ScrGetRows, ScrGetPitch);

  tpict();

  do
  {
	nr++;
	io_sprintf(cFileName,"%s%d%s","pt",nr,".jpg");
	
	print ("nr: %4d  - file: %s\n", nr, cFileName);
	
	tpict();

	nResult = fwrite_jpeg(&iScreen, tmpfile, 80, mremain());

	sprintf(cmd,"STOR %s\r\n",cFileName);
	
	fd_data= (FILE *)io_fopen(tmpfile,"r");
	
	if(!fd_data) return -1;
	
	nResult = FTP_command_data( handle, cmd, fd_ctrl, fd_data, FTPMODE_PASV | FTPDIR_SEND ); 
	
	if(nResult == -1) return -1;
	
	nResult = io_fclose(fd_data);	

	if ((rbempty()!=-1) || (nResult == -1))
	{
	  print("res=0x%x\n", nResult);
	  rs232rcv();
	  break;
	}
  } while (1);

  nResult = FTP_close(handle,fd_ctrl);

  return 0;
}
/*******************************************************************/
