/************************************************************************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Klaus Schneider,  VC


Program Description:
--------------------

The demo program receives data at SERVER_PORT 2000. The function TcpData() returns immediately, if 
there are no data available. If there are data at port 2000, it will display it on the telnet port 23.

If you open a new connection on SERVER_PORT 2000 it will close the old connection if one exists, at takes
the data from the new connection.

Example:

1. open a telnet session at port 23
2. start the program "tcpcom"
3. open an additional session at port 2000
4. type in some characters at port 2000, you will see the characters at the telnet port
5. open a new session at port 2000. The program will close the old session automatically and print
   out the characters from the new session
6. go to the telnet port and press 'q'. This will close the port 2000 session and quit the program

*/
/************************************************************************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

/************************************************************************************************************************/

int  TcpData             (unsigned int sock, unsigned int *Alias);

int  TCP_Open            (int Port, unsigned int *NewSock);
int  TCP_Connection      (unsigned int timeout, unsigned int *NewSock);
int  TCP_Accept          (unsigned int Sock, unsigned int *FromSock);
int  TCP_Send            (unsigned int Sock, char *SendData, int SendBytes);
int  TCP_Receive         (unsigned int Sock, char *ReceiveData, int ReceiveBytes);
void TCP_Close           (unsigned int Sock);

/************************************************************************************************************************/

#define SERVER_PORT     2000  /* waiting at port for any connection */

/************************************************************************************************************************/

void main (void)

{
  int           response=-1, result;
  unsigned int  sock, alias; 


  result = TCP_Open(SERVER_PORT, &sock);
  
  if (result ==  0) print("New stream socket created\n");
  if (result == -1) print("Create stream socket failed\n");
  if (result == -2) print("Stream bind failed\n");
  if (result == -3) print("Listen failed\n");

  do
  {
      if (kbhit()) response=rs232rcv(); 
      
      if (response == 'q'  ) goto QUIT;
 
      TcpData(sock, &alias);
      
      time_delay(100);
  }
  while(1);

QUIT:

  print("Quit\n");
  
  TCP_Close(alias);
  TCP_Close(sock);
}

/************************************************************************************************************************/

int TcpData (unsigned int sock, unsigned int *Alias)
{
  int             ConnectionTimeOut, result;
  char            rec;
  static unsigned newsock=0, ContactingRemoteSocket, AliasSocket;
  static int      ConnectionExists=0;

  
  ConnectionTimeOut = -1; // ms

  /* Are there connections at any port? */
  result = TCP_Connection(ConnectionTimeOut, &ContactingRemoteSocket);
/*
  {
  int status;
  status = VCRT_geterror(sock);
  if (status == VCRT_OK) {printf("\nSock Connection ok\n");}
  else printf("\nError Sock Connection = %d\n",status);
  status = VCRT_geterror(ContactingRemoteSocket);
  if (status == VCRT_OK) {printf("\nContactSock Connection ok\n");}
  else printf("\nError ContactSock Connection = %d\n",status);
  }
*/ 
  print ("Connection Result = %d\n", result);

  /* check for connection errors */
  if (result) 
  {
    // if (result ==  1) print ("Connection Time Out Error\n");
    if (result == -1) print ("VCRT Connection Socket Error = %d\n", ContactingRemoteSocket);
    return(result);
  }

  print ("Server Socket=0x%X  Contacting Remote Socket=0x%X\n", sock, ContactingRemoteSocket);


  /* Is there one who wants to connect to port SERVER_PORT */
  if (sock == ContactingRemoteSocket)
  {
    print("New Connection on Server Socket\n");
    
    // Close existing connection
    if (ConnectionExists) 
    {  
      ConnectionExists=0;
      TCP_Close(newsock);
      print("Close Old Socket\n");
    }
    
    result = TCP_Accept(ContactingRemoteSocket, &AliasSocket);

    if (result)
    {
      print("Accept failed, error 0x%x\n",VCRT_geterror(AliasSocket));
      ConnectionExists=0;
      return(result);
    }
    else
    {
      print("accepted\n");
      print ("Alias Socket=0x%X\n", AliasSocket);
      ConnectionExists=1;
      newsock = AliasSocket;
      *Alias  = AliasSocket;
    }
  }
  else
  {
    if (VCRT_geterror(AliasSocket) == VCRT_OK) 
    {
      TCP_Receive(newsock, &rec, 1);

      if (VCRT_geterror(AliasSocket) == VCRT_OK) 
      {
        print("Char = ");
      // TCP_Send(newsock, &rec, 1);
        print("0x%x  >%c<\n", (int)rec, rec);
      }
      else
      {
        if(ConnectionExists) TCP_Close(AliasSocket);
      }
    }
    else
    {
      if(ConnectionExists) TCP_Close(AliasSocket);
    }
  }


  // TCP_Close(AliasSocket);
  
  return(0);
}

/************************************************************************************************************************/

int TCP_Open (int Port, unsigned int *NewSock)
{
  unsigned  error, sock;
//  unsigned  opt_value=1, opt_length=sizeof(unsigned);
  sockaddr_in  laddr;


  /* server services port */
  laddr.sin_family      = AF_INET;
  laddr.sin_port        = Port;        /* server listen at port */
  laddr.sin_addr.s_addr = INADDR_ANY;  /* every IP addr can connect */


  /* use TCP/IP protocoll and search for a free port */
  sock = socket_stream();

  if (sock == VCRT_SOCKET_ERROR) 
  {
    // print("Create stream socket failed\n");
    return(-1);
  } 

  /* receive immediatly the data */  
  /*  
  error = setsockopt(sock, 0, OPT_RECEIVE_NOWAIT, &opt_value, opt_length);
  if (error != VCRT_OK)
  {
    print("Could not set socket option: OPT_RECEIVE_NOWAIT!\n");
  }
  */

  /* connection between free port and selected SERVER_PORT. Now SERVER_PORT is free again for new connections. */
  error = bind(sock, &laddr, sizeof(laddr));

  if (error != VCRT_OK) 
  {
    // print("Stream bind failed - 0x%x\n", error);
    return(-2);
  }

  /* activate sock for TCP/IP connections */
  error = listen(sock, 0);

  if (error != VCRT_OK) 
  {
    // print("Listen failed - 0x%x\n", error);
    return(-3);
  }
  
  print("\nFast Ethernet port %d\n\n", laddr.sin_port);

  *NewSock = sock;
  return(0);
}

/************************************************************************************************************************/

int TCP_Connection (unsigned int timeout, unsigned int *NewSock)
{
   /*
   timeout:
   -  0 means indefinitely
   - -1 means do not block
   -  else waiting time in ms
   
   return
   -  socket
   - -1 if no connection
   -  1 time out
   */

   *NewSock = VCRT_selectall(timeout);

   if (*NewSock == VCRT_SOCKET_ERROR) return(-1);
   if (*NewSock <= 0) return(1); // timeout

   return(0);
}

/************************************************************************************************************************/

int TCP_Accept (unsigned int Sock, unsigned int *FromSock)
{
   uint_16  rlen;
   sockaddr_in  raddr;
 
   /* Connection requested; accept it */
   rlen = sizeof(raddr);

   *FromSock = accept(Sock, &raddr, &rlen);

   if (*FromSock == VCRT_SOCKET_ERROR)
   {
     return(-1);
   }
   
   return(0);
}

/************************************************************************************************************************/

int TCP_Send (unsigned int Sock, char *SendData, int SendBytes)
{
  int result;
  
  result = send(Sock, (char *)SendData, SendBytes, 0);
  
  if(result==VCRT_ERROR) 
  {
    return(-1);
  }
  
  return(result);
}

/************************************************************************************************************************/

int TCP_Receive (unsigned int Sock, char *ReceiveData, int ReceiveBytes)
{
   return( recv(Sock, (char *)ReceiveData, ReceiveBytes, 0) ); 
}

/************************************************************************************************************************/

void TCP_Close (unsigned int Sock)
{
   shutdown(Sock, FLAG_CLOSE_TX);
}

/************************************************************************************************************************/
