#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

/************************************************************************************/
#define LOCAL_PORT      5005  /* local port 5005			 						*/
#define MEMORY          4096  /* memory to store data    							*/
#define TIMEOUT        10000  /* receive timeout in milliseconds                    */
/************************************************************************************/

void main(int argc, char *argv)
{
  unsigned char *linebuffer;
  int  count;
  sockaddr_in    laddr;
  unsigned       sock;
  unsigned       error;
  int ix=0;

  print("\nVCRT UDP recv with timeout demo - Version 1.0");

  /* allocate memory for image raw data */
  linebuffer=(unsigned char *)sysmalloc(MEMORY,3);
  if((int)linebuffer==0) print("not enough memory\n");

  /* server services port; ladr = local address	*/
  laddr.sin_family      = AF_INET;
  laddr.sin_port        = LOCAL_PORT;    /* listen port           */
  laddr.sin_addr.s_addr = INADDR_ANY;    /* listen on any IP addr */

  /* use UDP socket */
  sock = socket_dgram();

  if (sock == VCRT_SOCKET_ERROR)
  {
	print("\nCreate UDP socket failed");
	return;
  } 
  
  /* bind to local address */
  error = bind(sock, &laddr, sizeof(laddr));

  if (error != VCRT_OK) 
  {
	print("\nUDP bind failed - 0x%x", error);
	return;
  } 
  
  print("\nWaiting for UDP packets on port: %d\n\n",LOCAL_PORT);

  while(1)
  {

#if 1
   	/* check for activity on socket sock */ 
	if(VCRT_selectset(&sock,1,TIMEOUT)==0)
	{
	  printf("selectset timeout!");
	  goto shutdown;
	}
#else
   	/* check all sockets of the current task */		   
	if(VCRT_selectall(TIMEOUT)==0)
	{
	  printf("selectall timeout!");
	  goto shutdown;
	}
#endif	   
	
	/* receive data  */
	count = recv(sock, (char *)linebuffer, MEMORY, 0 );
	if (count == VCRT_ERROR) 
	{
	  printf("recvfrom error: 0x%lx\n", VCRT_geterror(sock));
	  return;
	}

	printf("%d receiving UDP data\n",ix++);

  }

shutdown: 

  error = shutdown(sock, 0);
  if (error != VCRT_OK)
  {
	print("\nClose connection failed, 0x%x", error);
  }
  else
  {
	print("\nConnection closed");
  } 

  sysfree(linebuffer);

  return;

} /* Endbody */

/* End of File	*/
