/************************************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program Description:
--------------------

Example of using pipes and user events.
When a key (letter) is pressed, the program evmain writes this letter 10
times to a pipe, and sets an event.
The program evback waits for this event and when the event is set, evback
reads the pipe and outputs its content.
If the event times out more than 5 times (no key pressed), evback sets another
event, which makes evmain end.

This example shows the use of user events with pipes, however pipes do not need 
events for synchronisation.


Revision History:
-----------------

   Date           Version    Author  Changes
   Mar 01, 2007   1.0        LB      Initial Version
   Apr 25, 2007   1.01       LB      open() and close() out of the loop
								     Removed event_connect_to_task(), not
								     required anymore with VCRT 5.26
  Jan 07, 2008    1.02     	LB	Bug fix : replaced getvar(USR_EVENT+1) by getvar(USR_EVENT)+1

*/
/************************************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

/************************************************************************************/

#define KEY_EVENT getvar(USR_EVENT)
#define TIMEOUT_EVENT getvar(USR_EVENT) + 1
#define PIPENAME  "pipe0:"
#define DATASIZE  10

/************************************************************************************/

void main(void)
{
  char buffer[DATASIZE];
  int i, ResWait, Timeout = 5000, to = 0;
  FILE *fpPipe;

  // Allow events
  //event_connect_to_task();

  // Open pipe for read
  fpPipe = io_fopen(PIPENAME, "r");

  do
    {
      // Wait for event from evback
      ResWait = wait(KEY_EVENT, Timeout);

      switch(ResWait)
        {
          case 1:
              print("Event occured : key pressed\n");

			  // Read from pipe
			  io_read(fpPipe, (char *)buffer, sizeof(buffer));

			  // Display what was read from pipe
              print("Character : ");
			  for(i=0; i<DATASIZE; i++) print("%c", buffer[i]);
			  print("\n");

			  break;

          case 2:
    	      print("Event occured before wait was called\n");
			  break;

    	  case -1:
	    	  print("Event timed out\n");
			
			  // Set event if event times out more than 5 times
			  to++;
              if (to >= 5) {set_evt(TIMEOUT_EVENT); goto END;}

			  break;
        }
    }
  while(1);

END:
  io_fclose(fpPipe);
  print("End of background process\n");
  //event_disconnect();
  
}
