/************************************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program Description:
--------------------

Example of using pipes and user events.
When a key (letter) is pressed, the program evmain writes this letter 10
times to a pipe, and sets an event.
The program evback waits for this event and when the event is set, evback
reads the pipe and outputs its content.
If the event times out more than 5 times (no key pressed), evback sets another
event, which makes evmain end.

This example shows the use of user events with pipes, however pipes do not need 
events for synchronisation.


Revision History:
-----------------

   Date           Version    Author  Changes
   Mar 01, 2007   1.00       LB      Initial Version
   Apr 25, 2007   1.01       LB      open() and close() out of the loop
  Jan 07, 2008    1.02     	LB	Bug fix : replaced getvar(USR_EVENT+1) by getvar(USR_EVENT)+1

*/
/************************************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

/************************************************************************************/

#define KEY_EVENT getvar(USR_EVENT)
#define TIMEOUT_EVENT getvar(USR_EVENT) + 1
#define PIPENAME  "pipe0:"
#define DATASIZE  10

/************************************************************************************/

void main(void)
{
  int i, Timeout = 1000, ResWait;
  char Key, buffer[DATASIZE];
  FILE *fpPipe;

  // Install pipe
  io_pipe_install(PIPENAME, sizeof(buffer));

  // Open pipe for write
  fpPipe = io_fopen(PIPENAME, "w");

  // Start background process
  exec2("fd:/evback.000");


  do
    {
      if (kbhit())
        {
          Key = rs232rcv();

          // Set event if a letter is pressed
		  if ((Key >= 0x61) && (Key <= 0x7A))
		    {
			  // Write to a buffer
              for(i=0; i<DATASIZE; i++) buffer[i]=Key;

			  // Write buffer to pipe
			  io_write(fpPipe, (char *)buffer, sizeof(buffer));

			  // Set event
              set_evt(KEY_EVENT);

			}
		}

	  time_delay(5);

	  // Wait for event from evback (set if KEY_EVENT times out more than 5 times)
	  ResWait = wait(TIMEOUT_EVENT, Timeout);
	  if ((ResWait == 1) || (ResWait == 2)) goto END;

    }
  while(Key != 0x1B);
  
END:
  io_fclose(fpPipe);
  print("End of main process\n");

}
