/*******************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Klaus Schneider,  VC


Program Description:
--------------------

Demo for pipes.

*/
/*******************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

/*******************************************************************/

#define DATASIZE  10
#define PIPENAME  "PipeName"

/*******************************************************************/

void main(void)
{
  int  i, buffer[DATASIZE], PipeMode=1;
  char *temp;
  FILE *fpPipe;


  ///////////////////////////////////////////////////////////////////
  // install pipe
  ///////////////////////////////////////////////////////////////////

  io_pipe_install(PIPENAME, sizeof(buffer));

  if (PipeMode)
  {
    // mode FIFO: The write operation waits, until pipe can store all values.
    fpPipe = io_fopen(PIPENAME, "w");
  }
  else
  {
    // mode CIRC: The write operation doesn't wait. You will loose the oldes values if the pipe is full.
    fpPipe = io_fopen(PIPENAME, "x");
  }

  if(!fpPipe)
  {
    pstr("Error: Can't define Pipe\n");
    goto Exit;
  }
  
  ///////////////////////////////////////////////////////////////////
  // or change pipe mode with
  ///////////////////////////////////////////////////////////////////
  {
    // unsigned int flag = CIRC;
    unsigned int flag = FIFO;

    io_ioctl(fpPipe, IO_PIPE_CHMOD, &flag); // only with "&flag" possible
  }

  ///////////////////////////////////////////////////////////////////
  // pipe access with io_write() and io_read()
  ///////////////////////////////////////////////////////////////////

  print("\npipe access with io_write() and io_read()\n");
  
  // ini buffer
  for(i=0; i<DATASIZE; i++) buffer[i]=i;

  // write data
  io_write(fpPipe, (char *)buffer, sizeof(buffer));

  // delete buffer
  for(i=0; i<DATASIZE; i++) buffer[i]=0;

  // read data
  io_read(fpPipe, (char *)buffer, sizeof(buffer));  

  // print buffer
  for(i=0; i<DATASIZE; i++) print("buffer[%d] = %d\n", i, buffer[i]);


  ///////////////////////////////////////////////////////////////////
  // pipe access with io_fputc() and io_fgetc()
  ///////////////////////////////////////////////////////////////////

  print("\npipe access with io_fputc() and io_fgetc()\n");
  
  if (io_fstatus(fpPipe)) print("At least one character in Pipe\n");
  else                    print("Empty Pipe\n");

  // ini buffer
  for(i=0; i<DATASIZE; i++) buffer[i]=i;

  // write data
  io_write(fpPipe, (char *)buffer, sizeof(buffer));

  if (io_fstatus(fpPipe)) print("At least one character in Pipe\n");
  else                    print("Empty Pipe\n");

  // delete buffer
  for(i=0; i<DATASIZE; i++) buffer[i]=0;

  // read data
  temp = (char *)buffer;
  while(io_fstatus(fpPipe)) // true, if data available
  {
    *temp++ = io_fgetc(fpPipe);  
  }

  // print buffer
  for(i=0; i<DATASIZE; i++) print("buffer[%d] = %d\n", i, buffer[i]);

  if (io_fstatus(fpPipe)) print("At least one character in Pipe\n");
  else                    print("Empty Pipe\n");



  ///////////////////////////////////////////////////////////////////
  // general pipe information
  ///////////////////////////////////////////////////////////////////
  {
    unsigned int size, count;

    io_ioctl(fpPipe, IO_PIPE_SIZE, &size);
    io_ioctl(fpPipe, IO_PIPE_GETCOUNT, &count);

    print("Pipe size is = %d Byte, %d Bytes are in pipe\n", size, count);
  }



  ///////////////////////////////////////////////////////////////////
  // close pipe - pipe memory is still allocated !
  // uninstall pipe function is not ready yet.
  ///////////////////////////////////////////////////////////////////

  Exit:

  io_fclose(fpPipe);

}

/*******************************************************************/

