/*******************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Klaus Schneider,  VC


Program Description:
--------------------

This program stores an array of 10 int values in the flash memory
(file datafile.dat). After that it reads the values from the flash
eprom or MMC.

14.11.2006 LB Updated to latest version of io_ioctl() (3 parameters instead of 2)

*/
/*******************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

/*******************************************************************/

void main ()

{
  int  i, var[10], res;
  char *pchar;
  FILE *fp;

  char name[]="md:/datafile.dat";

  /* also possible is
  char name[]="fd:/datafile.dat";     // is equal to xxx.002
  char name[]="fd:/datafile.exe";     // is equal to xxx.000
  char name[]="fd:/datafile.txt";     // is equal to xxx.001
  char name[]="fd:/datafile.jpg";
  char name[]="fd:/datafile.000";
  char name[]="fd:/datafile.250";
  */


  if(getvar(MMC) >= 0) print("Multimedia Card available\n");
  else                 print("Multimedia Card not available\n");



  /***************/
  /* FILE ACCESS */
  /***************/

  print("\nFILE ACCESS\n");


  /* ini variables  (var[0]=0, var[1]=1, ...) */
  /********************************************/
  for (i=0;i<10;i++) var[i]=i;



  /* delete old file if it exists */
  /*******************************/
  fp = io_fopen(name,"c");
  if(fp)
    {
    print("delete old file %s\n",name);
    io_ioctl(fp, IO_DEL, NULL);
    io_fclose(fp);
    }
  else
    {
    print("no old file to delete\n");
    }


  /* write file */
  /**************/
  fp = io_fopen(name,"w");
  if(fp)
    {
    res=io_write(fp, (char *)var, sizeof(var));
    if (res >= 0)  print("%d bytes sucessfully written to file %s\n",res,name);
    else           print("can't write to file %s\n",name);
    io_fclose(fp);
    }
  else
    {
    print("cannot open file for write\n");
    }


  /* delete variables */
  /********************/
  for (i=0;i<10;i++) var[i]=-1;


  /* read data from file */
  /************************/
  fp = io_fopen(name,"r");;
  if(fp)
    {
    res=io_read(fp, (char *)var, sizeof(var));
    if (res >= 0)  print("%d bytes sucessfully read from file %s\n",res,name);
    else           print("can't read from file %s\n",name);
    io_fclose(fp);
    }
  else
    {
    print("cannot open file for read\n");
    }


  /* print results */
  /*****************/
  for (i=0;i<10;i++) print("var[%d] = %d\n",i,var[i]);     





  /************************/
  /* RANDOM FILE ACCESS ! */
  /************************/

  print("\nRANDOM FILE ACCESS\n");


  /* ini variables  (var[0]=0, var[1]=1, ...) */
  /********************************************/
  for (i=0;i<10;i++) var[i]=i;


  /* delete old file if it exists */
  /*******************************/
  fp = io_fopen(name,"c");
  if(fp)
    {
    print("delete old file %s\n",name);
    io_ioctl(fp, IO_DEL, NULL);
    io_fclose(fp);
    }
  else
    {
    print("no old file to delete\n");
    }


  /* write to file */
  /*****************/
  fp = io_fopen(name,"w");
  if(fp)
    {
    pchar=(char *)var;
    for (i=sizeof(var); i>0; i--) 
      {
      if(io_fputc(*pchar++, fp) < 0) break;
      }

    if (i==0) print("sucessful write to file %s\n",name);
    else      print("can't write to file %s\n",name);
    io_fclose(fp);
    }
  else
    {
    print("cannot open file for write\n");
    }



  /* delete variables */
  /********************/
  for (i=0;i<10;i++) var[i]=-1;



  /* random read from file */
  /*************************/
  fp = io_fopen(name,"r");
  if(fp)
    {
    io_fseek(fp, -1*(int)sizeof(int), IO_SEEK_END); // point to last integer
    pchar=(char *)var;
    do
      {
      // read one integer (4 Bytes)
      for (i=0; i<sizeof(int); i++)
         {
         res=io_fgetc(fp);
         if (res >= 0) *pchar++=res;
         }

      io_fseek(fp, -2*(int)sizeof(int), IO_SEEK_CUR); // point to the previous integer
      }
    while(res >= 0);

    if ((int)pchar-(int)var == sizeof(var)) print("sucessful read from file %s\n",name);
    else                                    print("can't read from file %s\n",name);
    io_fclose(fp);
    }
  else
    {
    print("cannot open file for read\n");
    }



  /* print results */
  /*****************/
  for (i=0;i<10;i++) print("var[%d] = %d\n",i,var[i]);     

}
