/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2005 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: disp.c
***                                                            
*** Comments: 
***		disp shell command
***		change display parameters
***
*** Revision History:
***    Date          Version    Author  Changes
***    Oct 24, 2005  5.23		MS		Initial Release		        
***    Oct 25, 2005  5.23		MS		make gamma available for all cams		        
***    Oct 26, 2005  5.23		MS		explain colormode in brackets		        
***
***************************************************************************
*END***********************************************************************/

#include "userdef.h"
#include "register.h"
#include <vcrt.h>
#include "shell.h"
#include <sysvar.h>
#include <vclib.h>
#include <math.h>

int colormode(int mode);
int gamma(int gamma_mul_100);
int period(int disp_period);

extern disp_gopt(), disp_popt(), disp_default();

extern int option;

options dispopt[]=
{
    {'C'   ,(void (*)()) disp_default},
    {'G'   ,(void (*)()) disp_gopt   },
    {'P'   ,(void (*)()) disp_popt   },
    {'\0'  ,(void (*)()) disp_default}
};

char * cmodes[DISP_MODE_MAX+1]=
{
  "IDLE",
  "GREY",
  "RGB",
  "BAYER",
  "BAYERGREY",
  "YCBCR"
};

dispcmd(int argc, char *argv)
{


getopt(&argc,&argv,dispopt,ERR04);

if(getvar(COLOR_MODE) == -1 && option != 1)
{
  pstr("color camera only!\n");
  return -1;
}

if(option == 0)
  {
  pstr("colormode: ");
  if(argc < 2)
	colormode(-1);
  else
	colormode( getnum(argv,ERR02,10) );
  }
else
if(option == 1)
  {
  pstr("gamma: ");
  if(argc < 2)
	gamma(-1);
  else
	gamma( getnum(argv,ERR02,10) );
  }
else
if(option == 2)
  {
  pstr("display period: ");
  if(argc < 2)
	period(-1);
  else
	period( getnum(argv,ERR02,10) );
  }

}


disp_gopt()
{
option = 1;
}

disp_popt()
{
option = 2;
}

disp_default()
{
option = 0;
}


int colormode(int mode)
{

  if(mode == -1 || mode > DISP_MODE_MAX)
	mode = getvar(COLOR_MODE);
  else
	setvar(COLOR_MODE,mode); 

  print("%d [%s]\n",mode,cmodes[mode]);
  return 0;
}

int gamma(int gamma_100)
{
  float gamma = (float)gamma_100/100.0;
  if(gamma_100 == -1 || gamma_100 > 100)
	gamma = (float)getvar(GAMMA)/100.0;
  else
  {
	setvar(GAMMA,gamma_100);
    init_LUT_gamma(gamma);	
  }
  print("%1.2f\n",gamma);
  return 0;
}
  
int period(int disp_period)
{
  if(disp_period == -1)
	disp_period = getvar(DISP_PERIOD);
  else
	setvar(DISP_PERIOD,disp_period); 

  print("%d\n",disp_period);
  return 0;
}

/* EOF */



