/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2005 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: color2.c
***                                                            
*** Comments: 
*** 
***    Demoprogram for the usage of color image variables
***	   program displays a color bar test chart and transforms it into
***    several different color models
***
*** Revision History:
***    Date          Version    Author  Changes
***    Nov 21, 2005  3.01       ME      Initial Version
***
***************************************************************************
*END***********************************************************************/

#define NEW_IMAGE_VAR			/* important for color image variables	*/

#include <sysvar.h>
#include <vcrt.h>
#include <vclib.h>
#include <flib.h>
#include <macros.h>

/**************************************************************************/

void main(void)
{
	image a, b;
	U32 old_start;
	U32 old_mode;

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
	/**/ 	print("Init Color Lib = %d\n", init_licence("C1122334455"));         /**/
    /*******************************************************************************/


	print("display color bar test chart\n");
	rs232rcv();

	ImageAllocate(&a, IMAGE_RGB, 768, 582);			/* allocate RGB color image				*/
	a.dx = 752;										/* set dx to 752						*/


	cset(&a, 0, 0, 128);							/* set screen to medium blue			*/

	a.dx = 256;										/* reduce size of image variable a (dx)	*/
	a.dy = 256;										/* reduce size of image variable a (dy)	*/

	old_start = getvar(DISP_START);					/* save old start address				*/
	old_mode  = getvar(COLOR_MODE);					/* save old color mode					*/

	setvar(DISP_START, (int)a.st);					/* set DISP_START to start of			*/
													/* image variable						*/

	ImageAssignC(&b, a.st, IMAGE_RGB, a.dx, 1, 		/* b is an image with the same features	*/
						a.pitch, a.ccmp1, a.ccmp2); /* as image a but with dy = 1			*/

	setvar(COLOR_MODE, DISP_RGB);					/* set appropriate color display mode	*/

	print("100 %% amplitude, 100 %% saturation\n");
	ColorBar(&a, 255, 255);
	//print("75 %% amplitude, 100 %% saturation\n");
	//ColorBar(&a, 191, 255);
	//print("75 %% amplitude, 75 %% saturation\n");
	//ColorBar(&a, 191, 191);
	//print("100 %% amplitude, 75 %% saturation\n");
	//ColorBar(&a, 255, 191);
	rs232rcv();

	a.type = IMAGE_CBCR444;							/* change type to target color model	*/

	RGB_YCbCr(&a, &a);								/* convert into YCbCr model				*/
	print_image(&b);								/* print one line of image data			*/

	rs232rcv();

	print("normalized:\n");

	a.type = IMAGE_YUVNORM;							/* change type to target color model	*/
	YCbCr_NORM(&a, &a);								/* normalize chrominance				*/
	print_image(&b);								/* print one line of image data			*/


	rs232rcv();

	ImageFree(&a);									/* release memory for color image		*/

	setvar(DISP_START, old_start);					/* restore old start address			*/
	setvar(COLOR_MODE, old_mode);					/* restore old color mode				*/  
}

/* EOF */
