/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2005 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: findcolor.c
***                                                            
*** Comments: 
*** 
***    Demoprogram for detection of a precise color and binarization.
***    Designed for VC2065C / VC4465C.
***
*** Revision History:
***    Date          Version    Author  Changes
***    May 11, 2006  1.00       LB      Initial Version
***    Jan 12, 2007  1.01       LB      Cosmetic changes
***
***************************************************************************
*END***********************************************************************/

#define NEW_IMAGE_VAR			/* important for color image variables	*/

#include <sysvar.h>
#include <vcrt.h>
#include <vclib.h>
#include <flib.h>
#include <macros.h>
#include <math.h>

/************************************************************************************/

void color_classif(U8 *table);
void set_color(U8 *table);

/************************************************************************************/

void main(void)
{
	U8 *table;

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
	/**/ 	print("Init Color Lib = %d\n", init_licence("C1122334455"));         /**/
    /*******************************************************************************/

	vmode(vmOvlLive);

	/* Allocate table for classif */
	table = vcmalloc (256 * 256);

	/* Image processing */
	set_color(table);
	color_classif(table);
	
	/* Free memory */
	vcfree(table);	
  		
}

/************************************************************************************/

void set_color(U8* table)
{
	image ihs, rgb, bayer, ovl;
	int i, j, dx = 768, dy = 582, sum_h = 0, sum_s = 0;
	int Xstart = 10, Ystart = 10, SizeOvl = 64;
	int Hinf, Hsup, Sinf, Ssup;
	float  mean_h, mean_s;
	U32 old_start, old_mode;

	/*********************************************/
	/* Parameters : tolerance Hue and Saturation */
	/*********************************************/
	int tolH = 10;
	int tolS = 50;

	/* Yellow frame on overlay */
	OvlClearAll;
	ImageAssign(&ovl, OvlByteAddr(Xstart,Ystart), SizeOvl, SizeOvl, OvlGetPitch);

	framed(&ovl, 0x80);			/* set bit 7	 */
	set_ovlmask(0xff);
	set_overlay_bit(7,255,255,0);	/* yellow		 */
	print("Please place desired color inside yellow frame\n");
	print("Press any key to start\n");

	rs232rcv();


	/* Allocate images */
	ImageAllocate(&rgb, IMAGE_RGB, dx, dy);
	ImageAllocate(&ihs, IMAGE_IHS, dx, dy);			

	cset(&rgb, 0, 0, 128);

	old_start = getvar(DISP_START);					/* save old start address				*/
	old_mode  = getvar(COLOR_MODE);					/* save old color mode					*/

	setvar(COLOR_MODE, DISP_RGB);
	setvar(DISP_START, (int)rgb.st);

	ImageAssignC(&bayer, getvar(CAPT_START), IMAGE_BAYER,
		getvar(HWIDTH), getvar(VWIDTH), getvar(VPITCH), (U8 *)0, (U8 *)0);

	/* Take picture */			
	tpict();	

	/* Bayer to RGB */
	BayerToRGB(&bayer, &rgb);

	/* Disp RGB image */
	print("Displaying RGB image... (press a key)\n");

	rs232rcv();

	/* RGB to HSI */
	RGB_IHS(&rgb, &ihs);

	/* Calculate mean value of Hue inside yellow frame */
	for(i=Xstart;i<Xstart + SizeOvl;i++)
	{
		for(j=Ystart;j<Ystart + SizeOvl;j++)
		{ 
			sum_h = sum_h + *(ihs.ccmp1 + j*dx + i);
		}
	}

	print("Sum H = %d\n", sum_h);
	mean_h = (float)sum_h / (SizeOvl*SizeOvl);
	print("Mean H = %f\n\n", mean_h);

	/* Calculate mean value of Saturation inside yellow frame */
	for(i=Xstart;i<Xstart + SizeOvl;i++)
	{
		for(j=Ystart;j<Ystart + SizeOvl;j++)
		{ 
			sum_s = sum_s + *(ihs.ccmp2 + j*dx + i);
		}
	}

	print("Sum S = %d\n", sum_s);
	mean_s = (float)sum_s / (SizeOvl*SizeOvl);
	print("Mean S = %f\n\n", mean_s);

	/* Filling table for color classification */
	/* Here all pixels with calculated H value +/- tolH will be black, the rest white */
	Hinf = mean_h - tolH;
	Hsup = mean_h + tolH;
	Sinf = mean_s - tolS;
	Ssup = mean_s + tolS;

	for(i=0;i<256;i++)
	 {                
	  for(j=0;j<256;j++)
	     {
	       if(Hinf<i && i<Hsup && Sinf<j && j<Ssup) {table[256*j + i]=0;}
	       else {table[256*j + i]=255;}    
	     }
	  }

	print("Color saved\n");

	/*Free memory*/
	ImageFree(&ihs);
	ImageFree(&rgb);

	/* Back to normal display */
	setvar(DISP_START, old_start);					/* restore old start address			*/
	setvar(COLOR_MODE, old_mode);					/* restore old color mode				*/  
	vmode(vmLive);

}

/************************************************************************************/

void color_classif(U8 *table)
{
	image ihs, rgb, bayer, classified;
	int dx = 768, dy = 582, val, old_start, old_mode;

	old_start = getvar(DISP_START);					/* save old start address				*/
	old_mode  = getvar(COLOR_MODE);					/* save old color mode					*/

	ImageAssignC(&bayer, getvar(CAPT_START), IMAGE_BAYER,
		getvar(HWIDTH), getvar(VWIDTH), getvar(VPITCH), (U8 *)0, (U8 *)0);

	ImageAllocate(&rgb, IMAGE_RGB, dx, dy);
	ImageAllocate(&ihs, IMAGE_IHS, dx, dy);
	ImageAllocate(&classified, IMAGE_GREY, dx, dy);

	/*Take another picture*/
	print("Ready to take picture... (press a key)\n");
	rs232rcv();
	tpict();

	/* Bayer to RGB */
	BayerToRGB(&bayer, &rgb);

	/* Disp RGB image */
	print("Displaying RGB image... (press a key)\n");
	rs232rcv();

	/* RGB to HSI */
	RGB_IHS(&rgb, &ihs);

	/* Color classification */
	val = color_classify(&ihs, &classified, table);
	print("color_classify returns %d\n\n",val);

	/* Displaying classified image */
	print("Displaying classified image\n");
	setvar(COLOR_MODE, DISP_GREY);
	setvar(DISP_START, (int)classified.st);

	rs232rcv();

	/*Free memory*/
	ImageFree(&classified);
	ImageFree(&ihs);
	ImageFree(&rgb);					   										

	/* Back to normal display */
	setvar(DISP_START, old_start);					/* restore old start address			*/
	setvar(COLOR_MODE, old_mode);					/* restore old color mode				*/  
	vmode(vmLive);	

}

/************************************************************************************/


