/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2005 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: wb.c
***                                                            
*** Comments: 
*** 
***    Shell command of white balance for camera models
***    with hardware LUT (VC2065EC and VC4465C) ONLY
***
***
*** Revision History:
***    Date          Version    Author  Changes
***    Oct 25, 2005  5.23       MS      modified whitebal.c to use in shell
***
***************************************************************************
*END***********************************************************************/

#define NEW_IMAGE_VAR

//#include <stdio.h>
#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>
#include <flib.h>

int wbcmd (int argc, char *argv)
{
  int   red, green, blue; 
  int   bright, min=100, max=180;
  image bayer, ovl;

  if( (getvar(COLOR_MODE) == -1) || (getvar(OVLY_START) == 0))
  {
	print("whitebalance not available!\n");
	return -1;
  }
  
  ScrSetLogPage(ScrGetPhysPage);
  OvlSetLogPage(OvlGetPhysPage);
//  ImageAssign(&bayer, ScrByteAddr(0,0), ScrGetColumns, ScrGetRows, ScrGetPitch);
  ImageAssign(&bayer, ScrByteAddr((((getvar(HWIDTH)-256)/2) & ~1),((getvar(VWIDTH)-256)/2) & ~1), 256, 256, ScrGetPitch);
  bayer.type = IMAGE_BAYER;
  
  ImageAssign(&ovl,   OvlByteAddr((((getvar(HWIDTH)-256)/2) & ~1),((getvar(VWIDTH)-256)/2) & ~1), 256, 256, OvlGetPitch);

  framed(&ovl, 0x80);			/* set bit 7	 */
  set_ovlmask(0xff);
  set_overlay_bit(7,255,255,0);	/* yellow		 */
  print("Please place white object inside yellow frame\n");
  print("and select a brightness between %d and %d\n", min, max);
  print("Press any key for start and end\n");

  getchar();
 
  // IMPORTANT: select standard values before calculate white balance !!!
  init_color_lut(1024, 1024, 1024);
  
  // get new values for white balance
  do
  {
    tpict();  
    bright = WhiteBalanceValues(&bayer, &red, &green, &blue);
    print("Max = %3d  [%d...%d]\n", bright, min, max);    
  }
  while (!kbhit());
  
  // program new values for white balance
  init_color_lut(red, green, blue);

  print("\n");
  print("Red   = %3d\n", red);
  print("Green = %3d\n", green);
  print("Blue  = %3d\n", blue);
  print("\n");
  
  return 0;
}

/* EOF */
