/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2006 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: edge1.c
***                                                            
*** Comments: 
*** 
***    edge detector demo, calls edge()
***
*** Revision History:
***    Date          Version    Author  Changes
***    Oct 26, 2006  3.02       ME      Initial Version
***
***************************************************************************
*END***********************************************************************/

#define NEW_IMAGE_VAR
#include <vcrt.h>
#include <vclib.h>
#include <flib.h>
#include <macros.h>
#include <stdlib.h>
#include <register.h>
#include <sysvar.h>
#include <math.h>

/**************************************************************************/

void main(void)
{
  image capt, grad, dst, out, ovl;

  I32 response, ms;
  I32 type  		= 4;	/* set default values for edge parameters	*/
  I32 sigma0 		= 0;
  I32 sigma1 		= 6;
  I32 BinMode 		= 2;
  I32 MinContr    	= 2;
  I32 hysteresis 	= 1;
  I32 thresh      	= 5;
  I32 output      	= 0;
  float fsigma = (float)sigma0 + ((float) sigma1) / 10.0;


  /////////////////////////////////////////////////////////////////////////////////////
  // initialize licence key 
  /////////////////////////////////////////////////////////////////////////////////////

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
	/**/ 	print("Init Extension Lib = %d\n", init_licence("E1122334455"));     /**/
    /*******************************************************************************/

  /////////////////////////////////////////////////////////////////////////////////////
  // assign capture and overlay image variable 
  /////////////////////////////////////////////////////////////////////////////////////

  ImageAssign(&capt, getvar(DISP_START), getvar(HWIDTH),  getvar(VWIDTH),  getvar(VPITCH));
  ImageAssign(&ovl,  getvar(OVLY_START), getvar(DHWIDTH), getvar(DVWIDTH), getvar(VPITCH));
  
  /////////////////////////////////////////////////////////////////////////////////////
  // allocate memory for gradient image, grey image and output image 
  /////////////////////////////////////////////////////////////////////////////////////

  ImageAllocate(&grad, IMAGE_VECTOR, getvar(HWIDTH), getvar(VWIDTH));
  ImageAllocate(&dst,  IMAGE_GREY,   getvar(HWIDTH), getvar(VWIDTH));
  ImageAllocate(&out,  IMAGE_GREY,   getvar(VPITCH), getvar(DVWIDTH));
  
  /////////////////////////////////////////////////////////////////////////////////////
  // clear output image and overlay 
  /////////////////////////////////////////////////////////////////////////////////////

  set(&out, 0);	/* clear full display image	*/
  set(&ovl, 0);	/* clear full overlay image	*/
  
  /////////////////////////////////////////////////////////////////////////////////////
  // resize out to capture size redirect display to image variable out 
  /////////////////////////////////////////////////////////////////////////////////////

  ImageAssign(&out, out.st, getvar(HWIDTH),   getvar(VWIDTH),   getvar(VPITCH));

  setvar(DISP_START, (I32)out.st);
  ScrSetLogPage((I32)out.st);


  /////////////////////////////////////////////////////////////////////////////////////
  // initialize tables for edge() 
  /////////////////////////////////////////////////////////////////////////////////////

  init_gradient_2x2();
  init_gradient_3x3();
  
  
  print("edge detection demo vs. 1.0:\n");
  print("Vision Components 2006\n\n");
  
 
  print("the following commands are available:\n\n");
  print("change filter type .................. f\n");
  print("change sigma ........................ s\n");
  print("binarization mode ................... b\n");
  print("minimum contrast .................... c\n");
  print("threshold percentage ................ t\n");
  print("output image type ................... o\n");
  
  
    do
      {
      tpict(); 
  
      ms = getvar(MSEC);
  	  if(output == 0)
        edge(&capt, &dst, type, fsigma, BinMode*hysteresis, MinContr, (float)thresh / 100.0, 255);
  	  else
        edge(&capt, &grad, type, fsigma, BinMode*hysteresis, MinContr, (float)thresh / 100.0, 255);
      ms = getvar(MSEC)-ms; if(ms<0) ms+=1000;
  
      print("%3dms\r", ms);
  
  	  if(output == 0)
  	    copy(&dst, &out);
  	  else
  	    {
        display_directions(&grad, 20, 0, 0);
  	    }
  
  //    if (1)                               
      if (kbhit())                     
        {
        response=ReadKeyboard(1);
  
        /* filter type	*/
        if (response == 'f')
          {
          ChangeValue(&type,      0,  5, 1, "Select Filter Type");
  
          switch (type)
            {
            case 0 : 								
    	      print("no filter\n");
    	      break;
            case 1 : 								
  	        print("sobel filter\n");
              break;
            case 2 :								
    	      print("moving average filter\n");
              break;
            case 3 :                  		   
  	          print("recursive gauss filter\n");
              break;
            case 4 :                  		   
    	      print("non-recursive gauss filter\n");
              break;
            case 5 :
  	        print("recursive ISEF\n");
              break;
            }
  
          }
  
        /* sigma	*/
        if (response == 's')
          {
          ChangeValue(&sigma0, 0,  5, 1, "Select Sigma, Leading Digit");
          ChangeValue(&sigma1, 0,  9, 1, "Select Sigma, Second Digit");
  		  fsigma = (float)sigma0 + ((float) sigma1) / 10.0;
  
          print("sigma: %3.1f\n", fsigma);
          }
  
        /* binarization mode	*/
        if (response == 'b')      
          {
          ChangeValue(&BinMode, 0,  2, 1, "Select Binarization Mode");
  
          switch (BinMode)
            {
            case 0 : 								
    	      print("no binarization\n");
    	      break;
            case 1 : 								
  	        print("vc mode binarization\n");
              break;
            case 2 :								
    	      print("canny mode binarization\n");
              break;
            }
  
          ChangeValue(&hysteresis, -1,  1, 2, "Hysteresis Threshold -1 = yes");
   
   		  print("hysteresis: %d\n", hysteresis);
          if(hysteresis == 1)
            print("no hysteresis threshold\n");
          else   
            print("hysteresis threshold\n");
          }
  
        if (response == 'c')      
          {
          ChangeValue(&MinContr, 1, 255, 1, "Select Minimum Contrast");
          print("minimum contrast: %d\n", MinContr); 
          }
  
        if (response == 't')      
          {
          ChangeValue(&thresh, 1, 99, 1, "Select Threshold in Percent");
          }
  
        if (response == 'o')      
          {
          ChangeValue(&output, 0, 1, 1, "Select Output Image Variable");
  		  if(output==0)
		    {
            set_ovlmask(0);
  		    print("type = IMAGE_GREY\n");
			}
  		  else
		    {
            set_ovlmask(0xff);
  		    print("type = IMAGE_VECTOR\n");
			}
          }
  
        }
      while(kbhit()) ReadKeyboard(1);
      }
    while(response != KEY_ESC);
  
  
  setvar(DISP_START, getvar(CAPT_START));
  ScrSetLogPage((I32)capt.st);

  set_ovlmask(0);
  
  ImageFree(&out);
  ImageFree(&dst);
  ImageFree(&grad);
  
  deinit_gradient_2x2();
  deinit_gradient_3x3();
  
}




/* EOF */
