/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2009 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: fftmain.c
***                                                            
*** Comments: 
*** 
***    test and demo program for FFT
***
*** Revision History:
***    Date          Version    Author  Changes
***    Mar 19, 2009  3.06       KS, ME  first version 
***    Apr 07, 2009  3.06       ME      first release
***
***************************************************************************
*END***********************************************************************/

#define NEW_IMAGE_VAR
#include <vcrt.h>
#include <vclib.h>
#include <flib.h>
#include <macros.h>
#include <stdlib.h>
#include <register.h>
#include <sysvar.h>
#include <math.h>
#include <mathf.h>

/**************************************************************************/

void main(void)
{
  I32     border, x_IMG, x_FFT, x_IFFT, y, dx, dy, pitch, c, Mean, Scale, Error;
  I32     ms, DisLog, DelFreq, AngHist;
  image   Source, SrcCplx, InvCplx, FFT, IFFT, Polar;
  U32     AngleHisto[180], i;
  I32     AngMax, AngI;
  float   a[2][2], t[2];
  
  /////////////////////////////////////////////////////////////////////////////////
  // init all VC library licences
  /////////////////////////////////////////////////////////////////////////////////

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
	/**/ 	print("Init Extension Lib = %d\n", init_licence("E1122334455"));     /**/
    /*******************************************************************************/

  /////////////////////////////////////////////////////////////////////////////////
  // activate VC Keypad inputs
  /////////////////////////////////////////////////////////////////////////////////

  INTERFACE_MODE(SERIAL);
  

  /////////////////////////////////////////////////////////////////////////////////
  // ini variables 
  /////////////////////////////////////////////////////////////////////////////////
  
  // ini working page
  ScrSetLogPage(ScrGetPhysPage);
  OvlSetLogPage(OvlGetPhysPage);
  
  // ini overlay
  OvlClearAll;
  
  // Define Overlay Color
  set_overlay_bit(7,  65,  90, 255); // 128 blue

  // switch to overlay mode
  set_ovlmask(255);
  vmode(vmOvlLive);

  // log. FFT display mode
  DisLog         =   0; 
  DelFreq        =   0;
  AngHist        =   0;

  // ini Source and FFT areas
  dx             = 128;
  dy             = 128;

  border         = (ScrGetColumns - 3 * dx) / 4;
  x_IMG          = border + 0 * (border + dx);  
  x_FFT          = border + 1 * (border + dx);  
  x_IFFT         = border + 2 * (border + dx);  

  y              = (ScrGetRows      - dy) / 2;  
  pitch          =  ScrGetPitch;

  // Define image variable Source
  ImageAssign(&Source, OvlByteAddr(x_IMG, y), dx, dy, pitch);  
  framed(&Source, 128);
  ImageAssign(&Source, ScrByteAddr(x_IMG, y), dx, dy, pitch);  

  // Define image variable FFT
  ImageAssign(&FFT, OvlByteAddr(x_FFT, y), dy, dx, pitch);
  framed(&FFT, 128);
  ImageAssign(&FFT, ScrByteAddr(x_FFT, y), dy, dx, pitch);

  // Define image variable IFFT
  ImageAssign(&IFFT, OvlByteAddr(x_IFFT, y), dx, dy, pitch);
  framed(&IFFT, 128);
  ImageAssign(&IFFT, ScrByteAddr(x_IFFT, y), dx, dy, pitch);


  
  /////////////////////////////////////////////////////////////////////////////////
  // allocate memory and initialise FFT tables
  /////////////////////////////////////////////////////////////////////////////////

  ImageAllocate(&SrcCplx, IMAGE_CMPLX16, dy, dx);
  ImageAllocate(&InvCplx, IMAGE_CMPLX16, dx, dy);
  ImageAllocate(&Polar,   IMAGE_CMPLX16, dx/2, dy/2);

  ms = getvar(MSEC);
  Error = CalcPolarCoordinates (&Polar, 180);	/* calculate polar coordinates with 180 degrees	*/
  ms = getvar(MSEC)-ms;
  if(ms<0) ms+=1000;

  printf("\nCalcPolarCoordinates [%dx%d]  %3d msec Error = %d\n", dx/2, dy/2, ms, Error);

  /////////////////////////////////////////////////////////////////////////////////
  // do FFT calculateion
  /////////////////////////////////////////////////////////////////////////////////
  
  printf("Press 'l' live mode\n");
  printf("Press 'g' toggle display logarithmus\n");
  printf("Press 'w' write images\n");
  printf("Press 'd' delete frequency\n");
  printf("Press 'a' angular histogram\n");
  printf("Press any key to start (ESC=quit)\n");
  

  do
  {
    // generate image
    tpict();

    ///////////////////////////////////////////////////////////////////////////////////////
    // FFT
    ///////////////////////////////////////////////////////////////////////////////////////

    ms = getvar(MSEC);

    Error = vc_fft(&Source, &SrcCplx, &Mean, &Scale);

    ms = getvar(MSEC)-ms;
    if(ms<0) ms+=1000;

    printf("\nGenCplxImg         %dx%d  Mean=%3d  Scale=%2d: %3d msec Error = %d\n", dx, dy, Mean, Scale, ms, Error);

	if(DelFreq)
	  {
	  Error = DeleteFreq (&SrcCplx, &Polar, 20, 200);
      printf("\nDeleteFreq Error = %d\n", Error);
	  }

	if(AngHist)
	  {
	  Error = CalcAngleHisto (&SrcCplx, &Polar, 0, 5, 1000, AngleHisto, 180); 
      printf("\nCalcAngleHisto Error = %d\n", Error);
	  AngMax = 0; AngI = 0;
	  for(i=0; i<180; i++)
	    {
		if(AngleHisto[i] > AngMax)
		  {
		   AngMax = AngleHisto[i];
		   AngI   = i;
		  }  
		}
	  printf("angle: %d deg\n", AngI);

      calc_rotation_matrix((float)-AngI, (float)Source.dx/2, (float)Source.dy/2, a, t);
	  affine_image_transform(&Source, &IFFT, a, t, 0);
	  }

    DisplayFFT(&SrcCplx, &FFT, DisLog);


    ///////////////////////////////////////////////////////////////////////////////////////
    // Inverse FFT
    ///////////////////////////////////////////////////////////////////////////////////////

    ms = getvar(MSEC);

    Error = vc_ifft(&SrcCplx, &InvCplx, &Mean, &Scale);

    ms = getvar(MSEC)-ms;
    if(ms<0) ms+=1000;

    printf("GenCplxImg Inverse %dx%d  Mean=%3d  Scale=%2d: %3d msec Error = %d\n", dx, dy, Mean, Scale, ms, Error);

	if(AngHist==0)
      DisplayInvFFT(&InvCplx, &IFFT, Mean, Scale);


    // delete char buffer
    while(kbhit()) getchar();
    
    c=getchar();

    switch(c)
    {
        case 'g': 
                  if (DisLog) DisLog=0; 
                  else        DisLog=1; //(I32) (255.0 / log((255*255*dx)>>15));
                  printf("DisLog=%d\n", DisLog);
                  break;
        case 'l': 
                  vmode(vmOvlLive); 
                  getchar();
                  break;
        case 'w': 
    			  fwrite_image("md:/source.bmp", &Source);
    			  fwrite_image("md:/FFT.bmp", &FFT);
    			  fwrite_image("md:/IFFT.bmp", &IFFT);
                  break;
        case 'd': 
                  if (DelFreq) DelFreq=0; 
                  else         DelFreq=1; 
                  printf("DelFreq=%d\n", DelFreq);
                  break;
        case 'a': 
                  if (AngHist) AngHist=0; 
                  else         AngHist=1; 
                  printf("angular histogram = %d\n", AngHist);
                  break;
    }
  }
  while(c != 0x1b);  // ESC


  ///////////////////////////////////////////////////////////////////////////////////////
  // release allocation
  ///////////////////////////////////////////////////////////////////////////////////////

  ImageFree(&InvCplx);
  ImageFree(&SrcCplx);
}



/* EOF */
