/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2008 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: hough3.c
***                                                            
*** Comments: 
*** 
***    example for use of hough transform with edge image
***    and line extraction
***
*** Revision History:
***    Date          Version    Author  Changes
***    Apr 15, 2008  3.02       ME      initial version
***
***************************************************************************
*END***********************************************************************/

#define NEW_IMAGE_VAR
#include <vcrt.h>
#include <vclib.h>
#include <flib.h>
#include <macros.h>
#include <stdlib.h>
#include <register.h>
#include <sysvar.h>
#include <math.h>

#include "\ti\vclib\licence.h"


#define BINAR_VALUE1 	(200)
#define BINAR_VALUE2 	(64)
#define HOUGH_ANGLES    (128)


void main(void)
{
  image src, dst, Hough32;
  HoughControl Control;
  I32 ms, i, j, nr, err;
  HLine *p;
  I32 *xy;

  /////////////////////////////////////////////////////////////////////////////////////
  // initialize licence key 
  /////////////////////////////////////////////////////////////////////////////////////

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
	/**/ 	print("Init Extension Lib = %d\n", init_licence("E1122334455"));     /**/
    /*******************************************************************************/

  /////////////////////////////////////////////////////////////////////////////////////
  // set some image variables and hough defaults 
  /////////////////////////////////////////////////////////////////////////////////////

  ImageAssign(&src, getvar(DISP_START), getvar(HWIDTH), getvar(VWIDTH), getvar(VPITCH));

  ImageAllocate(&dst,     IMAGE_VECTOR,   getvar(HWIDTH),    getvar(VWIDTH));
  ImageAllocate(&Hough32, IMAGE_GREY32, HoughCalcDx(&src),   HOUGH_ANGLES);

  HoughDefaults(&Control);
  Control.LineNumber  = 5000;
  Control.MinPix = 20 * BINAR_VALUE1;	/* threshold for number of pixels in line	*/

  /////////////////////////////////////////////////////////////////////////////////////
  // initialize gradient and hough system tables 
  /////////////////////////////////////////////////////////////////////////////////////

  init_gradient_3x3();
  init_gradient_2x2();

  HoughInit();
  HoughInit2(&Control);

  xy = vcmalloc(20000);	/* hope that this is sufficient	*/

  /////////////////////////////////////////////////////////////////////////////////////
  // take a picture 
  /////////////////////////////////////////////////////////////////////////////////////

  tpict();

  /////////////////////////////////////////////////////////////////////////////////////
  // calculate edge image
  /////////////////////////////////////////////////////////////////////////////////////

  edge_fast(&src, &dst, BINAR_VALUE1);				/* edge detection				*/

  get_component(&dst, &src, 0);						/* copy grey image to source	*/
  getchar();										/* for visualization			*/ 

  /////////////////////////////////////////////////////////////////////////////////////
  // now Hough transform
  /////////////////////////////////////////////////////////////////////////////////////

  ms = getvar(MSEC);
  HoughTransform(&dst, &Hough32, &Control);
  ms = getvar(MSEC)-ms; if(ms<0) ms+=1000;
  print("Hough        [%3dx%3d] = %3dms\n", src.dx, src.dy, ms);


  ///////////////////////////////////////////////////////////////////////////////////////
  // find lines in Hough transform
  ///////////////////////////////////////////////////////////////////////////////////////

  ms  = getvar(MSEC);
  err = FindHoughLine(&Hough32, &dst, &Control);
  ms  = getvar(MSEC)-ms; if(ms<0) ms+=1000;

  if(err)
    {
    print("error # %d\n", err);
	print("hough line overrun:   %d\n", ERR_HOUGH0);
	print("hough max iterations: %d\n", ERR_HOUGH1);
	}
  else
    {
  ///////////////////////////////////////////////////////////////////////////////////////
  // calculate number of lines found
  ///////////////////////////////////////////////////////////////////////////////////////
				
    nr = HoughRank(Control.maxptr, HL_VALUE, 0);
    print("Find Lines   [%7d] = %3dms\n", nr, ms);


  ///////////////////////////////////////////////////////////////////////////////////////
  // output lines and draw into screen buffer
  ///////////////////////////////////////////////////////////////////////////////////////

    p = Control.maxptr;
    i = 0;

    while(p != NULL)
      {
      print("i: %03d cx: %6e cy: %6e b: %6e strength: %d\n", i, p->cx, p->cy, p->b, p->strength);
      draw_lined(&src, p->cx, p->cy, p->b, 255);
      nr=GetHoughPixels(&dst, p, &Control, xy);
	  if(nr>0)
	    for(j=0; j<nr; j++)
		  printf("j: %d x: %d y: %d\n", j, xy[2*j], xy[2*j+1]);

	  getchar();
      p = p->next;
      i++;
      }
    }

  ///////////////////////////////////////////////////////////////////////////////////////
  // deinitialize and release memory
  ///////////////////////////////////////////////////////////////////////////////////////

  vcfree(xy);

  deinit_gradient_3x3();
  deinit_gradient_2x2();

  HoughDeinit();
  HoughDeinit2(&Control);

  ImageFree(&dst);
  ImageFree(&Hough32);

}

/* EOF */
