/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2008 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: HoughC.c
***                                                            
*** Comments: 
*** 
***    demo program for circular Hough Transform
***
*** Revision History:
***    Date          Version    Author  Changes
***    Jun 19, 2008  3.02       KS  initial version
***    Aug 21, 2008  3.02       ME  modified for VCLIB
***    Sep 08, 2008  3.02       ME  first release
***
***************************************************************************
*END***********************************************************************/

#define NEW_IMAGE_VAR

#include <vcrt.h>
#include <flib.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

#include <math.h>
#include <mathf.h>
#include <string.h>


#define TRANRED    1
#define TRANYEL    3
#define COLGREY    4
#define COLRED     8
#define COLBLACK  16
#define COLWHITE  32
#define COLGREEN  64
#define COLBLUE  128


void main(void)

{   
  I32        ms, sec, Error;
  I32        CentreX, CentreY, Radius, Strength;
  image      Src, Ovl;
  image      ImgVec, Hough32;
  char       response;
  HCirclePar  HCP[1];
  HCircle    Circle[1];


  /////////////////////////////////////////////////////////////////////////////////////
  // initialize licence key 
  /////////////////////////////////////////////////////////////////////////////////////

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
	/**/ 	print("Init Extension Lib = %d\n", init_licence("E1122334455"));     /**/
    /*******************************************************************************/


  /////////////////////////////////////////////////////////////////////////////////////
  // set some image variables
  /////////////////////////////////////////////////////////////////////////////////////

  ImageAssign(&Src, getvar(CAPT_START), ScrGetColumns, ScrGetRows, ScrGetPitch);
  ImageAssign(&Ovl, getvar(OVLY_START), OvlGetColumns, OvlGetRows, OvlGetPitch);

  ImageAllocate(&ImgVec, IMAGE_VECTOR, ScrGetColumns, ScrGetRows);
  ImageAllocate(&Hough32, IMAGE_GREY32, ScrGetColumns, ScrGetRows);



  /////////////////////////////////////////////////////////////////////////////////////
  // ini display colors
  /////////////////////////////////////////////////////////////////////////////////////

  set_translucent(1, 250,   0,   0); /* TRANRED    1 red translucent 		*/
  set_translucent(2, 255, 255,   0); /* TRANYEL    3 yellow translucent 	*/
  set_overlay_bit(2, 210, 210, 210); /* COLGREY    4 grey 					*/
  set_overlay_bit(3, 250,   0,   0); /* COLRED     8 red 					*/
  set_overlay_bit(4,  70,  70,  70); /* COLBLACK  16 black 					*/
  set_overlay_bit(5, 250, 250, 250); /* COLWHITE  32 white 					*/
  set_overlay_bit(6,   0, 200,   0); /* COLGREEN  64 green 					*/
  set_overlay_bit(7,  65,  90, 255); /* COLBLUE  128 blue 					*/

  set_ovlmask(255);
  set(&Ovl, 0);
  vmode(vmOvlLive);

  /////////////////////////////////////////////////////////////////////////////////////
  // init gradient tables
  /////////////////////////////////////////////////////////////////////////////////////

  Error = 0;
  ms  = getvar(MSEC); sec = getvar(SEC);

  if (Error == 0) Error = init_gradient_2x2();
  if (Error == 0) Error = init_gradient_3x3();

  ms  = 1000 * (getvar(SEC) - sec) + (getvar(MSEC) - ms);
  printf("Init Gradient [%3dx%3d] = %5dms\n", Src.dx, Src.dy, ms);

  /////////////////////////////////////////////////////////////////////////////////////
  // now init hough circle
  /////////////////////////////////////////////////////////////////////////////////////

  ms  = getvar(MSEC); sec = getvar(SEC);


  if (Error == 0) Error = HoughCDefaults(HCP);		/* call before InitHoughCircle()	*/
  if (Error == 0) Error = InitHoughCircle(HCP);

  ms  = 1000 * (getvar(SEC) - sec) + (getvar(MSEC) - ms);
  printf("Init Hough    [%3dx%3d] = %5dms\n", Src.dx, Src.dy, ms);


  /////////////////////////////////////////////////////////////////////////////////////
  // start loop 
  /////////////////////////////////////////////////////////////////////////////////////
  
  do
  {
    print("\n");
    // print("SysMemFree             = %6d\n", sysmemfree());

    set(&Ovl, 0);
    
  
    /////////////////////////////////////////////////////////////////////////////////////
    // take a picture 
    /////////////////////////////////////////////////////////////////////////////////////

    tpict();
  

    /////////////////////////////////////////////////////////////////////////////////////
    // calculate edge image
    /////////////////////////////////////////////////////////////////////////////////////

    ms  = getvar(MSEC); sec = getvar(SEC);

    edge_fast(&Src, &ImgVec, 128); 

    ms  = 1000 * (getvar(SEC) - sec) + (getvar(MSEC) - ms);
    print("edge_fast    [%3dx%3d] = %5dms\n", Src.dx, Src.dy, ms);


    /////////////////////////////////////////////////////////////////////////////////////
    // now do hough circle transform
    /////////////////////////////////////////////////////////////////////////////////////

    ms  = getvar(MSEC); sec = getvar(SEC);

    if (Error == 0) Error = HoughCircleTransform(&ImgVec, &Hough32, HCP);

    ms  = 1000 * (getvar(SEC) - sec) + (getvar(MSEC) - ms);
    print("Hough Circle [%7d] = %5dms\n", Error, ms);


    ///////////////////////////////////////////////////////////////////////////////////////
    // ready
    ///////////////////////////////////////////////////////////////////////////////////////

    print("\n");
    print("\n");
    print("Press space to start HoughCircle\n");
    print("Press  'q'  to quit program\n");
    print("Press  'l'  to display live image\n");
    print("Press  'e'  to display edge image\n");
    print("Press  'h'  to display hough image\n");
    print("Press  'o'  to delete overlay\n");
	print("Press  'b'  to select bestcircle\n");
    print("Press  's'  show circles\n");
    print("\n");

  // find first circle
  response = 's';

  do
  {
    if (response == 'l') {vmode(vmOvlLive);}
    if (response == 'o') {set(&Ovl, 0);}
    if (response == 'e') {tpict(); get_component(&ImgVec, &Src, 0);}
    if (response == 'h') {I32 ix, iy; tpict(); if(Error == 0) {ImgConvert_I32_U8(&Hough32, &Src); print("Max Hough = %3d\n", FindMaxImage32(&Hough32, &ix, &iy));}}

    if (response == 'b') {ChangeValue(&HCP->bestfit,   0, 1, 1, "BestCircle approximation 1=yes");}

    if (response == 's')
    {
      ms  = getvar(MSEC); sec = getvar(SEC);

      if(Error == 0) 
      {
		printf("Error: %d\n", FindHoughCircle (&Hough32, HCP, Circle));

        CentreX   = Circle->x0;
        CentreY   = Circle->y0;
        Radius    = Circle->r;
        Strength  = Circle->strength; // 0 ..1024

        if(Radius) draw_circled(&Ovl, CentreX, CentreY, Radius, COLRED);
        MarkCross(&Ovl, CentreX, CentreY, COLGREEN, 10);
      }

      ms  = 1000 * (getvar(SEC) - sec) + (getvar(MSEC) - ms);
      print("Find Circle  [%3dx%3d] = %5dms   ->  X=%3d  Y=%3d  Rad=%3d  Strength=%6d\n", Src.dx, Src.dy, ms, CentreX, CentreY, Radius, Strength);
    }


    while(rbready()) rs232rcv();
    response = rs232rcv();
  }
  while( (response != 'q') && (response != ' ') );
  
  }
  while(response != 'q');

  ///////////////////////////////////////////////////////////////////////////////////////
  // deinitialize and release memory
  ///////////////////////////////////////////////////////////////////////////////////////

  ms  = getvar(MSEC); sec = getvar(SEC);

  DeinitHoughCircle(HCP);

  ms  = 1000 * (getvar(SEC) - sec) + (getvar(MSEC) - ms);
  print("Deinit Hough [%3dx%3d] = %5dms\n", Src.dx, Src.dy, ms);

  deinit_gradient_3x3();
  deinit_gradient_2x2();

  ImageFree(&Hough32);
  ImageFree(&ImgVec);

  vmode(vmLive);
}



/* EOF */
