/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2009 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: linescan.c
***                                                            
*** Comments: 
*** 
***    linescan demo for VC4002L linescan camera
***
*** Revision History:
***    Date          Version    Author  Changes
***    Aug 19, 2009  3.02       ME      Initial Version
***
***************************************************************************
*END***********************************************************************/

#define NEW_IMAGE_VAR
#include <vcrt.h>
#include <vclib.h>
#include <flib.h>
#include <macros.h>
#include <stdlib.h>
#include <register.h>
#include <sysvar.h>
#include <math.h>
#include <mathf.h>

#define RLCSIZE     0x200000		/* 2.0 MBytes for RLC					*/



main()
{
  image capt;
  image src1, src2;
  I8 *offs;
  U16 *shade;

  I32 response=0, ms, x=-4;
  I32 i, dx, dy;

  U16 *rlc, *slc;
  I32 nobj;
  ftr *features;

  float t1, t2;


  /////////////////////////////////////////////////////////////////////////////////////
  // initialize licence key 
  /////////////////////////////////////////////////////////////////////////////////////

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
	/**/ 	print("Init Extension Lib = %d\n", init_licence("E1122334455"));     /**/
    /*******************************************************************************/

  /////////////////////////////////////////////////////////////////////////////////////
  // assign capture and overlay image variable 
  /////////////////////////////////////////////////////////////////////////////////////

  dx = getvar(HWIDTH);
  dy = getvar(VWIDTH);

  printf("dx = %d dy = %d\n", dx, dy);

  ImageAssign(&capt, getvar(DISP_START), dx,  dy,  getvar(VPITCH));
  
  /////////////////////////////////////////////////////////////////////////////////////
  // allocate memory for gradient image, grey image and output image 
  /////////////////////////////////////////////////////////////////////////////////////

  rlc      = rlcmalloc(RLCSIZE);					/* main RLC buffer				*/
  offs     = byte_malloc(dx);						/* pixel offset					*/
  shade	   = byte_malloc(dx * sizeof(U16));			/* shading table				*/

  ImageAllocate(&src1, IMAGE_GREY, getvar(HWIDTH), getvar(VWIDTH));
  ImageAllocate(&src2, IMAGE_GREY, getvar(HWIDTH), getvar(VWIDTH));
  
  /////////////////////////////////////////////////////////////////////////////////////
  // initialize offset and shading tables 
  /////////////////////////////////////////////////////////////////////////////////////

  printf("linescan demo vs. 1.0:\n");
  printf("Vision Components 2009\n\n");


  /* example for unity shading, will be overwritten	*/

  for(i=0; i<dx; i++)
    {
	offs[i]  = 0;		/* range (-128, +127)	*/
	shade[i] = 256;		/* 256 <- factor 1.0	*/
	}
  
  printf("line calibration, place white object before camera\n"); rs232rcv();

  t1 = (float)shutter(200);		/* two different shutter values	*/
  tpict();
  copy(&capt, &src1);

  t2 = (float)shutter(300);
  tpict();
  copy(&capt, &src2);

  line_calibrate(&src1, &src2, t1, t2, offs, shade);
  
//  for(i=0; i<dx; i++) printf("i = %d offset = %03d shading = %08x\n", i, offs[i], shade[i]);
     
  printf("ready\n"); rs232rcv();
  shutter(200);   
 
    do
      {
      if (kbhit())  response=ReadKeyboard(1);

      tpict(); 
  
      ms = getvar(MSEC);

      shading_correct(&capt, &capt, offs, shade);				/* shading correction		*/
//	  printf("after shading\n"); rs232rcv();
      line_IIR(&capt, &capt, 256, 128, 256, 128);				/* recursive line filter	*/
//	  printf("after line IIR\n"); rs232rcv();

      if((slc=rlcmk(&capt, 128+x, rlc, RLCSIZE)) == NULL)		/* run-length code			*/
        {
        printf("RLC overrun\n");
        goto endloop;
        }

//      rlcout(&capt, rlc, 0, 255);							   	/* optional display			*/


      if(sgmt(rlc, slc)==NULL)                 					/* object labelling 		*/
       {
       printf("object number overrun\n");
       goto endloop;
       }

endloop:
      ms = getvar(MSEC)-ms; if(ms<0) ms+=1000;
  
      printf("%3dms\r", ms);
 
  	 }
    while(response != KEY_ESC);
   
    ImageFree(&src1);
    ImageFree(&src2);

  	byte_free(shade);
  	byte_free(offs);
    rlcfree(rlc);
  
  
}






/* EOF */
