/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2006 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: moments.c
***                                                            
*** Comments: 
*** 
***    moments calculation demo for labelled RLC
***
*** Revision History:
***    Date          Version    Author  Changes
***    Jan 17, 2005  3.01       ME      initial version
***
***************************************************************************
*END***********************************************************************/

#define NEW_IMAGE_VAR

#include <vcrt.h>
#include <flib.h>
#include <vclib.h>
#include <stdlib.h>
#include <register.h>
#include <sysvar.h>
#include <math.h>

#define LENGTH	0x80000		/* maximum length for RLC		*/
#define MAXOBJ	100			/* maximum number of objects	*/

/**************************************************************************/

void main(void)
{
	image a;
	int i, nobj, t;
	U16 *rlc;
	U16 *next;
	moment mx[MAXOBJ];
	moment *ma;

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
	/**/ 	print("Init Extension Lib = %d\n", init_licence("E1122334455"));     /**/
    /*******************************************************************************/


	/* set image variable to size of screen	*/

	a.st    = (U8 *)getvar(CAPT_START);
	a.dx    = getvar(HWIDTH);
	a.dy    = getvar(VWIDTH);
	a.pitch = getvar(VPITCH);


	/* allocate memory for RLC				*/

	rlc=rlcmalloc(LENGTH);


	/* print message						*/

	print("moment calculation demo\n");


	/* take picture							*/

	tpict();


	/* main program								*/

	if((next=rlcmk(&a, 128, rlc, LENGTH)) != 0)		/* create RLC			*/
	  {
	  rlcout(&a, rlc, 0, 255);						/* output binary image	*/
	  }
	else
	  {
	  print("error creating RLC. ABORT!\n");
	  goto exit_pgm;
	  }


	rlcout(&a, rlc, 40, 160);           			/* optional rlc display */


	/* object labelling for area filter			*/

	next=sgmt(rlc,next);                  			/* object labelling     */


	/* area filter								*/

	rlc_arf(rlc, rlc, 20);							/* area filter			*/
													/* min object size = 20	*/


	/* second object labelling					*/

	next=sgmt(rlc,next);                  			/* object labelling again*/



	/* now moment calculation					*/

	t=getvar(MSEC);									/* time measurement		 */


	nobj = rlc_moments(rlc, mx, MAXOBJ);

	t  = getvar(MSEC) - t;
	t  = (t<0) ? t+1000 : t;						/* time measurement		 */

	print("elapsed time: %d\n", t);
	print("number of objects: %d\n", nobj);


	/* print results							*/

	ma = mx;

	for(i=0; i<nobj; i++)
	  {
	  print("area: %6.2f c_x: %6.2f c_y: %6.2f\n", BN_tof(&ma->mu00), mom_calc_cgx(ma), mom_calc_cgy(ma));
	  print("angle: %6.2f ecc: %6.2f\n", mom_calc_angle(ma), mom_calc_ecc(ma));
	  print("phi1: %1.4E phi2: %1.4E\n", mom_calc_phi1(ma), mom_calc_phi2(ma)); 
	  print("------------------------------------------------------\n");
	  ma++;
	  }


	exit_pgm:
	rlcfree(rlc);										/* release RLC memory	*/
}


/* EOF */
