/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2006 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: nhls.c
***                                                            
*** Comments: 
*** 
***    demoprogram: binary image processing
***    RLC QIN and number of holes property
***
*** Revision History:
***    Date          Version    Author  Changes
***    Jan 17, 2006  3.01       ME      Initial Version
***
***************************************************************************
*END***********************************************************************/

#define NEW_IMAGE_VAR

#include <vcrt.h>
#include <vclib.h>
#include <stdlib.h>
#include <register.h>
#include <sysvar.h>

#define LENGTH	0x80000		/* maximum length for RLC		*/
#define MAXOBJ	100			/* maximum number of objects	*/

/**************************************************************************/

void main(void)
{
	image a;
	U16 *rlc, *slc, *next;
	U32 holes[MAXOBJ], qin[MAXOBJ];
	I32 i, nobj;

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
	/**/ 	print("Init Extension Lib = %d\n", init_licence("E1122334455"));     /**/
    /*******************************************************************************/

	/* set image variable to size of screen	*/

	a.st    = (U8 *)getvar(CAPT_START);
	a.dx    = getvar(HWIDTH);
	a.dy    = getvar(VWIDTH);
	a.pitch = getvar(VPITCH);


	/* allocate memory for RLC				*/

	rlc=rlcmalloc(LENGTH);


	/* print message						*/

	pstr("number of holes property\n");


	/* main program								*/

	if((slc=rlcmk(&a, 128, rlc, LENGTH)) != 0)		/* create RLC			*/
	  {
	  rlcout(&a, rlc, 0, 255);						/* output binary image	*/
	  }
	else
	  {
	  print("error creating RLC. ABORT!\n");
	  goto exit_pgm;
	  }


	/* object labelling, necessary for arf		*/

	next=sgmt(rlc,slc);
	if(next == NULL)
	  {
	  print("object number overrun. ABORT!\n");
	  goto exit_pgm;
	  }


	/* area filter to eliminate small objects	*/

	rlc_arf(rlc, rlc, 20);								/* in-place operation	*/
	rlcout(&a, rlc, 0, 255);							/* output binary image	*/
														/* output result		*/
	print("area filter ready, hit key\n");
	rs232rcv();


	/* second labelling operation				*/

	next=sgmt(rlc,slc);
	if(next == NULL)
	  {
	  print("object number overrun. ABORT!\n");
	  goto exit_pgm;
	  }


	nobj = rlc_qin(rlc, qin, MAXOBJ);
	nobj = rlc_nhls(rlc, holes, MAXOBJ);


	/* output results							*/


	for(i=0; i<nobj; i++)
	  {
	  print("object number: %d is hole of object %d :: number of holes: %d\n", i, qin[i], holes[i]);
	  }



	exit_pgm:
	rlcfree(rlc);										/* release RLC memory	*/
}


/* EOF */
