/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2000-2006 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: rlcand.c
***                                                            
*** Comments: 
*** 
***    example program for binary image processing tutorial
***    functions of 2 runlength codes: RLC AND
***
*** Revision History:
***    Date          Version    Author  Changes
***    Jan 13, 2006  3.01       ME      file created, taken from rlc2.c
***
***************************************************************************
*END***********************************************************************/
#include <vcrt.h>
#include <vclib.h>
#include <register.h>
#include <sysvar.h>


/*FUNCTION*-------------------------------------------------------------
*
* Function Name   : rlc2
* Input Values	  : 
*	rlca  		  : source RLC A
*	rlcb	  	  :	source RLC B
*	dest	      : destination RLC
*   func		  : RLC basic function pointer (performs the
*				  : actual operation)
* Returned Values : pointer past end of output RLC (aligned),
*                   NULL if rlca and rlcb have different formats
* Comments        :	main function for all operations of 2 RLCs
*
*END*-----------------------------------------------------------------*/

U16 *rlc2(U16 *rlca, U16 *rlcb, U16 *dest, U16 * (*func)() )
{
	I32 dx, dy;
	I32 i;

	rlca += 2;                 	/* skip SLC address                         */
	rlcb += 2;                 	/* skip SLC address                         */
	dx = *rlca++;    			/* read dx                                  */
	dy = *rlca++;    			/* read dy                                  */

	if (dx != *rlcb++) return (NULL);
	if (dy != *rlcb++) return (NULL);

	*dest++ = 0;				/* output 0L for unlabeled RLC              */
	*dest++ = 0;

	*dest++ = dx;      			/* output number of lines                   */
	*dest++ = dy;      			/* output line delimiter                    */

	for(i=0; i<dy; i++)
	   {
	   func(&rlca, &rlcb, &dest, dx);     	/* execute function         	*/
	   }

	if((U32)dest & 2) dest++;	/* integer alignment						*/

	return(dest); 
}


/*FUNCTION*-------------------------------------------------------------
*
* Function Name   : rlc_andf
* Input Values	  : 
*	rlca  		  : RLC source 1 handle
*	rlcb		  : RLC source 2 handle
*	dst	  		  :	result RLC handle
*	dx		      : line width
* Returned Values : pointer past end of output RLC
* Comments        :	basic function for RLC AND
*
*END*-----------------------------------------------------------------*/

U16 *rlc_andf(U16 **rlca, U16 **rlcb, U16 **dst, I32 dx)
{
	U16 posa, posb;
	U16 ca, cb;
	U16 *p1 = *rlca;
	U16 *p2 = *rlcb;
	U16 *pr = *dst;

	ca = (I32)((I16)*p1++); 	
	cb = (I32)((I16)*p2++); 	

	*pr++ = ca & cb;    /* output result color  */

	posa = *p1++;
	posb = *p2++;

	while(1)
	  {
	  if(posa>posb)             /* posa>posb    */
	    {
	    if(ca) *pr++ = posb;
	    cb = ~cb;               /* toggle cb    */
	    posb = *p2++;
	    }
	  else
	    {
	    if(posa!=posb)          /* posa<posb    */
	      {
	      if(cb) *pr++ = posa;
	      ca = ~ca;             /* toggle ca    */
	      posa = *p1++;
	      }
	    else
	      {                     /* posa=posb    */
	      if(posa==dx)
	        {
	        *pr++ = dx;         /* output EOL   */
	        break;
	        }
	      if(ca==cb) *pr++ = posa;
	      ca = ~ca;
	      cb = ~cb;
	      posa = *p1++;
	      posb = *p2++;
	      }
	    }
	  }

	*rlca = p1;
	*rlcb = p2;
	*dst  = pr;

	return(pr);
}



/* EOF */


