/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2011 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: blob.c
***                                                            
*** Comments: 
*** 
***    	The program is searching for black objects on a white background.
***		Black areas must be darker than the threshold value. The program
***		returns the size, the position of every black area and shows the
***		position on the monitor.
***
*** Revision History:
***    Date          Version    Author  Changes
***    Jul 04, 2002  1.00       KS      initial version
***    Feb 03, 2011  3.11       LB		updated to latest libs
***
***************************************************************************
*END***********************************************************************/

#define NEW_IMAGE_VAR

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>
#include <flib.h>
#include <licence.h>

/*******************************************************************/

I32  blobtest     (int x, int y, int dx, int dy, int color);
I32  find_objects (image *area, ftr *f, int maxobjects, int th, int display);
void cross_image  (int line, int column, int size, int color);

/*******************************************************************/

int ms;

/*******************************************************************/

void main(void)

  {   
  int   x, y, dx, dy;
  char  resp;

  dx = ScrGetColumns;
  dy = ScrGetRows;
  x  = 0;
  y  = 0;
  
    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
    /*******************************************************************************/

  // Working Page
  ScrSetLogPage((int)ScrGetPhysPage);

  print("\nblob analysis ('w'=white 'b'=black objects)\n"); resp=rs232rcv();
  tpict();
  
  do
    {
	// Take picture
    tpict();

	// Do blob detection
    if (resp=='w') blobtest(x, y, dx, dy, -1);
    if (resp=='b') blobtest(x, y, dx, dy,  0);

    print("\nblob analysis (%dx%d): t=%dms",dx,dy,ms); 
    print("\n\nnew run? ('w'=white 'b'=black 'q'=quit)");  
    if (kbhit()) resp=rs232rcv();
    }
  while(resp !='q' );

  // Live picture at the end
  while(cancel_capture_rq());
  vmode(vmLive);
  }

/*******************************************************************/

#define MAXOBJ  4000

I32 blobtest (int x, int y, int dx, int dy, int color)

  {
  int i, j=0, nobj,threshold=100,display=1;
  image area;
  ftr *f;
  
  // Allocate memory for features
  f = (ftr *)sysmalloc((sizeof(ftr)*(MAXOBJ))/sizeof(I32) + 1, MDATA);
  if ((int)f == -1 || f == NULL) {print("Allocation Error for f[], sysmemfree = %d\n", sysmemfree()); rs232rcv(); return(-1);}

  // Search area is the whole image
  ImageAssign(&area, ScrByteAddr(x,y), dx, dy, ScrGetPitch);
  
  // feature extraction
  nobj=find_objects(&area, f, MAXOBJ, threshold, display);
  
  print("\n\nFound %d white and black objects (-1 means more than %d objects)", nobj, MAXOBJ);

  if (color) print("\nwhite objects:");
  else       print("\nblack objects:");

  for(i=0;i<nobj;i++)
    {
    // Looking for color objects (black=0, white=-1)
    if (f[i].color==color) 
      {
      // Print results on telnet
      print("\nArea %d: Size=%d Center of Gravity = (%d,%d)",
               ++j, f[i].area,f[i].x_center+x, f[i].y_center+y);
	  // Draw blobs on image	
      MarkCross(&area, (int)f[i].x_center+x, (int)f[i].y_center+y, 127, 10);
      }
    }
	
  sysfree(f);
  
  return(0);
  }

/*******************************************************************/

int find_objects (image *Src, ftr *f, I32 maxobjects, I32 Thresh, I32 display)

  {
  long maxRLC = 0x050000;
  U16 *rl1, *rl2, *slc;
  int  objects;

  // allocate DRAM Memory
  rl1 = rlcmalloc(maxRLC);
  if (rl1 == NULL) {pstr("DRAM Memory overrun\n"); return(-1);}

  // create RLC
  ms = getvar(MSEC);
  rl2 = rlcmk(Src, Thresh, rl1, maxRLC);
  if (rl2 == NULL) {pstr("RLC overrun\n"); rlcfree(rl1); return(-1);}

  // you can use some filter (dilate, erode for example)
  if (1) // 1=with filter 0=no filter
    {
    slc=erode2(rl1,rl2);
    // slc=dilate(rl1,rl2);
    // slc=rlc_mf(rl1,rl2,0,4);
    }
  else
    {
    slc=rl2;
    rl2=rl1;
    }
  
  // object labelling
  if(sgmt(rl2, slc)==NULL) {pstr("Object number overrun\n"); rlcfree(rl1); return(-1);}

  // feature extraction
  objects = rl_ftr2(rl2, f, maxobjects);

  ms = getvar(MSEC)-ms;
  if(ms<0) ms+=1000;
 
  // display RLC
  if (display)
   {
   if(rlcout(Src, rl1, 0, 255) < 0) printf("Error in rlcout\n");
   }

  // free allocation
  rlcfree(rl1);  

  return(objects);
  }

/*******************************************************************/
