/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 1997-2006 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: compare.c
***                                                            
*** Comments: 
***
***    demoprogram: binary image processing
***					binary object comparison
***    
*** The program provides a menu for object definition and comparison.
*** With "define object", the user can select a threshold for binarisation,
*** chose between white and black binary objects, finally he can select
*** one of the objects displayed sequentially on the screen.
*** With "compare", a picture is taken, RLC is generated with the same
*** threshold as above, the system searches for an object with the same
*** color and similar object area (within 10% of the original).
*** The found object is moved, so that its center of gravity matches
*** the center of gravity of the original object. Finally the objects are
*** compared using the RLC XOR function and pixel counting 
***
*** Revision History:
***    Date          Version    Author  Changes
***    1998		 	 1.0		ME		first version for ADSP
***    Jan 12, 2006  2.0        ME      port to TI, VCLIB 3.x
***    Jul 10, 2007  2.01       LB      replace chprint_ov() with chprint1,
***										cosmetic changes
***
***************************************************************************
*END***********************************************************************/

#define NEW_IMAGE_VAR

#include <vcrt.h>
#include <vclib.h>
#include <flib.h>
#include <register.h>
#include <sysvar.h>
#include <macros.h>

/***************************************************************************/

#define stx ((7*getvar(VWIDTH))/8*ptch+60)

#define LNG   0x80000     	/* maximum RLC length       */

U16 *rlca=NULL;    			/* RLC scratch area         */
U16 *rlcb=NULL;    			/* RLC object area          */
I32 col = -1;				/* global object color		*/

/***************************************************************************/

void define(void);
void compare(void);
void finish(void);
void terminate(void);
int fcomp(ftr *f);
char r_out(U16 *rlc);
void clr_ovl();

/***************************************************************************/

typedef struct
  {
  int thr;              /* threshold            */
  long area;            /* object area          */
  int xc;               /* centroid x           */
  int yc;               /* centroid y           */
  } cp_data;

cp_data cpd;
int tm=1;               /* test mode flag       */

/***************************************************************************/

void main()
{
	char c;

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
    /*******************************************************************************/

	rlca = rlcmalloc(LNG);
	rlcb = rlcmalloc(LNG);

	vmode(0);                       /* max processor speed                  */

	pstr("compare: \n");
	pstr("Binary Object Compare using RLC Vs. 2.0\n");
	pstr("Copyright Vision Components   1998-2006\n\n");
	pstr("press ESC to abort or any other key to continue\n");
	if(rs232rcv()==0x1b) return;

	while(1)
	  {
	  rs232snd(0x0c);
	  pstr("compare: Binary Object Compare Vs. 1.0\n");
	  pstr("main menu               Copyright 1998\n\n\n");
	  pstr("define object  ......................... (1)\n");
	  pstr("compare ................................ (2)\n");
	  pstr("set test mode  ..........................(3)\n");
	  pstr("set automatic mode  .....................(4)\n");
	  pstr("exit  .................................. (e)\n");
	  pstr("\n"); c=rs232rcv();

	  switch (c)
	    {
	    case '1':     define();                     finish(); break;
	    case '2':     compare();                    finish(); break;
	    case '3':     tm=1;                         finish(); break;
	    case '4':     tm=0;                         finish(); break;
	    case '5':                                   finish(); break;
	    case 'e':     terminate();                  rlcfree(rlca);
	    											rlcfree(rlcb);
	    											return;
	    default:      break;
	    }
	  }

}

/***************************************************************************/

void finish()
{}

/***************************************************************************/

void define()
{
	char c;
	I32 thr=128, obj=0;
	image a, b;
	U16 *r1, *r2, *r0=rlca;
	U16 *r3=rlcb;
	feature f;
	I32 ptch=getvar(VPITCH);

	ImageAssign(&a, getvar(CAPT_START), getvar(HWIDTH), getvar(VWIDTH), ptch); 	/* main image area      */
	ImageAllocate(&b, IMAGE_GREY, getvar(HWIDTH), getvar(VWIDTH));


	vmode(0);                       /* live video mode      */

	rs232snd(0x0c);
	pstr("define object for comparison\n");
	pstr("hit key to take picture\n");
	rs232rcv();

	tpict();
	copy(&a, &b);

	pstr("select threshold\n");
	pstr("+   : increase threshold\n");
	pstr("-   : decrease threshold\n\n");
	pstr("ESC : finish operation\n\n");

	do
	  {
	  print("thr = %d     \r",thr);
	  binarize(&b,&a,thr,0,255);    /* fast binarisation    */
	  c=rs232rcv();                 /* read character       */
	  if(c == '+') thr++;           /* increase threshold   */
	  if(c == '-') thr--;           /* decrease threshold   */
	  if(thr < 0)   thr=0;
	  if(thr > 255) thr=255;
	  }
	while(c != 0x1b);

	cpd.thr = thr;                  /* store threshold to   */
	                                /* global data          */

	if((r1=rlcmk(&b, thr, r0, LNG)) == NULL)
	  {
	  pstr("rlc overrun\ntry again\n");
	  pstr("hit any key to continue\n");
	  rs232rcv();
	  ImageFree(&b);
	  return;
	  }

	rlcout(&a, r0, 40, 200);
	pstr("eliminating small objects\n");

	#ifdef MEDIAN
	   rlc_mf(r0,r0,  0, 2);
	r1=rlc_mf(r0,r0, -1, 2);

	r2=erode(r0,r1);
	r1=dilate(r1,r2);
	#else
	sgmt(r0, r1);                  			/* object labelling     */
	rlc_arf(r0, r0, 20);					/* area filter, min object size = 20	*/
	r2=r0;
	#endif

	rlcout(&a, r2, 60, 180);

	if((sgmt(r2,r1)) == NULL)
	  {
	  pstr("object number overrun\ntry again\n");
	  pstr("hit any key to continue\n");
	  rs232rcv();
	  ImageFree(&b);
	  return;
	  }

	pstr("now select object color\n");
	pstr("+/- : toggle color\n");
	pstr("ESC : finish operation\n\n");

	do
	  {
	  if(col!=0)
	    pstr("use white objects only\r");
	  else
	    pstr("use black objects only\r");
	  c=rs232rcv();                 /* read character       */
	  if(c == '+') col = ~col;      /* toggle color         */
	  if(c == '-') col = ~col;      /* toggle color         */
	  }
	while(c != 0x1b);

	pstr("now select object\n");
	pstr("+   : next object\n");
	pstr("-   : previous object\n");
	pstr("ESC : finish operation\n\n");

	do
	  {
	  if(rlc_cut(r2,r3,obj) == 0L) break;
	  rlcout(&a,r3,0,255);

	  rs232snd(0x0c);
	  print("object number: %d\n",obj);
	  rlc_feature(&f,r3,-1);
	  print("area       : %6lx \n",f.area);
	  print("x-center   : %ld\n",f.x_center);
	  print("y-center   : %ld\n",f.y_center);
	  print("x-min      : %d\n",f.x_min);
	  print("x-max      : %d\n",f.x_max);
	  print("y-min      : %d\n",f.y_min);
	  print("y-max      : %d\n",f.y_max);
	  print("x-last     : %d\n",f.x_lst);

	  c=rs232rcv();                 /* read character           */
	  if(c == '+') obj++;           /* increase object number   */
	  if(c == '-') obj--;           /* decrease object number   */
	  if(obj < 0) obj=0;
	  }
	while(c != 0x1b);

	cpd.area = f.area;
	cpd.xc   = (int)f.x_center;
	cpd.yc   = (int)f.y_center;


	ImageFree(&b);
}

/***************************************************************************/

void compare()
{
	char c=0;
	image a, txt;
	ftr f[500];
	U16 *r0=rlca;
	U16 *r3=rlcb;
	U16 *r1, *r2, *r4;
	int i, nobj;
	char s[40];
	I32 ptch=getvar(VPITCH);
	U8 *sto =(U8 *)getvar(OVLY_START);

	ImageAssign(&a, getvar(CAPT_START), getvar(HWIDTH), getvar(VWIDTH), ptch); 	/* main image area      */
	ImageAssign(&txt, sto+stx,  480, 64, ptch); 								/* text field / overlay */

	clr_ovl();
	chprint1("comparing ...",&txt,4,4,128);
	pstr("comparing ...\n");
	pstr("press ESC to return to main menu\n");

	while(c != 0x1b)
	  {
	  tpict();
	  if(tm==0)
	//    vmode(4);
	    vmode(5);	/* overlay still	*/
	  else
	    vmode(5);	/* overlay still	*/

	  if((r1=rlcmk(&a, cpd.thr, r0, LNG)) != NULL)
	    {
	#ifdef MEDIAN
	    rlc_mf(r0,r0,0, 10);
	    r1=rlc_mf(r0,r0,-1,10);
	#else
	    sgmt(r0, r1);                  			/* object labelling     */
	    rlc_arf(r0, r0, 20);					/* area filter, min object size = 20	*/
	#endif

	    c=r_out(r0);
	    if((r2=sgmt(r0,r1)) != NULL)			/* object labelling						*/
	      {
	      nobj=rl_ftr2(r0, f, 500);				/* feature extraction					*/

	      if(tm==1) print("number of objects: %d\n",nobj);
	      for(i=0;i<nobj;i++)
	        {
	        if(fcomp(&f[i])==0) break;			/* search for objects with correct area	*/
	        }

	      if(i==nobj)
	        {
	        chprint1("not found    ",&txt,4,4,128);
	        }
	      else
	        {
	        if(tm==1)
	          {
	          print("object found, no: %d\n",i);
	          print("x_c: %d  y_c: %d\n",
	                (int)f[i].x_center,(int)f[i].y_center);
	          }
	        r4=rlc_cut(r0,r2,i);
	        c=r_out(r2);

	        r2=rlc_move(r2, r4, cpd.xc-f[i].x_center, cpd.yc-f[i].y_center);
	        c=r_out(r4);

	        rlcxor(r4,r3,r2);					/* XOR object compare					*/
	        c=r_out(r2);
	        io_sprintf(s,"diff.: %d %%      \r",(100*rlc_area(r2,-1))/cpd.area);
			pstr(s);
	        chprint1(s,&txt,4,4,128);
	        }
	      }
	    else
	      {
	      pstr("object no. overrun \r");
	      return;
	      }
	    }
	  else
	    {
	    pstr("rlc overrun       \r");
	    }

	  } /* end while */

}

/***************************************************************************/

char r_out(U16 *rlc)			/* output RLC image to screen		*/
{
	image a;
	I32 ptch=getvar(VPITCH);

	if(tm==0)
	  {
	  if(rbempty() != -1)
	    return(rs232rcv());
	  else
	    return((char)0);
	  }

	ImageAssign(&a, getvar(CAPT_START), getvar(HWIDTH), getvar(VWIDTH), ptch); 	/* main image area      */
	rlcout(&a,rlc,0,255);
	pstr("hit key\n");
	return(rs232rcv());

}

/***************************************************************************/

void terminate(void)
{
	pstr("thank you for using compare\n");
}

/***************************************************************************/

int fcomp(ftr *f)							   /* search for objects	*/
{											   /* in feature list		*/
	long area=f->area;							   /* with correct area		*/
	long ar2=cpd.area;

	if(f->color != col)  return(-1);                /* wrong object color   */
	if((area > (ar2*9)/10) & (area < (ar2*11)/10))  /* 10 % area tolerance */
	  return(0);
	else
	  return(-1);

}

/***************************************************************************/

void clr_ovl()      /* clear overlay                        	*/
{
	image ovl;

	ovl.st    = (U8 *)getvar(OVLY_START);
	ovl.dx    = getvar(DHWIDTH);
	ovl.dy    = getvar(DVWIDTH);
	ovl.pitch = getvar(VPITCH);

	set(&ovl,0);
}

/***************************************************************************/


/* EOF */
