/*******************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Klaus Schneider, Peter Neuhaus  VC


Program Description:
--------------------

Edge detection along an arbitrary search line (improvement over EdgeDetection.c : the search line is not limited to horizontal or vertical).
See Programming tutorial chapter 7.3.


*/
/*******************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>
#include <flib.h> 

/*******************************************************************/

void DisplayGraph                (image Graph, U8 *values, I32 nr);
void IniDisplay                  (void);
	
/*******************************************************************/


void main(void)
{
 
	I32  x, y, x1=1, y1=1, x2=600, y2=340, dx, dy, Color = 128,Pitch, MaxLength ;
	U8 response;

	U8 *pTemp, *LineVal;
	U8 **LineAddr;
	I32 *LineXY;

	image Source, Graph, Area;
	
	/*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
    /*******************************************************************************/

	// initialise graph area
	IniDisplay();

	dx = ScrGetColumns;
	dy = 256;
	x  =    0;
	y  = DispGetRows -256;
	ImageAssign(&Graph, ScrByteAddr(x, y), dx, dy,ScrGetPitch);
	ImageAssign(&Area, OvlByteAddr(x, y), dx, dy, ScrGetPitch);
	frameo(&Area);


	ScrSetLogPage(ScrGetPhysPage); 
	ImageAssign(&Source,ScrByteAddr(0,0),ScrGetColumns,ScrGetRows,ScrGetPitch);

	/////////////////////////////////////////////////////////////////////////////////////
	// ini values
	/////////////////////////////////////////////////////////////////////////////////////
	Pitch = Source.pitch;

	MaxLength = max(Source.dx, Source.dy) + 1;

	LineXY = (I32 *)sysmalloc(2 * MaxLength, MDATA); //memory allocation pixel for coordinates
	LineAddr = (U8 **)sysmalloc(MaxLength, MDATA);   //memory alocation for address list
	LineVal = (U8 *)sysmalloc(MaxLength, MDATA);     //memory allocation for grey values

	/////////////////////////////////////////////////////////////////////////////////////
	// calculate intersection points and draw corner line0
	/////////////////////////////////////////////////////////////////////////////////////
	dx = x2 - x1;
	dy = y2 - y1;

	MaxLength = linexy(dx, dy, LineXY);

	pTemp = (U8 *)Source.st + x1 + y1 * Pitch;

	FL_adcalc_U8(MaxLength, LineXY, LineAddr, pTemp, Pitch);

	/////////////////////////////////////////////////////////////////////////////////////
	// take image and read and display grey values along line
	/////////////////////////////////////////////////////////////////////////////////////
	do
	  {
		tpict();

		 // read line
		FL_rplist_U8(MaxLength, LineAddr, LineVal);

	
		// read line		
		DisplayGraph(Graph, LineVal, dx-1);

		// draw line into image
		if (Color >= 0) 
			FL_wpset_U8 (MaxLength, LineAddr, Color);
			else 
			FL_wpxor_U8 (MaxLength, LineAddr, -Color);


		///////////////////////////////////////////////////////////////////////////////////////
		// Wait For Response
		///////////////////////////////////////////////////////////////////////////////////////

	    print("'q'=quit\n");

	    response = rs232rcv();

	  }
	  while(response != 'q');


	vmode(0);

	sysfree(LineVal);
	sysfree(LineAddr);
	sysfree(LineXY);
}

/************************************************************************************************************************/

void IniDisplay (void)
{
	  image Temp;

	  ScrSetLogPage(ScrGetPhysPage);
	  OvlSetLogPage(OvlGetPhysPage);

	  // Clear Overlay
	  ImageAssign(&Temp, OvlByteAddr(0, 0), DispGetColumns, DispGetRows, ScrGetPitch);
	  set(&Temp, 0);  

	  // Clear Image
	  ImageAssign(&Temp, ScrByteAddr(0, 0), DispGetColumns, DispGetRows, ScrGetPitch);
	  set(&Temp, 0);  

	  set_ovlmask(255);

	  vmode(vmOvlLive);
}

/************************************************************************************************************************/

void DisplayGraph(image Graph, U8 *values, I32 nr)
{
  I32 i, x, y, dy, xold, yold;


  dy    = Graph.dy;
  x     = 0;
  y     = 0;


  set(&Graph, 0);
    
	  for (i=0; i<nr; i++)
	  {
	    xold = x;
		yold = y;

	    x = i;
	    y = dy - values[i];

	    lined(&Graph, xold, yold, x, y, 255);
	  }
}

/************************************************************************************************************************/
