/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 1998-2006 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: flaw.c
***                                                            
*** Comments: 
***
***    demoprogram: binary image processing
***					flaw detection with unsharp masking and RLC feature 
***    				extraction
***
*** Revision History:
***    Date          Version    Author  Changes
***    1998		 	 1.0		ME		first version for ADSP
***    Jan 12, 2006  2.0        ME      port to TI, VCLIB 3.x
***
***************************************************************************
*END***********************************************************************/

#include <vcrt.h>
#define NEW_IMAGE_VAR

#include <vclib.h>
#include <flib.h>
#include <register.h>
#include <sysvar.h>
#include <macros.h>

#include <vcrt.h>
#include <vclib.h>
#include <stdlib.h>

/**************************************************************************/

void main(void)
{
	int x=10;
	image a, b;
	U16 *rlc;
	I32 maxlng = 0x080000;
	U16 *rl2, *rl3, *slc;
	ftr f[1000];
	int i, nobj;
	I32 ptch=getvar(VPITCH);

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
    /*******************************************************************************/

	ImageAssign(&a, getvar(CAPT_START), getvar(HWIDTH), getvar(VWIDTH), ptch); 	/* main image area      */
	ImageAllocate(&b, IMAGE_GREY, getvar(HWIDTH), getvar(VWIDTH));

	rlc = rlcmalloc(maxlng);


	print("flaw detection using unsharp masking v2.0\n");

	vcsetup();

	tpict();

	maskx(&a, &b, 32, 32, 128);             /* unsharp masking  */
	copy(&b, &a);							/* just to see		*/

	print("masked image - hit key\n"); rs232rcv();

	/* create RLC                                               */

	if((rl2=rlcmk(&b, 128+x, rlc, maxlng)) == NULL)
	  {
	  pstr("RLC overrun\n");
	  return;
	  }

	rlcout(&a, rlc, 0, 255);

	pstr("binarized image - hit key\n"); rs232rcv();
	rl3=dilate(rlc,rl2);                    /* binary dilation  */
	rlcout(&a, rl2, 0, 255);

	pstr("dilated image - hit key\n"); rs232rcv();
	slc=erode(rl2,rl3);                     /* binary erosion   */
	rlcout(&a, rl3, 0, 255);

	pstr("eroded image - hit key\n"); rs232rcv();

	if(sgmt(rl3,slc)==0L)                   /* object labelling     */
	  {
	  pstr("object number overrun\n");
	  return;
	  }

	nobj=rl_ftr2(rl3, f, 1000);             /* feature extraction   */

	print("number of objects: %d\n",nobj);

	for(i=0;i<nobj;i++)
	  {
	  print("i: %03x area: %06x x_c: %03x y_c: %03x\n",
	         i,f[i].area,(int)f[i].x_center,(int)f[i].y_center);
	  print("x_min: %03x x_max: %03x y_min: %03x y_max: %03x x_lst: %03x",
	          f[i].x_min,f[i].x_max,f[i].y_min,f[i].y_max,f[i].x_lst);
	  print(" color: %1x\n",f[i].color&1);
	  }

	ImageFree(&b);
	rlcfree(rlc);
}

/* ************************************************* */

/* EOF */
