/*******************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Klaus Schneider,  VC


Program Description:
--------------------

The program is reading Pixels from the DRAM (complete image)
and calculates the brightness of the pixels in 2 different ways:


1) pixel list
 
2) direct addr 

It shows the time for each way.


ATTENTION: Live Mode is always updating the DRAM !!!


Physical page: 

The image will be stored at this page or displayed 
from this page

Logical  page: 

On this page you work. For example ScrByteAddr(x,y) 
always refers to the logical page

Normaly logical and physical page is the same.

But it is possible to display an image and work on an 
other image (ping pong)

*/
/*******************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

/*******************************************************************/

int Brightness2(image *area);
int Brightness4(image *area);

/*******************************************************************/

void main(void)

  {
  int time_ms, brightness;
  image area;

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
    /*******************************************************************************/


  /* working page = physical page */   
  ScrSetLogPage(ScrGetPhysPage);
  ImageAssign(&area,ScrByteAddr(0,0), ScrGetColumns, ScrGetRows, ScrGetPitch);  

  vmode(vmLive);
  print("Get image and calculate the sum of it. Press any key\n");

  rs232rcv();           /* wait for any key */

  tpict();              



  /* Possibility 2: Read with function pixel list(); */
  /* ----------------------------------------------- */
  time_ms = getvar(MSEC);
  brightness=Brightness2(&area);
  time_ms = getvar(MSEC)-time_ms;
  if (time_ms<0) time_ms+=1000;
  print("pixel list:       Pixels=%ld  Sum=%ld Time=%d ms\n",(long)(area.dx*area.dy),(long)brightness,time_ms);



   /* Possibility 4: Read with function rpix(); */
  /* ----------------------------------------- */
  time_ms = getvar(MSEC);
  brightness=Brightness4(&area);
  time_ms = getvar(MSEC)-time_ms;
  if (time_ms<0) time_ms+=1000;
  print("direct addr.:     Pixels=%ld  Sum=%ld Time=%d ms\n",(long)(area.dx*area.dy),(long)brightness,time_ms);
  }

/*******************************************************************/

int Brightness2(image *area)

  {
  int   i, j, dx, dy, sum=0;
  int   *v_list, *a_list, *xy, *ptemp;

  dx   =area->dx;
  dy   =area->dy;

  v_list = (int *)vcmalloc(dx);  
  a_list = (int *)vcmalloc(2*dx);  

  /* add lines together */
  for(j=0;j<dy;j++)
    {
    xy= a_list;    /* restore pointer */

    /* Generate Address list for one line */
    for (i=0; i<dx; i++) 
      {
      *xy++ = i;
      *xy++ = 0;
      }

    xy= a_list;    /* restore pointer */

    ad_calc(dx, a_list, (long *)a_list, (long)j*area->pitch+area->st, area->pitch);
    rp_list(dx, (long *)a_list, v_list);

    ptemp=v_list;

    for (i=0; i<dx; i++) 
      {
      sum+=0xff&(*ptemp++);
      }
    }


  vcfree(a_list);         /* Don't forget */
  vcfree(v_list);         /* Don't forget */

  return(sum);
  }

/*******************************************************************/

int Brightness4(image *area)

  {
  int   i, j, dx, dy, sum=0;
  long  addr, pitch;
  unsigned char *ppix;


  addr=area->st;
  dx=area->dx;
  dy=area->dy;
  pitch=area->pitch;

  for(j=0;j<dy;j++)
    {
    ppix=(unsigned char *)(addr);

    for (i=0; i<dx; i++) 
      {
      sum+=*ppix++;
      }
    addr+=pitch;
    }

  return(sum);
  }

/*******************************************************************/

