/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 1997-2006 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: track.c
***                                                            
*** Comments: 
***
***    demoprogram: binary image processing
***					object tracking
***    
***	   The program demonstrates the live tracking of a bright object.
***    First, a threshold for binarisation is determined using the
***    histogram-function. The threshold is set to (min+max)/2.
***    A runlength code is generated using this threshold and 
***    rlc_feature() is called to produce the features of the
***    non-labelled RLC. The object is found at the center of gravity
***    for the white objects. For subsequent frames the search area
***    is reduced from full-frame to (X_SIZE, Y_SIZE) and the object will
***    be tracked. If the white objects touches the boundaries of the
***    search area, or no object is found, the tracking is considered
***    unsuccessful and the search area is set to full-frame again
***
*** Revision History:
***    Date          Version    Author  Changes
***    ~1997		 1.0		ME		first version for ADSP
***    Jan 11, 2006  2.0        ME      port to TI, VCLIB 3.x
***
***************************************************************************
*END***********************************************************************/

#include <vcrt.h>
#define NEW_IMAGE_VAR

#include <vclib.h>
#include <flib.h>
#include <register.h>
#include <sysvar.h>
#include <macros.h>

/**************************************************************************/
	
#define SIZE 0x80000        /* size of RLC - plenty of room !   */
#define COLOR -1            /* 0: black, -1: white              */
#define X_SIZE 200          /* x size of tracking window        */
#define Y_SIZE 200          /* y size of tracking window        */

void clr_ovl(void);
void calc(U32 *hist, long n, int *avg, int *min, int *max);
void show1(image *a, int x, int c);
void show(image *a, int x, int c);
void track(void);
void setwindow(feature *f, image *v, image *w, image *full);

/**************************************************************************/

void main()
{

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
    /*******************************************************************************/


	clr_ovl();		
	vmode(4);           /* live image + overlay                     */
	track();
}

void clr_ovl()      /* clear overlay                        	*/
{
	image ovl;

	ovl.st    = (U8 *)getvar(OVLY_START);
	ovl.dx    = getvar(DHWIDTH);
	ovl.dy    = getvar(DVWIDTH);
	ovl.pitch = getvar(VPITCH);

	set(&ovl,0);
}


void track()
{
	U8 *st, *sto;
	int dx = getvar(HWIDTH) - 8;
	int dy = getvar(VWIDTH);
	int ptch = getvar(VPITCH);
	U32 rst, tst;
	int rx, ry, min, max, avg;
	int tx, ty, i, thr, x, y;
	char c=' ';
	U16 *rlc;										/* pointer to RLC		*/
	U32 hist[256];									/* histogram buffer		*/
	image full;                                     /* full window          */
	image w;                                        /* tracking window      */
	image w2;                                       /* same in 1st halfframe*/
	image r_ovl;                                    /* right bar            */
	image text1;                                    /* copyright notice     */
	feature f;                                      /* feature list         */
	I32 t, t0;										/* time measurement		*/


	st  = (U8 *)getvar(CAPT_START);					/* start of screen		*/
	sto	= (U8 *)getvar(OVLY_START);					/* start of overlay		*/

	ImageAssign(&full, st,  dx, dy, ptch); 			/* full window          */
	ImageAssign(&w,   sto,  dx, dy, ptch); 			/* overlay memory       */
	ImageAssign(&w2,   st,  dx/2, dy, 2*ptch); 		/* image memory         */

	rlc = rlcmalloc(SIZE);							/* allocate RLC space	*/


	/* right bar display for illumination condition: min, max, average		*/

	#define rdx 32									/* distance from right	*/
	#define rdy 260									/* distance from bottom */
	rx = 7*dx/8 - rdx/2;                            /* x upper left corner  */
	ry = (dy - rdy)/2;                              /* y upper left corner  */
	rst = ry*ptch + rx;                   			/* of                   */
	ImageAssign(&r_ovl, sto+rst,  rdx, rdy, ptch); 	/* right bar            */


	/* text field 															*/

	#define tdx 512									/* distance from right	*/
	#define tdy 64									/* distance from bottom */
	tx = (dx - tdx)/2;                              /* x upper left corner  */
	ty = 7*dy/8 - tdy/2;                            /* y upper left corner  */
	tst = ty*ptch + tx;                   			/* of                   */
	ImageAssign(&text1, sto+tst,  rdx, rdy, ptch); 	/* text field           */

	dframeo(&r_ovl);            /* draw right bar                           */

	chprint1("track vs. 2.0",&text1,4,4,128);
	dmarkerz(&w);

	avg=min=max=0;
	i=0;

	/* main loop ************************************************************/

	while(c != 0x1b)
	  {
	  if(rbempty() != -1) c=rs232rcv();             /* read character       */

	  tpict();
	  vmode(4);
	  t0 = getvar(MSEC);
	  histo(&w2,hist);
	  show(&r_ovl,min,1);
	  show(&r_ovl,max,1);
	  show1(&r_ovl,avg,1);
	  calc(hist,(long)w2.dx*w2.dy,&avg,&min,&max);
	  show(&r_ovl,min,1);
	  show(&r_ovl,max,1);
	  show1(&r_ovl,avg,1);
	  thr=(min+max)/2;
	/*  print("min: %d max: %d thr: %d avg: %d\r",min,max,thr,avg);*/
	  rlcmk(&w2, thr, rlc, SIZE);
	  rlc_feature(&f,rlc,COLOR);
	  dmarkerz(&w);             /* clear window */
	  setwindow(&f,&w,&w2,&full);
	  y=(int)(2L*(w2.st-st) / w2.pitch);
	  x=(int)((w2.st-st) - y * w2.pitch / 2);
	  t = getvar(MSEC)-t0;
	  if(t<0) t+=1000;

	  if((x>1) && (y>1))
	    print("x: %d y: %d time: %d msec \r", x+X_SIZE/2, y+Y_SIZE/2, t);
	  else
	    print("not found   time: %d msec        \r", t);
	 if(i == 200)
	    chprint1("                 ",&text1,4,4,128);
	  else
	  if(i == 100)
	    chprint1("Vision Components",&text1,4,4,128);
	  if(i < 201)
	    i++;
	  dmarkerz(&w);             /* draw window  */
	  }

	rlcfree(rlc);
}

void calc(U32 *hist, long n, int *avg, int *min, int *max)
{
	int i;
	long v;

	i=0;
	while((hist[i]==0) && (i<256)) i++;
	*min=i;

	i=255;
	while((hist[i]==0) && (i>=0)) i--;
	*max=i;

	v=0L;
	for(i=0;i<256;i++)
	  {
	  v   += (long)i * hist[i];
	  }
	v = v/n;

	*avg = (int)v;
}

void show1(image *a, int x, int c)
{
	int i;
	U8  *ad1, *ad2;
	int pitch = a->pitch;

	ad1 = a->st + (a->dy - 2) * pitch + 2;
	ad2 = ad1;

	x = (x < 0)   ? 0   : x;
	x = (x > 255) ? 255 : x;

	ad1 -=  x     * pitch;
	ad2 -=  (x+1) * pitch;

	for(i=0;i<(a->dx)/2-2;i++)
	   {
	   *ad1++ ^= c;         /* avg double line                      */
	   *ad2++ ^= c;
	   }

}

void show(image *a, int x, int c)
{
	int i;
	U8 *ad1, *ad2;
	int pitch = a->pitch;

	ad1 = a->st + (a->dy - 2) * pitch + 2;
	ad2 = ad1;

	x = (x < 0)   ? 0   : x;
	x = (x > 255) ? 255 : x;

	ad1 -=  x     * pitch;
	ad2 -=  (x+1) * pitch;

	for(i=0;i<a->dx-3;i++)
	   {
	   *ad1++ ^= c;         /* x double line                      	*/
	   *ad2++ ^= c;
	   }

}

void setwindow(feature *f, image *v, image *w, image *full)
{
	int x, y;
	U8 *st, *sto, *adr, *lst;

	st  = (U8 *)getvar(CAPT_START);					/* start of screen		*/
	sto	= (U8 *)getvar(OVLY_START);					/* start of overlay		*/

	x = f->x_center - X_SIZE/2;
	y = 2*f->y_center - Y_SIZE/2;
	adr = w->st + y * (w->pitch / 2) + x;
	lst = st + (full->dy - Y_SIZE)*(full->pitch);

	if((f->x_min != 0) && (f->x_max != w->dx)       /* object must be   	*/
	  && (f->y_min != 0) && (f->y_max != w->dy)     /* completely       	*/
	  && (adr > st) && (adr < lst))
	  {                                             /* inside of window 	*/
	  v->st += y * (v->pitch) + x;
	  v->dx = X_SIZE;
	  v->dy = Y_SIZE;
	  w->st = adr;
	  w->dx = X_SIZE;
	  w->dy = Y_SIZE/2;
	  }
	else
	  {
	  v->st = sto;                                  /* take full window 	*/
	  v->dx = full->dx;                             /* for next search  	*/
	  v->dy = full->dy;
	  w->st = full->st;                             /* take full window 	*/
	  w->dx = full->dx;                             /* for next search  	*/
	  w->dy = full->dy/2;
	  }

}


/* EOF */
