/*******************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Klaus Schneider,  VC


Program Description:
--------------------

Image acquisition with AutoShutter

*/
/*******************************************************************/

#include <vcrt.h>
#include <flib.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

/*******************************************************************/

I32 AutoShutterImage (image *area, I32 bright, I32 minsh, I32 maxsh);

/*******************************************************************/

void main(void)

  {
  I32   AreaX, AreaY, AreaDX, AreaDY;
  I32   sh, MinSh, MaxSh, MeanGrey;
  image Area, AreaOvl;


    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
    /*******************************************************************************/


  /* Set logical page */
  ScrSetLogPage((I32)ScrGetPhysPage);
  OvlSetLogPage((I32)OvlGetPhysPage);

  /* clear overlay */
  OvlClearAll;

  /* live image with overlay */
  set_ovlmask(255);
  vmode(vmOvlLive);

  /* define area */
  AreaDX = ScrGetColumns-100;
  AreaDY = ScrGetRows   -100; 
  AreaX  = (ScrGetColumns- AreaDX)/2;
  AreaY  = (ScrGetRows   - AreaDY)/2;

  /* define overlay area */
  ImageAssign(&AreaOvl, OvlBitAddr(AreaX, AreaY), AreaDX, AreaDY, OvlGetPitch);  
  
  /* define image area */
  ImageAssign(&Area, ScrByteAddr(AreaX,AreaY), AreaDX, AreaDY, ScrGetPitch);  
  
  /* Drawings */
  frameo(&AreaOvl);

  print("Auto Shutter\n");
  
  MinSh    =   1000; // us
  MaxSh    = 200000; // us
  MeanGrey =    128; // grey value in image Area
  
  while(!kbhit()) 
  {
    sh = AutoShutterImage(&Area, MeanGrey, MinSh, MaxSh); 
    if (sh > 0) print("shutter=%3dms [%d..%dms]  MeanGrey=%d\n", sh/1000, MinSh/1000, MaxSh/1000, mean(&Area));
    else        print("shutter out of range [%d..%dms]\n", MinSh/1000, MaxSh/1000);
  }

  rs232rcv();

  vmode(vmLive);
  }


/*************************************************************************************************************************/

I32 AutoShutterImage (image *area, I32 bright, I32 minsh, I32 maxsh)
{
  I32  average, tol=20, MaxAttemptions=3;
  static I32  sh = -1;

  // set shutter time
  if(sh < 0) sh = shutter((minsh + maxsh) / 2);

  // get image
  tpict();

  // get image brightness
  average = mean(area);

  // control image brightness
  if ( abs(average-bright) > tol)
  {
    if(bright > 250-tol) bright=250-tol;
    if(bright <   5+tol) bright=  5+tol;

    do
    {
      MaxAttemptions--;

      // speed up system if overloaded
      if (average==255) average*=2;

      // protect division by zero
      if (average <  5) average=5;

      sh = (sh *  bright) / average;

      if(sh > maxsh) sh=maxsh;
      if(sh < minsh) sh=minsh;

      sh = shutter(sh);

      tpict();

      average = mean(area);
    }
    while(abs(average-bright)>tol && MaxAttemptions);
  }

  if (abs(average-bright) > tol) return(-sh);

  return(sh);
}
