/************************************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program Description:
--------------------

Example for parallel image acquisition.
For a omplete description see Programming Tutorial.

*/
/************************************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>
#include <flib.h>

/*******************************************************************/

void main(void)
{
	int Screen1, Screen2, Screen3, Overlay1, Overlay2, Overlay3, Screen2_not_aligned;
	int Screen3_not_aligned, Overlay2_not_aligned, Overlay3_not_aligned;
	int t0, t2, t3, t5, t6, t8, t9, key = 0;
	int tr1, tr2, tr3;
	image CleanScreen, a, b, c, d, e, f;
	
    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
    /*******************************************************************************/

	/* Initialising Licences */
	print("VCLIB licence = %d\n",init_licence("LT2F0AC6161C"));
	print("COLORLIB licence = %d\n",init_licence("LC2F102C9855"));
	
	/* Memory Allocation for addtional image and overlay pages */
	// obtaining start address of page 1 = default capture page
	Screen1 = getvar(CAPT_START);
	Overlay1 = OvlGetPhysPage;
	
	// Allocate screen page 2+3
	Screen2_not_aligned = (int)DRAMDisplayMalloc();
	Screen2 = (Screen2_not_aligned&0xFFFFFC00) + 1024;
	print("Screen2 =%x\n", Screen2);
	Screen3_not_aligned = (int)DRAMDisplayMalloc();
	Screen3 = (Screen3_not_aligned&0xFFFFFC00) + 1024;
	print("Screen3 =%x\n", Screen3);
	
	// Allocate overlay page 2+3
	Overlay2_not_aligned = (int)DRAMDisplayMalloc();
	Overlay2 = (Overlay2_not_aligned&0xFFFFFC00) + 1024;
	Overlay3_not_aligned = (int)DRAMDisplayMalloc();
	Overlay3 = (Overlay2_not_aligned&0xFFFFFC00) + 1024;

	/* Preparation of screen pages, image variables and other settings */
	/* Clean borders image*/
	ImageAssign(&CleanScreen, Screen1, DispGetColumns, DispGetRows, DispGetPitch);
	set(&CleanScreen, 0);
	ImageAssign(&CleanScreen, Screen2, DispGetColumns, DispGetRows, DispGetPitch);
	set(&CleanScreen, 0);
	ImageAssign(&CleanScreen, Screen3, DispGetColumns, DispGetRows, DispGetPitch);
	set(&CleanScreen, 0);
	
	/* Clean borders overlay*/
	ImageAssign(&CleanScreen, Overlay1, DispGetColumns, DispGetRows, DispGetPitch);
	set(&CleanScreen, 0);
	ImageAssign(&CleanScreen, Overlay2, DispGetColumns, DispGetRows, DispGetPitch);
	set(&CleanScreen, 0);
	ImageAssign(&CleanScreen, Overlay3, DispGetColumns, DispGetRows, DispGetPitch);
	set(&CleanScreen, 0);
	
	/* Assiging image variables for source and destination for Sobel filter */
	ScrSetLogPage(Screen1); // the following ScrByteAddr refers to the current logical page
	ImageAssign(&a, ScrByteAddr(100,100), DispGetColumns/2, DispGetRows/2, DispGetPitch);		//image var "a" is assigned to Screen1
	ScrSetLogPage(Screen2);
	ImageAssign(&b, ScrByteAddr(100,100), DispGetColumns/2, DispGetRows/2, DispGetPitch);
	ScrSetLogPage(Screen3);
	ImageAssign(&c, ScrByteAddr(100,100), DispGetColumns/2, DispGetRows/2, DispGetPitch);
	
	/* Assiging image variables for the overlay */
	ImageAssign(&d, Overlay1, DispGetColumns, DispGetRows, DispGetPitch);
	ImageAssign(&e, Overlay2, DispGetColumns, DispGetRows, DispGetPitch);
	ImageAssign(&f, Overlay3, DispGetColumns, DispGetRows, DispGetPitch);
	
	/* setting overlay colors*/
	set_overlay_bit(7,255,0,0);
	set_overlay_bit(6,0,255,0);
	set_overlay_bit(5,0,0,255);
	/* setting the camera to the correct start mode */
	set_ovlmask(255); // turning the overlay on, see section 5.1.
	tpict(); // tpict() is required in order to set the camera to still mode,
	//vmode(vmstill) is not sufficient, since it waits until the next image has been completed until the camera actually turns into still mode!
	//This should be avoided when using capture_requests!
	
	
	/* Main Program Loop with initial capture_requests */
	tr1 = capture_request(getvar(EXPCNT), getvar(GAIN), (int *)Screen1, 0); // first capture request, C1
	tr2 = capture_request(getvar(EXPCNT), getvar(GAIN), (int *)Screen2, 0); // second capture request, C2
	
	do
	{
		if (kbhit()) key = rs232rcv();
		setvar(MSEC,0);
		t0 = getvar(MSEC);
		
		while(tr1 > getvar(IMGREADY)); // wait that first image has been written to memory
		t2 = getvar(MSEC);
		tr3 = capture_request(getvar(EXPCNT), getvar(GAIN), (int *)Screen3, 0);
		sobel(&a,&a); // process Screen1
		markerd(&d,128); // draw overlay1
		ScrSetDispPage(Screen1);
		OvlSetPhysPage(Overlay1);
		display_update(2);
		t3 = getvar(MSEC);
		
		while(tr2 > getvar(IMGREADY)); // wait that the second image has been written to memory
		t5 = getvar(MSEC);
		tr1 = capture_request(getvar(EXPCNT), getvar(GAIN), (int *)Screen1, 0);
		sobel(&b,&b); // process Screen2
		markerd(&e,64); // draw overlay2
		ScrSetDispPage(Screen2);
		OvlSetPhysPage(Overlay2);
		display_update(2);
		t6 = getvar(MSEC);
		
		while(tr3 > getvar(IMGREADY)); // wait that second image has been written to memory
		t8 = getvar(MSEC);
		tr2 = capture_request(getvar(EXPCNT), getvar(GAIN), (int *)Screen2, 0);
		sobel(&c,&c); // process Screen3
		markerd(&f,32); // draw overlay3
		ScrSetDispPage(Screen3);
		OvlSetPhysPage(Overlay3);
		display_update(2);
		t9 = getvar(MSEC);
		print("t0= %d, t2 = %d, t3= %d. t5 = %d, t6 = %d, t8 = %d, t9 = %d\n",t0, t2, t3, t5, t6, t8, t9);
	}
	while(key != 'q');

	/* Program Closure */
	// switching all page settings back to default mode
	OvlSetPhysPage(Overlay1);
	OvlSetLogPage(Screen1);
	ScrSetDispPage(Screen1);
	display_update(2); // if not inserted here, the camera does not switch back to the
	//default display page!
	ScrSetCaptPage(Screen1);
	ScrSetLogPage(Screen1);
	
	// Release momory allocation in reverse order!
	DRAMPgFree(Overlay3_not_aligned);
	DRAMPgFree(Overlay2_not_aligned);
	DRAMPgFree(Screen3_not_aligned);
	DRAMPgFree(Screen2_not_aligned);
	
	vmode(vmLive); //switching camera back to live mode
}
/*******************************************************************/

