/************************************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program Description:
--------------------

Example of using timestamps.


Revision History:
-----------------

   Date           Version    Author  Changes
   Apr 19, 2007   1.00       LB      Initial Version
   Jul 10, 2007   1.01       LB      Added time and date display (possible from VCRT 5.25.10)

*/
/************************************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>
#include <string.h>
#include <flib.h>

/************************************************************************************/

void main(void)
  {
    int i, TrackingNumber;
	int MilliSec;
	int tz = 1, sec, min, hour, day, month, year;
	long long LastTS, SecTS;
	imgts * ts_table = (imgts *)getvar(TIMESTAMP);

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
    /*******************************************************************************/

    /* Go to live mode */
    vmode(vmLive);

	/* Press a key to take a picture */
	print("Press a key to take a picture...\n\n");
	rs232rcv();

	vmode(vmStill);
	tpwait();


	/*********************/
    /* Taking picture... */
	/*********************/
	TrackingNumber = capture_request(getvar(EXPCNT), getvar(GAIN), (int*)ScrGetCaptPage, 0);
    while(TrackingNumber > getvar(IMGREADY));


	/*****************************/
	/* Display all 20 timestamps */
	/*****************************/
	print("exposure timestamps : 0x%08lx\n",getvar(TIMESTAMP));
	print("Displaying all 20 timestamps\n");

	ts_table = (imgts *)getvar(TIMESTAMP);

	for(i=1;i<=IMGTS_SIZE && ts_table;i++,ts_table++)
		{
			print("%02d (0x%x) ",i,ts_table);
			print("nr= %d ",ts_table->imageno);
			print("ts= %lu\n",ts_table->exptimestamp);
		}


	/*****************************************************************/
    /* Find timestamp corresponding to the picture in circular table */
	/*****************************************************************/
	print("\nTracking number of the picture : %d\n", TrackingNumber);

	ts_table = (imgts *)getvar(TIMESTAMP);

    for(i=1;i<=IMGTS_SIZE && ts_table;i++,ts_table++)
      {
		if(ts_table->imageno == TrackingNumber)
		  {
		    print("Picture number = %d\n", ts_table->imageno);
		    print("Timestamp = %lu\n", ts_table->exptimestamp);

			LastTS = ts_table->exptimestamp;
		  }
      }


	/*****************************************/
	/* Calculate corresponding time and date */
	/*****************************************/
	MilliSec = LastTS % 1000ULL;     /* Get milliseconds */
	SecTS = (long)(LastTS / 1000);   /* Get seconds*/

	c_timedate(SecTS, tz, &sec, &min, &hour, &day, &month, &year);

	print("time: %02d:%02d:%02d:%03d\n",hour,min,sec,MilliSec);
    print("date: %02d/%02d/%02d\n",month,day,year);

  }
