/*******************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Klaus Schneider,  VC


Program Description:
--------------------
Reworked version of the PING1.C demo program, supplied by VC.
Modified by Plamen Kozhuharov, atto-Systems, to draw overlay in
ping-pong mode and to check overlay blinking.


*/
/*******************************************************************/

#include <string.h>
#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

void set_ovlmask(int mask);
/*******************************************************************/

void fill_box (image *img, int x, int y, int dx, int dy, int  clr );

/*******************************************************************/

void main(void)
{
    image source;
    int OldScreen, NewScreen, NewScreenAlign; /*, response =0;*/
    int delay = 1;

    unsigned long time;

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
    /*******************************************************************************/


    /*............. enable overlay display */
    vmode(4);
    set_lut_comp(255,0,0);      /* red color    */
    set_ovlmask(1);

    OldScreen = getvar(OVLY_START);     /* get addr of old screen */
    NewScreen=(int)DRAMDisplayMalloc(); /* allocate new screen page */

    /* Image display addresses have to be in alignments of 1024 */
    NewScreenAlign=(NewScreen&0xFFFFFC00) + 1024;

    /* switch to new screen page */
    setvar(OVLY_START,NewScreenAlign);

    source.dx    = DispGetColumns;
    source.dy    = DispGetRows;
    source.pitch = ScrGetPitch;

    for(;;)
    {
      if(getvar(OVLY_START) == OldScreen)
      {
        source.st = NewScreenAlign;
        set(&source, 0);
        fill_box(&source, 30, 10, source.dx-90, 30, 1 );
        fill_box(&source, 30, 50, source.dx-90, 30, 1 );

        setvar(OVLY_START, NewScreenAlign);
      }

      else      /* OVLY_START == NewScreenAlign */
      {
        source.st = OldScreen;
        set(&source, 0);
        fill_box(&source, 30, 10, source.dx-90, 30, 1 );
        fill_box(&source, 30, 50, source.dx-90, 30, 1 );
 
        setvar(OVLY_START, OldScreen);
     }

      /* 
	  Display Update immediately

      #define UPDATE_DISP (0)
      #define UPDATE_OVLY (1)
      #define UPDATE_DISP_OVLY (2)
	  */

      display_update(UPDATE_OVLY);
      wait_vsync();
      

      /* Break the loop on any pressed key */
      if(kbhit()) {rs232rcv(); break;}


      // Insert changing delay time in the loop to ensure that we do not have accidental display synchronization.
      time_delay(delay);
      if(++delay > 30) delay = 1;

    } 


    /* switch back to old overlayn page */
    setvar(OVLY_START, OldScreen);
    source.st = getvar(OVLY_START);
    set(&source, 0);         /* clear ovl */

    /* release overlay allocation */
    DRAMPgFree(NewScreen);	 
}

/*******************************************************************/

void fill_box (image *img, int x, int y, int dx, int dy, int  clr )
{
    int addr;
    int i;

    if(x >= img->dx) return;
    if(x + dx > img->dx) dx = img->dx - x;
    addr = img->st + y*img->pitch + x;

    for(i=y; i<y+dy; i++)
    {
       if(i >= img->dy) break;
       if(i >= 0) memset((char *)addr,clr,dx);
       addr += img->pitch;
    }
    return;
}

/*******************************************************************/
