/*******************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Klaus Schneider,  VC


Program Description:
--------------------

The program shows a cross in the overlay. You can move it with 
the cursor keys. 
It shows also how you can use overlay text.

09/07/07 LB : update to VCLIB 3.04

*/
/*******************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>
#include <flib.h>

/*******************************************************************/

void main(void)

  {
  char  textstr[40];
  unsigned char *brightness;
  int   CenterX, CenterY, key, step=10, live;
  image DrawOverlay, TextOverlay;
  
    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1234567890") or init_licence("LT1234567890") */
    /* Extension Lib : init_licence("E1234567890") or init_licence("LE1234567890") */
    /* Color Lib :     init_licence("C1234567890") or init_licence("LC1234567890") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
    /*******************************************************************************/ 

  /* Working Page */
  ScrSetLogPage((int)ScrGetPhysPage);
  OvlSetLogPage((int)OvlGetPhysPage);

  /* Clear Overlay */
  OvlClearAll;

  /* Display Live and Overlay */
  live=1;
  vmode(vmOvlLive);
  set_ovlmask(255);

  /* Define image variable DrawOverlay */
  ImageAssign(&DrawOverlay, OvlBitAddr(0,0), OvlGetColumns, OvlGetRows, OvlGetPitch);  

  /* text position */
  ImageAssign(&TextOverlay, OvlBitAddr(20,20), 160, 8, OvlGetPitch);  

  /* Draw a cross in the center */
  CenterX = OvlGetColumns/2;
  CenterY = OvlGetRows/2;
  linez(&DrawOverlay,CenterX,0,CenterX,OvlGetRows);
  linez(&DrawOverlay,0,CenterY,OvlGetColumns,CenterY);

  print("\n");
  print("Press ENTER  to toggle between live and DRAM image\n");
  print("Press CURSOR to move position\n");
  print("Press F1     for step wide (toggle)\n");
  print("Press ESC    to quite the program\n");
  print("\n");

  do 
    {
    /* show values */
    brightness=(unsigned char *)(ScrByteAddr(CenterX,CenterY));

    print("X=%d Y=%d B=%d\n", CenterX, CenterY, (int)(*brightness)); 

    sprint(textstr,"X=%d Y=%d B=%d      ", CenterX, CenterY, (int)(*brightness)); 
    chprint1(textstr,&TextOverlay,1,1,128);

    /* read Keypad */
    key = ReadKeyboard(1);

    /* delete old line */
    linez(&DrawOverlay,CenterX,0,CenterX,OvlGetRows);
    linez(&DrawOverlay,0,CenterY,OvlGetColumns,CenterY);

    /* position of the new line */
    switch (key)
      {
      case KEY_ENTER   : if(live) {live=0; vmode(vmOvlStill);} else {live=1; vmode(vmOvlLive);} break;
      case KEY_F1      : if(step==1) step=10; else step=1; break;
      case KEY_UP      : CenterY-=step; if (CenterY<            0) CenterY=0;             break;
      case KEY_DOWN    : CenterY+=step; if (CenterY>OvlGetRows   ) CenterY=OvlGetRows;    break;
      case KEY_RIGHT   : CenterX+=step; if (CenterX>OvlGetColumns) CenterX=OvlGetColumns; break;
      case KEY_LEFT    : CenterX-=step; if (CenterX<            0) CenterX=0;             break;
      }

    /* draw new line */
    linez(&DrawOverlay,CenterX,0,CenterX,OvlGetRows);
    linez(&DrawOverlay,0,CenterY,OvlGetColumns,CenterY);
    }
  while (key!=KEY_ESC);    

  /* Clear Overlay */
  OvlClearAll;

  /* Display Live and Overlay */
  vmode(vmLive);
  }

/***************************************************************************************************************************/
