/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2000-2004 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: dir.c
***                                                            
*** Comments: 
***		dir shell command
***		print directory of files
***
*** Revision History:
***    Date          Version    Author  Changes
***    Oct 10, 2002  5.00       ME      Initial Version
***	   Jul 19, 2004	 5.17		ME		cosmetic update
***    Dec 17, 2004  5.17		MS		free space on device is now printed	   
***    Feb 03, 2005  5.18		MS		extension of structure VC_DIRENT ! 
***
***************************************************************************
*END***********************************************************************/

#include "userdef.h"
#include "register.h"
#include <vcrt.h>
#include "shell.h"
#include <sysvar.h>

extern char dir_path[];


extern dir_xopt(), dir_aopt(), dir_default();


options diropt[]=
{
    {'X'   ,(void (*)()) dir_xopt   },
    {'A'   ,(void (*)()) dir_aopt   },
    {'\0'  ,(void (*)()) dir_default}
};


int option;

dicmd(int argc, char *argv)
{

getopt(&argc,&argv,diropt,ERR04);

if(option == 0)
  {
  pstr("user files\n\n");
  dir(argv);
  }
else
if(option == 1)
  {
  pstr("system files\n\n");
  dir("fd:/sys/");
  }
else
if(option == 2)
  {
  pstr("\n");
  dir("fd:/all/");
  }

}


dir_xopt()
{
option = 1;
}

dir_aopt()
{
option = 2;
}

dir_default()
{
option = 0;
}


typedef struct vc_readdir_struct				/* ME										*/
{
   unsigned				MAXNUM;		   			/* the maximum number of directory items 	*/
   char *				FPATH;		   			/* the file path							*/
   struct vc_dirent  *	DIRENT;		   			/* pointer to structure for the directory	*/
} VC_READDIR_STRUCT, *  VC_READDIR_STRUCT_PTR;


#define FS_MAXPATH  80							/* MAXPATH incl. final '\0' 				*/


typedef struct vc_dirent						/* ME										*/
{
   char						D_NAME[FS_MAXPATH];	/* the file name string including full path	*/
   unsigned					LENGTH;				/* file size								*/
   long						CR_DATE;			/* date of creation							*/
   long						WR_DATE;			/* date last modified						*/
   int 						ATTR;               /* file attribute */
   int 						LINK_COUNT;
   int 						DUMMY1;   
   int 						DUMMY2; 
} VC_DIRENT_STRUCT, * VC_DIRENT_STRUCT_PTR;



dir(char *argv)
{
FILE *fp;
VC_READDIR_STRUCT  rx;
VC_DIRENT_STRUCT   directory[100];
int i=0,rsize=0;
char name[20], ext[5];
char path[80];
char fname[20], fext[5];
int wc_name;
int wc_ext;

rx.MAXNUM = 100;
rx.DIRENT = directory;

strcpy(path,argv);
wc_name = name_wildcard(path, fname);
wc_ext  = ext_wildcard(path, fext);

get_dir(copyarg(path,argv));

print("path: %s\n",path);

fp = io_fopen(path,"c");

if(fp)
  {
  io_ioctl(fp, IO_READDIR, (unsigned int *)&rx);
  io_ioctl(fp, IO_REMAIN, &rsize);
  io_fclose(fp);


  while(*directory[i].D_NAME)
    {
    _FsFnSplit(directory[i].D_NAME, (char *)0, (char *)0, name, ext);

    if(wc_name || (!wc_name && !strcmp(name,fname))) 
	  if(wc_ext || (!wc_ext && !strcmp(ext,fext))) 
        {
        print("%-10s %-3s %08X ",name, ext, directory[i].LENGTH);
        print_timedate2(directory[i].WR_DATE);
	    print("\n");
	    }
	i++;
	}

  print("\n%08X free\n",rsize);

  }	
else
  print("open error for directory:\n");


}




void print_timedate2(long x)
{
int xsec, xminute, xhour, xday, xmonth, xyear;

if(x==0L) return;

c_timedate(x,getvar(TIMEZONE)+getvar(DAYLIGHT), &xsec,&xminute,
             &xhour,&xday,&xmonth,&xyear);
  
print("%02d/%02d/%02d ",(xmonth),(xday),(xyear-2000));
print("%02d:%02d:%02d ",(xhour),(xminute),(xsec));
}



char *get_dir(char *path)			/* get directory and apply default path if necessary	*/
{

char drv[20];
char pth[80];
char name[20];
char ext[5];

_FsFnSplit(path, drv, pth, name, ext);

if((!strcmp(drv,"")) && (!strcmp(pth,"")))
  _FsFnSplit(dir_path, drv, pth, (char *)0, (char *)0);		/* default path	*/

if(!strcmp(name,"*") || !strcmp(ext,"*"))
  _FsFnMerge(path,drv, pth, (char *)0, (char *)0);
else
  _FsFnMerge(path,drv, pth, name, ext);

return(path);
}

int name_wildcard(char *path, char *name)
{
char ext[5];

_FsFnSplit(path, (char *)0, (char *)0, name, ext);

if(!strcmp(name,"*") || (!strcmp(name,"") && !strcmp(ext,"")))
  return 1;
else
  return 0;

}

int ext_wildcard(char *path, char *ext)
{
char name[20];

_FsFnSplit(path, (char *)0, (char *)0, name, ext);

if(!strcmp(ext,"*") || (!strcmp(name,"") && !strcmp(ext,"")))
  return 1;
else
  return 0;

}


/* EOF */



