#include <register.h>
#include <vcrt.h>
#include <vclib.h>
#include <flib.h>
#include <macros.h>
#include <sysvar.h>
void set_res(uint_32 nthresh);
set_image_format(int hpos, int vpos, int hsize, int vsize);

//#define     VC_ASTRT0   0x01C40108          /* Video Capture Channel A Field 1 Start Register   */
#define     VC_ASTOP0   0x01C4010C          /* Video Capture Channel A Field 1 Stop Register    */
#define     VC_ATHRLD0  0x01C4011C          /* Video Capture Channel A Threshold Register       */

#define SEN_ADR         (0x5D)

//
// set resolution on vc4012e sensor demo
//
// Author: Michael Steinel Date: 23.04.2010
// 

int main(int argc, char * argv)
{
int i;

printf("resolution test routines for SBC4012\n");

tpict(); // stop running live mode

//exec2("fd://img2b.exe"); // start monitor

#if 0
//while(1)
{
  for(i=0;i<1200;i++)
  {
	 set_image_format(i,i,640,480); // move vga window
	 tpict();
  }
  for(i=1200;i;i--)
  {
	 set_image_format(i,i,640,480); // move vga window
	 tpict();
  }
}
#else
set_image_format(976,732,640,480); // centered vga window
#endif
return 0;
}

set_image_format(int hpos, int vpos, int hsize, int vsize)
{
char i2c_sendbuf[] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
I32 nthresh = hsize/(8*2); // MS 27.04.2010 fixed computing nthresh - 64 bit access DMA -

hsize = nthresh * (hsize / nthresh);				/* imgsize must be a multiple of nthresh	*/

print("size (h, v):  [%d, %d]\n", hsize, vsize);
setvar(HWIDTH, hsize);
setvar(VWIDTH, vsize);
setvar(VPITCH, hsize);

hsize -= 1;
vsize -= 1;

hpos += 0x10;
vpos += 0x36;

//R01    Row Start (54)
i2c_sendbuf[0] = 0x01;
i2c_sendbuf[1] = vpos >> 8;
i2c_sendbuf[2] = vpos & 0xff;
i2c_edma_write(3, SEN_ADR, i2c_sendbuf);
time_delay(2);

//R02    Column Start (16)
i2c_sendbuf[0] = 0x02;
i2c_sendbuf[1] = hpos >> 8;
i2c_sendbuf[2] = hpos & 0xff;
i2c_edma_write(3, SEN_ADR, i2c_sendbuf);
time_delay(2);

//R03    Row Size (1943)
i2c_sendbuf[0] = 0x03;
i2c_sendbuf[1] = vsize >> 8;
i2c_sendbuf[2] = vsize & 0xff;
i2c_edma_write(3, SEN_ADR, i2c_sendbuf);
time_delay(2);

//R04    Column Size (2591)
i2c_sendbuf[0] = 0x04;
i2c_sendbuf[1] = hsize >> 8;
i2c_sendbuf[2] = hsize & 0xff;
i2c_edma_write(3, SEN_ADR, i2c_sendbuf);
time_delay(2);

set_res(nthresh); //reprogram DMA
}


//MS 12.06.2008 set resolution of line camera VC4012 
void set_res(uint_32 nthresh)
{
  EDMA *edma16;                   /* video channel                    */
  EDMA *edma25;                   /* reload parameters                */
  uint_32 imgsize;
  uint_32 nframes;
  static uint_32 lines = 0xFFFFFFFF;

  if(getvar(VWIDTH) > 1944)
	setvar(VWIDTH,1944);

  if(getvar(VWIDTH) == 0)
	setvar(VWIDTH,1);

  if (lines == getvar(VWIDTH))
	return; // no change of lines
  else
	lines = getvar(VWIDTH);

  imgsize = getvar(HWIDTH)*getvar(VWIDTH);
  nframes = imgsize/nthresh/8;

  /* capturing video in raw data mode                                                             */

  /* set stop of image                                                                            */
//  *((volatile int *)(VC_ASTRT0))      = 0x000010;                    /* vblanking = 16 lines     */
  *((volatile int *)(VC_ASTOP0))      = ((imgsize<<4) & 0x0FFF0000)   /* image size split in 2    */
	+ (imgsize & 0x0FFF);       /* 12 bit parts             */


  /* set capture threshold                                                                        */
  *((volatile int *)(VC_ATHRLD0))     = nthresh * 0x010001;           /* duplicate NTHRESH        */

  /* configure EDMA channel                                                                       */
  edma16 = &((EDMA *)EDMA_BASE)[16];
  edma25 = &((EDMA *)EDMA_BASE)[25];


  /* channel 16 ****************************************************************/

  edma16->FCEC         = ((nframes-1)<<16)+nthresh*2;
  edma16->FIXEIX       = (nthresh*8)<<16;

  /* reload ********************************************************************/

  edma25->FCEC         = ((nframes-1)<<16)+nthresh*2;
  edma25->FIXEIX       = (nthresh*8)<<16;
}

/* EOF */
