#include <register.h>
#include <vcrt.h>
#include <vclib.h>
#include <flib.h>
#include <macros.h>
#include <sysvar.h>

void set_res(uint_32 nthresh);
void set_image_format_bin(int hpos, int vpos, int hsize, int vsize, int rowbin, int colbin);

//#define     VC_ASTRT0   0x01C40108          /* Video Capture Channel A Field 1 Start Register   */
#define     VC_ASTOP0   0x01C4010C          /* Video Capture Channel A Field 1 Stop Register    */
#define     VC_ATHRLD0  0x01C4011C          /* Video Capture Channel A Threshold Register       */

#define SEN_ADR         (0x5D)

//
// set resolution on vc4012e sensor demo
//
// Author: Michael Steinel Date: 23.04.2010
// 
// LB 25.11.10
// Modified set_image_format() for binning

int main(int argc, char * argv)
{
	
	printf("resolution test routines for SBC4012\n");
	
	tpict(); // stop running live mode
	
	set_image_format_bin(0, 0, 2592, 1944, 1, 4);
	
	return 0;
}

void set_image_format_bin(int hpos, int vpos, int hsize, int vsize, int rowbin, int colbin)
{
	char i2c_sendbuf[] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
	I32 nthresh = hsize/(8*2); // MS 27.04.2010 fixed computing nthresh - 64 bit access DMA -
	I32 R22_val = 0, R23_val = 0;
	
	
	hsize = nthresh * (hsize / nthresh);				/* imgsize must be a multiple of nthresh	*/
	
	if((rowbin != 1) && (rowbin != 2) && (rowbin != 4))
		{rowbin = 1; print("Row binning must be 1, 2 or 4!! Set to 1\n");}
		
	if((colbin != 1) && (colbin != 2) && (colbin != 4))
		{colbin = 1; print("Column binning must be 1, 2 or 4!! Set to 1\n");}
		
	if (rowbin == 1) R22_val = 0x0;		
	if (rowbin == 2) R22_val = 0x11;	// Binning 2X : Row_Skip = Row_Bin = 1
	if (rowbin == 4) R22_val = 0x33;	// Binning 4X : Row_Skip = Row_Bin = 3

	if (colbin == 1) R23_val = 0x0;		
	if (colbin == 2) R23_val = 0x11;	// Binning 2X : Col_Skip = Col_Bin = 1
	if (colbin == 4) R23_val = 0x33;	// Binning 4X : Col_Skip = Col_Bin = 3
	
	print("size (h, v):  [%d, %d]\n", hsize, vsize);
	setvar(HWIDTH, hsize / colbin);
	setvar(VWIDTH, vsize / rowbin);		// vsize/rowbin because of row binning
	setvar(VPITCH, hsize / colbin);
	
	hsize -= 1;
	vsize -= 1;
	
	hpos += 0x10;
	vpos += 0x38;	// Changed vpos to a multiple of 2*(Row_Bin + 1), tpict hangs otherwise
	
	//R01    Row Start (54)
	i2c_sendbuf[0] = 0x01;
	i2c_sendbuf[1] = vpos >> 8;
	i2c_sendbuf[2] = vpos & 0xff;
	i2c_edma_write(3, SEN_ADR, i2c_sendbuf);
	//R02    Column Start (16)
	i2c_sendbuf[0] = 0x02;
	i2c_sendbuf[1] = hpos >> 8;
	i2c_sendbuf[2] = hpos & 0xff;
	i2c_edma_write(3, SEN_ADR, i2c_sendbuf);
	
	//R03    Row Size (1943)
	i2c_sendbuf[0] = 0x03;
	i2c_sendbuf[1] = vsize >> 8;
	i2c_sendbuf[2] = vsize & 0xff;
	i2c_edma_write(3, SEN_ADR, i2c_sendbuf);
	//R04    Column Size (2591)
	i2c_sendbuf[0] = 0x04;
	i2c_sendbuf[1] = hsize >> 8;
	i2c_sendbuf[2] = hsize & 0xff;
	i2c_edma_write(3, SEN_ADR, i2c_sendbuf);
	
	
	//R22    Row Address Mode
	i2c_sendbuf[0] = 0x22;
	i2c_sendbuf[1] = 0;
	i2c_sendbuf[2] = R22_val;
	i2c_edma_write(3, SEN_ADR, i2c_sendbuf);
	
	//R23    Column Address Mode
	i2c_sendbuf[0] = 0x23;
	i2c_sendbuf[1] = 0;
	i2c_sendbuf[2] = R23_val;
	i2c_edma_write(5, SEN_ADR, i2c_sendbuf);
	
	set_res(nthresh); //reprogram DMA
}


//MS 12.06.2008 set resolution of line camera VC4012 
void set_res(uint_32 nthresh)
{
  EDMA *edma16;                   /* video channel                    */
  EDMA *edma25;                   /* reload parameters                */
  uint_32 imgsize;
  uint_32 nframes;
  static uint_32 lines = 0xFFFFFFFF;

  if(getvar(VWIDTH) > 1944)
	setvar(VWIDTH,1944);

  if(getvar(VWIDTH) == 0)
	setvar(VWIDTH,1);

  if (lines == getvar(VWIDTH))
	return; // no change of lines
  else
	lines = getvar(VWIDTH);

  imgsize = getvar(HWIDTH)*getvar(VWIDTH);
  nframes = imgsize/nthresh/8;

  /* capturing video in raw data mode                                                             */

  /* set stop of image                                                                            */
//  *((volatile int *)(VC_ASTRT0))      = 0x000010;                    /* vblanking = 16 lines     */
  *((volatile int *)(VC_ASTOP0))      = ((imgsize<<4) & 0x0FFF0000)   /* image size split in 2    */
	+ (imgsize & 0x0FFF);       /* 12 bit parts             */


  /* set capture threshold                                                                        */
  *((volatile int *)(VC_ATHRLD0))     = nthresh * 0x010001;           /* duplicate NTHRESH        */

  /* configure EDMA channel                                                                       */
  edma16 = &((EDMA *)EDMA_BASE)[16];
  edma25 = &((EDMA *)EDMA_BASE)[25];


  /* channel 16 ****************************************************************/

  edma16->FCEC         = ((nframes-1)<<16)+nthresh*2;
  edma16->FIXEIX       = (nthresh*8)<<16;

  /* reload ********************************************************************/

  edma25->FCEC         = ((nframes-1)<<16)+nthresh*2;
  edma25->FIXEIX       = (nthresh*8)<<16;
}

/* EOF */
